@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,18 +36,21 @@
 #include <cstring>
 #include <vector>
 #include <map>
+#include <unordered_map>
 #include <list>
 #include <algorithm>
+#include <memory>
+
+#ifdef IPESTRICT
+#include "ipeosx.h"
+#endif
 
 // --------------------------------------------------------------------
 
-typedef unsigned char uchar;
+#define IPE_PI 3.14159265358979323846
 
-#if defined(WIN32) || defined(__APPLE__)
-typedef unsigned short ushort;
-typedef unsigned int uint;
-typedef unsigned long int ulong;
-#endif
+using uchar = unsigned char;
+using uint = unsigned int;
 
 #ifdef WIN32
 #define IPESEP ('\\')
@@ -61,26 +64,13 @@ extern void ipeAssertionFailed(const char *, int, const char *);
 
 // --------------------------------------------------------------------
 
-extern void ipeDebug(const char *msg, ...);
-
-template<class T>
-class IpeAutoPtr {
-public:
-  IpeAutoPtr(T *ptr) : iPtr(ptr) { /* nothing */ }
-  ~IpeAutoPtr() { delete iPtr; }
-  T *operator->() { return iPtr; }
-  T &operator*() { return *iPtr; }
-  T *ptr() { return iPtr; }
-  T *take() { T *p = iPtr; iPtr = 0; return p; }
-private:
-  T *iPtr;
-};
+extern void ipeDebug(const char *msg, ...) noexcept;
 
 namespace ipe {
 
   //! Ipelib version.
   /*! \ingroup base */
-  const int IPELIB_VERSION = 70110;
+  const int IPELIB_VERSION = 70204;
 
   //! Oldest readable file format version.
   /*! \ingroup base */
@@ -88,6 +78,9 @@ namespace ipe {
   //! Current file format version.
   /*! \ingroup base */
   const int FILE_FORMAT = 70107;
+  const int FILE_FORMAT_NEW = 70204; // group url, pagenumberstyle
+
+  enum class LatexType { Default, Pdftex, Xetex, Luatex };
 
   // --------------------------------------------------------------------
 
@@ -95,50 +88,62 @@ namespace ipe {
 
   class String {
   public:
-    String();
-    String(const char *str);
-    String(const String &rhs);
-    String(const String &rhs, int index, int len);
-    String &operator=(const String &rhs);
-    ~String();
+    String() noexcept;
+    String(const char *str) noexcept;
+    String(const char *str, int len) noexcept;
+    String(const String &rhs) noexcept;
+    String(const String &rhs, int index, int len) noexcept;
+    String &operator=(const String &rhs) noexcept;
+#ifdef WIN32
+    String(const wchar_t *data) noexcept;
+#endif
+    ~String() noexcept;
+    static String withData(char *data, int len = 0) noexcept;
     //! Return character at index i.
-    inline char operator[](int i) const { return iImp->iData[i]; }
+    inline char operator[](int i) const noexcept { return iImp->iData[i]; }
     //! Is the string empty?
-    inline bool empty() const { return (size() == 0); }
-    //! Is the string empty?
-    inline bool isEmpty() const { return (size() == 0); }
+    inline bool empty() const noexcept { return (size() == 0); }
     //! Return read-only pointer to the data.
-    const char *data() const { return iImp->iData; }
+    const char *data() const noexcept { return iImp->iData; }
     //! Return number of bytes in the string.
-    inline int size() const { return iImp->iSize; }
+    inline int size() const noexcept { return iImp->iSize; }
     //! Operator syntax for append.
-    inline void operator+=(const String &rhs) { append(rhs); }
+    inline void operator+=(const String &rhs) noexcept { append(rhs); }
     //! Operator syntax for append.
-    void operator+=(char ch) { append(ch); }
+    inline void operator+=(const char *rhs) noexcept { append(rhs); }
+    //! Operator syntax for append.
+    void operator+=(char ch) noexcept { append(ch); }
     //! Create substring.
-    inline String substr(int i, int len = -1) const {
+    inline String substr(int i, int len = -1) const noexcept {
       return String(*this, i, len); }
     //! Create substring at the left.
-    inline String left(int i) const {
+    inline String left(int i) const noexcept {
       return String(*this, 0, i); }
-    String right(int i) const;
+    String right(int i) const noexcept;
     //! Operator !=.
-    inline bool operator!=(const String &rhs) const {
+    inline bool operator!=(const String &rhs) const noexcept {
+      return !(*this == rhs); }
+    //! Operator !=.
+    inline bool operator!=(const char *rhs) const noexcept {
       return !(*this == rhs); }
 
-    int find(char ch) const;
-    int rfind(char ch) const;
-    int find(const char *rhs) const;
-    void erase();
-    void append(const String &rhs);
-    void append(char ch);
-    bool operator==(const String &rhs) const;
-    bool operator<(const String &rhs) const;
-    String operator+(const String &rhs) const;
-    int unicode(int &index) const;
-    const char *z() const;
+    int find(char ch) const noexcept;
+    int rfind(char ch) const noexcept;
+    int find(const char *rhs) const noexcept;
+    void erase() noexcept;
+    void append(const String &rhs) noexcept;
+    void append(const char *rhs) noexcept;
+    void append(char ch) noexcept;
+    bool hasPrefix(const char *rhs) const noexcept;
+    bool operator==(const String &rhs) const noexcept;
+    bool operator==(const char *rhs) const noexcept;
+    bool operator<(const String &rhs) const noexcept;
+    String operator+(const String &rhs) const noexcept;
+    int unicode(int &index) const noexcept;
+    String getLine(int &index) const noexcept;
+    const char *z() const noexcept;
   private:
-    void detach(int n);
+    void detach(int n) noexcept;
   private:
     struct Imp {
       int iRefCount;
@@ -146,6 +151,8 @@ namespace ipe {
       int iCapacity;
       char *iData;
     };
+    static Imp *theEmptyString;
+    static Imp *emptyString() noexcept;
     Imp *iImp;
   };
 
@@ -220,24 +227,26 @@ namespace ipe {
 
   class Buffer {
   public:
-    Buffer();
-    ~Buffer();
-    Buffer(const Buffer &rhs);
-    Buffer &operator=(const Buffer &rhs);
+    Buffer() noexcept = default;
+    ~Buffer() noexcept;
+    Buffer(const Buffer &rhs) noexcept;
+    Buffer(Buffer &&rhs) noexcept;
+    Buffer &operator=(const Buffer &rhs) noexcept;
+    Buffer &operator=(Buffer &&rhs) noexcept;
 
-    explicit Buffer(int size);
-    explicit Buffer(const char *data, int size);
+    explicit Buffer(int size) noexcept;
+    explicit Buffer(const char *data, int size) noexcept;
     //! Character access.
-    inline char &operator[](int index) { return iImp->iData[index]; }
+    inline char &operator[](int index) noexcept { return iImp->iData[index]; }
     //! Character access (const version).
-    inline const char &operator[](int index) const
-    { return iImp->iData[index]; }
+    inline const char &operator[](int index) const noexcept {
+      return iImp->iData[index]; }
     //! Return size of buffer;
-    inline int size() const { return iImp->iSize; }
+    inline int size() const noexcept { return iImp ? iImp->iSize : 0; }
     //! Return pointer to buffer data.
-    inline char *data() { return iImp->iData; }
+    inline char *data() noexcept { return iImp->iData; }
     //! Return pointer to buffer data (const version).
-    inline const char *data() const { return iImp->iData; }
+    inline const char *data() const noexcept { return iImp->iData; }
 
   private:
     struct Imp {
@@ -245,7 +254,7 @@ namespace ipe {
       char *iData;
       int iSize;
     };
-    Imp *iImp;
+    Imp *iImp = { nullptr };
   };
 
   // --------------------------------------------------------------------
@@ -331,6 +340,7 @@ namespace ipe {
   public:
     BufferSource(const Buffer &buffer);
     virtual int getChar();
+    void setPosition(int pos);
   private:
     const Buffer &iBuffer;
     int iPos;
@@ -340,14 +350,13 @@ namespace ipe {
 
   class Platform {
   public:
-    typedef void (*DebugHandler)(const char *);
+    using DebugHandler = void (*)(const char *);
 
 #ifdef IPEBUNDLE
     static String ipeDir(const char *suffix, const char *fname = 0);
 #endif
 #ifdef WIN32
     static String winConv(const String &utf8, const String &encoding);
-    static String wideToUtf8(const wchar_t *wbuf);
     static void utf8ToWide(const char *utf8, std::vector<wchar_t> &wbuf);
     static FILE *fopen(const char *fname, const char *mode);
     static int mkdir(const char *dname);
@@ -361,11 +370,11 @@ namespace ipe {
     static void setDebug(bool debug);
     static String currentDirectory();
     static String latexDirectory();
-    static String fontmapFile();
+    static String latexPath();
     static bool fileExists(String fname);
     static bool listDirectory(String path, std::vector<String> &files);
     static String readFile(String fname);
-    static int runPdfLatex(String dir);
+    static int runLatex(String dir, LatexType engine) noexcept;
     static double toDouble(String s);
   };
 
@@ -393,7 +402,6 @@ namespace ipe {
     return f;
   }
 
-
 } // namespace
 
 // --------------------------------------------------------------------
