@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -58,6 +58,7 @@ namespace ipe {
     virtual Object *clone() const;
 
     virtual Group *asGroup();
+    virtual const Group *asGroup() const;
 
     virtual Type type() const;
 
@@ -81,6 +82,9 @@ namespace ipe {
     inline const Shape &clip() const { return iClip; }
     void setClip(const Shape &clip);
 
+    inline String url() const { return iUrl; }
+    void setUrl(String url);
+
     //! Return number of component objects.
     inline int count() const { return iImp->iObjects.size(); }
     //! Return object at index \a i.
@@ -110,6 +114,7 @@ namespace ipe {
 
     Imp *iImp;
     Shape iClip;
+    String iUrl;
   };
 
 } // namespace
