@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 #include "ipepage.h"
 #include "ipetext.h"
 #include "ipepdfparser.h"
-#include "ipefontpool.h"
+#include "iperesources.h"
 
 // --------------------------------------------------------------------
 
@@ -46,19 +46,19 @@ namespace ipe {
 
   class Latex {
   public:
-    Latex(const Cascade *sheet);
+    Latex(const Cascade *sheet, LatexType latexType);
     ~Latex();
 
     int scanObject(const Object *obj);
     int scanPage(Page *page);
+    void addPageNumber(int pno, int vno, int npages, int nviews);
     int createLatexSource(Stream &stream, String preamble);
     bool readPdf(DataSource &source);
     bool updateTextObjects();
-    FontPool *takeFontPool();
+    PdfResources *takeResources();
 
   private:
-    bool getXForm(const PdfObj *xform);
-    bool getEmbeddedFont(int fno, int objno);
+    bool getXForm(const PdfObj *xform, const PdfDict *ipeInfo);
     void warn(String msg);
 
   private:
@@ -71,6 +71,7 @@ namespace ipe {
     typedef std::list<Text::XForm *> XFormList;
 
     const Cascade *iCascade;
+    bool iXetex;
 
     PdfFile iPdf;
 
@@ -80,11 +81,8 @@ namespace ipe {
     //! List of XForm objects read from PDF file.  Objects owned!
     XFormList iXForms;
 
-    //! The embedded fonts. Owned!
-    FontPool *iFontPool;
-
-    //! Maps /F<n> to object number;
-    std::map<int, int> iFontObjects;
+    //! The resources from the generated PDF file.
+    PdfResources *iResources;
 
     friend class ipe::TextCollectingVisitor;
   };
