@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,12 +36,13 @@
 #include "ipepage.h"
 #include "ipedoc.h"
 #include "ipeimage.h"
-#include "ipefontpool.h"
 
 // --------------------------------------------------------------------
 
 namespace ipe {
 
+  class PdfResources;
+
   class PdfPainter : public Painter {
   public:
     PdfPainter(const Cascade *style, Stream &stream);
@@ -82,7 +83,7 @@ namespace ipe {
   class PdfWriter {
   public:
     PdfWriter(TellStream &stream, const Document *doc,
-	      const FontPool *pool,
+	      const PdfResources *resources,
 	      bool markedView, int fromPage, int toPage, int compression);
     ~PdfWriter();
 
@@ -100,11 +101,12 @@ namespace ipe {
     void paintView(Stream &stream, int pno, int view);
     void embedBitmaps(const BitmapFinder &bm);
     void createResources(const BitmapFinder &bm);
-    void embedFonts(const FontPool *pool);
+    void embedResources(const PdfResources *resources);
 
   private:
     TellStream &iStream;
     const Document *iDoc;
+    const PdfResources *iResources;
     //! Show only last view of each page?
     bool iMarkedView;
     //! Obj id of XML stream.
@@ -115,8 +117,6 @@ namespace ipe {
     int iBookmarks;
     //! Compression level (0..9).
     int iCompressLevel;
-    //! Obj id of font for page numbers.
-    int iPageNumberFont;
     //! Obj id of GState with opacity definitions.
     int iExtGState;
     //! Obj id of dictionary with pattern definitions.
