@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -76,10 +76,16 @@ namespace ipe {
       bool iDefined;
       //! Position on the page
       Vector iPos;
-      //! Font size.
-      double iFontSize;
+      //! Text size.
+      Attribute iSize;
       //! Text color.
-      Color iColor;
+      Attribute iColor;
+      //! Text alignment, horizontal.
+      THorizontalAlignment iHorizontalAlignment;
+      //! Text alignment, vertical.
+      TVerticalAlignment iVerticalAlignment;
+      //! Template text.
+      String iText;
     };
 
     StyleSheet();
@@ -115,10 +121,6 @@ namespace ipe {
     inline String preamble() const { return iPreamble; }
     //! Set LaTeX preamble.
     inline void setPreamble(const String &str) { iPreamble = str; }
-    //! Return Latex encoding.
-    inline String encoding() const { return iEncoding; }
-    //! Set Latex encoding
-    inline void setEncoding(const String &enc) { iEncoding = enc; }
 
     const Layout *layout() const;
     void setLayout(const Layout &margins);
@@ -132,9 +134,6 @@ namespace ipe {
     const PageNumberStyle *pageNumberStyle();
     void setPageNumberStyle(const PageNumberStyle &pns);
 
-    void addCMap(String s);
-    void allCMaps(std::vector<String> &seq) const;
-
     void setLineCap(TLineCap s);
     void setLineJoin(TLineJoin s);
     void setFillRule(TFillRule s);
@@ -164,7 +163,6 @@ namespace ipe {
     TilingMap iTilings;
     EffectMap iEffects;
     Map iMap;
-    String iEncoding;
     String iPreamble;
     Layout iLayout;
     TextPadding iTextPadding;
@@ -174,8 +172,6 @@ namespace ipe {
     TLineJoin iLineJoin;
     TLineCap iLineCap;
     TFillRule iFillRule;
-
-    std::vector<String> iCMaps;
   };
 
 
@@ -207,7 +203,6 @@ namespace ipe {
     const StyleSheet::TitleStyle *findTitleStyle() const;
     const StyleSheet::PageNumberStyle *findPageNumberStyle() const;
     String findPreamble() const;
-    String findEncoding() const;
 
     TLineCap lineCap() const;
     TLineJoin lineJoin() const;
@@ -215,7 +210,6 @@ namespace ipe {
 
     void allNames(Kind kind, AttributeSeq &seq) const;
     int findDefinition(Kind kind, Attribute sym) const;
-    void allCMaps(std::vector<String> &seq) const;
 
   private:
     std::vector<StyleSheet *> iSheets;
