@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -60,6 +60,24 @@ AppUiBase::AppUiBase(lua_State *L0, int model)
   }
   lua_pop(L, 1); // coordinates_format
 
+  lua_getfield(L, -1, "width_notes_bookmarks");
+  if (lua_isnumber(L, -1)) {
+    iWidthNotesBookmarks = (int) lua_tointeger(L, -1);
+  }
+  lua_pop(L, 1); // width_notes_bookmarks
+
+  iUiScale = 100;
+  lua_getfield(L, -1, "ui_scale");
+  if (lua_isnumber(L, -1))
+    iUiScale = (int) lua_tointeger(L, -1);
+  lua_pop(L, 1); // ui_scale
+
+  iToolbarScale = 100;
+  lua_getfield(L, -1, "toolbar_scale");
+  if (lua_isnumber(L, -1))
+    iToolbarScale = (int) lua_tointeger(L, -1);
+  lua_pop(L, 1); // toolbar_scale
+
   iScalings.push_back(1);
   lua_getfield(L, -1, "scale_factors");
   if (lua_istable(L, -1)) {
@@ -134,6 +152,9 @@ void AppUiBase::buildMenus()
   addItem(EEditMenu, "Duplicate", "duplicate");
   addItem(EEditMenu, "Select all", "select_all");
   addItem(EEditMenu);
+  addItem(EEditMenu, "Pick properties", "pick_properties");
+  addItem(EEditMenu, "Apply properties", "apply_properties");
+  addItem(EEditMenu);
   addItem(EEditMenu, "Insert text box", "insert_text_box");
   addItem(EEditMenu, "Edit object", "edit");
   addItem(EEditMenu, "Edit object as XML", "edit_as_xml");
@@ -159,7 +180,7 @@ void AppUiBase::buildMenus()
 
   addItem(EPropertiesMenu);
 
-  startSubMenu(EPropertiesMenu, "Text style");
+  startSubMenu(EPropertiesMenu, "Text style", ESubmenuTextStyle);
   iTextStyleMenu = endSubMenu();
 
   startSubMenu(EPropertiesMenu, "Horizontal alignment");
@@ -194,7 +215,13 @@ void AppUiBase::buildMenus()
   addItem(EModeMenu, "Mathematical symbols", "mode_math");
   addItem(EModeMenu, "Paragraphs", "mode_paragraph");
   addItem(EModeMenu, "Marks", "mode_marks");
-  addItem(EModeMenu, "Rectangles (with Shift: squares)", "mode_rectangles");
+  addItem(EModeMenu, "Axis-parallel rectangles (with Shift: squares)",
+	  "mode_rectangles1");
+  addItem(EModeMenu, "Axis-parallel rectangles, by center (with Shift: squares)",
+	  "mode_rectangles2");
+  addItem(EModeMenu, "Rectangles (with Shift: squares)", "mode_rectangles3");
+  addItem(EModeMenu, "Parallelograms (with Shift: axis-parallel)",
+	  "mode_parallelogram");
   addItem(EModeMenu, "Lines and polylines", "mode_lines");
   addItem(EModeMenu, "Polygons", "mode_polygons");
   addItem(EModeMenu, "Splines", "mode_splines");
@@ -211,17 +238,22 @@ void AppUiBase::buildMenus()
 
   // @ means the action can be used while drawing
   // * means the action is checkable (on/off)
-  // Checkable actions work differently in Qt and Win32:
+  // Checkable actions work differently in Qt and Win32/Cocoa:
   //   Qt already toggles the state
-  //   In Win32 the action needs to toggle the state.
+  //   In Win32/Cocoa the action needs to toggle the state.
 
-  addItem(ESnapMenu, "Snap to vertex", "@snapvtx");
-  addItem(ESnapMenu, "Snap to control point", "@snapctl");
-  addItem(ESnapMenu, "Snap to boundary", "@snapbd");
-  addItem(ESnapMenu, "Snap to intersection", "@snapint");
-  addItem(ESnapMenu, "Snap to grid", "@snapgrid");
-  addItem(ESnapMenu, "Angular snap", "@snapangle");
-  addItem(ESnapMenu, "Automatic snap", "@snapauto");
+  addItem(ESnapMenu, "Snap to vertex", "@*snapvtx");
+  addItem(ESnapMenu, "Snap to control point", "@*snapctl");
+  addItem(ESnapMenu, "Snap to boundary", "@*snapbd");
+  addItem(ESnapMenu, "Snap to intersection", "@*snapint");
+  addItem(ESnapMenu, "Snap to grid", "@*snapgrid");
+  addItem(ESnapMenu, "Angular snap", "@*snapangle");
+  addItem(ESnapMenu, "Automatic snap", "@*snapauto");
+  addItem(ESnapMenu);
+  startSubMenu(ESnapMenu, "Grid size", ESubmenuGridSize);
+  iGridSizeMenu = endSubMenu();
+  startSubMenu(ESnapMenu, "Radial angle", ESubmenuAngleSize);
+  iAngleSizeMenu = endSubMenu();
   addItem(ESnapMenu);
   addItem(ESnapMenu, "Set origin && snap", "@set_origin_snap");
   addItem(ESnapMenu, "Hide axes", "@hide_axes");
@@ -272,10 +304,10 @@ void AppUiBase::buildMenus()
   addItem(ELayerMenu);
   addItem(ELayerMenu, "Select all in active  layer",
     "select_in_active_layer");
-  startSubMenu(ELayerMenu, "Select all in layer");
+  startSubMenu(ELayerMenu, "Select all in layer", ESubmenuSelectLayer);
   iSelectLayerMenu = endSubMenu();
   addItem(ELayerMenu, "Move to active layer", "move_to_active_layer");
-  startSubMenu(ELayerMenu, "Move to layer");
+  startSubMenu(ELayerMenu, "Move to layer", ESubmenuMoveLayer);
   iMoveToLayerMenu = endSubMenu();
 
   addItem(EViewMenu, "Next view", "next_view");
@@ -306,7 +338,7 @@ void AppUiBase::buildMenus()
   addItem(EPageMenu, "Edit notes", "edit_notes");
   addItem(EPageMenu, "Page sorter", "page_sorter");
   addItem(EPageMenu);
-#ifdef IPEUI_WIN32
+#ifndef IPEUI_QT
   // In Qt these are created using "toggleViewAction()"
   addItem(EPageMenu, "Notes", "@*toggle_notes");
   addItem(EPageMenu, "Bookmarks", "@*toggle_bookmarks");
@@ -327,7 +359,9 @@ void AppUiBase::buildMenus()
   }
   lua_pop(L, 2); // developer, prefs
 
+#ifndef IPEUI_COCOA
   addItem(EHelpMenu, "&About Ipe", "about");
+#endif
 
   // build Ipelet menu
   lua_getglobal(L, "ipelets");
@@ -371,14 +405,17 @@ void AppUiBase::buildMenus()
 
 // --------------------------------------------------------------------
 
-void AppUiBase::canvasObserverWheelMoved(int degrees)
+void AppUiBase::canvasObserverWheelMoved(double degrees, bool precise)
 {
-  if (degrees > 0)
-    action("wheel_zoom_in");
-  else if (degrees < 0)
-    action("wheel_zoom_out");
-  else
-    // result of a zoom gesture
+  if (degrees != 0.0) {
+    lua_rawgeti(L, LUA_REGISTRYINDEX, iModel);
+    lua_getfield(L, -1, "wheel_zoom");
+    lua_insert(L, -2); // model
+    lua_pushnumber(L, degrees);
+    lua_pushboolean(L, precise);
+    lua_call(L, 3, 0);
+  } else
+    // result of a zoom gesture (Windows only, currently)
     setZoom(iCanvas->zoom());
 }
 
@@ -444,6 +481,11 @@ void AppUiBase::canvasObserverSizeChanged()
 
 // --------------------------------------------------------------------
 
+int AppUiBase::actionInfo(lua_State *L) const
+{
+  return 0; // only Windows will override this
+}
+
 static void call_selector(lua_State *L, int model, String name)
 {
   // calls model selector
@@ -582,7 +624,7 @@ void AppUiBase::showMarksInCombo(const Cascade *sheet)
   sheet->allNames(ESymbol, seq);
   for (uint i = 0; i < seq.size(); ++i) {
     String s = stripMark(seq[i]);
-    if (!s.isEmpty()) {
+    if (!s.empty()) {
       addCombo(EUiMarkShape, s);
       iComboContents[EUiMarkShape].push_back(s);
     }
