@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -34,8 +34,6 @@
 
 #include "ipecanvas.h"
 
-using namespace ipe;
-
 // avoid including Lua headers here
 typedef struct lua_State lua_State;
 
@@ -56,6 +54,13 @@ typedef HMENU MENUHANDLE;
 typedef QWidget *WINID;
 typedef QMenu *MENUHANDLE;
 #endif
+#ifdef IPEUI_COCOA
+#include <Cocoa/Cocoa.h>
+typedef NSWindow *WINID;
+typedef NSMenu *MENUHANDLE;
+#endif
+
+using namespace ipe;
 
 class AppUiBase;
 extern WINID check_winid(lua_State *L, int i);
@@ -76,6 +81,11 @@ public:
 	 EUiSymbolSize, EUiGridSize, EUiAngleSize,
 	 EUiView, EUiPage, EUiViewMarked, EUiPageMarked };
 
+  // tags for submenus
+  enum { ESubmenuGridSize = 1000, ESubmenuAngleSize,
+	 ESubmenuTextStyle, ESubmenuSelectLayer, ESubmenuMoveLayer,
+	 ESubmenuFin };
+
 public:
   virtual ~AppUiBase();
 
@@ -85,10 +95,11 @@ public:
   void setGridAngleSize(Attribute abs_grid, Attribute abs_angle);
   void setAttributes(const AllAttributes &all, Cascade *sheet);
 
+  void luaShowPathStylePopup(Vector v);
+  void luaBookmarkSelected(int index);
+
 public:  // What platforms must implement:
   virtual WINID windowId() = 0;
-  virtual int actionId(const char *name) const = 0;
-  virtual int actionAlwaysOn(const char *name) const = 0;
   virtual void closeWindow() = 0;
   virtual bool actionState(const char *name) = 0;
   virtual void setActionState(const char *name, bool value) = 0;
@@ -102,19 +113,22 @@ public:  // What platforms must implement:
   virtual void action(String name) = 0;
   virtual void setActionsEnabled(bool mode) = 0;
   virtual void setMouseIndicator(const char *s) = 0;
+  virtual void setSnapIndicator(const char *s) = 0;
   virtual void setBookmarks(int no, const String *s) = 0;
   virtual void setToolVisible(int m, bool vis) = 0;
   virtual int pageSorter(lua_State *L, Document *doc,
 			 int width, int height, int thumbWidth) = 0;
   virtual int clipboard(lua_State *L) = 0;
   virtual int setClipboard(lua_State *L) = 0;
+  // Only used on Windows to compute shortcuts:
+  virtual int actionInfo(lua_State *L) const;
 
 protected:   // What platforms must implement:
   virtual void addRootMenu(int id, const char *name) = 0;
   // if title == 0, add separator
   virtual void addItem(int id, const char *title = 0,
 		       const char *name = 0) = 0;
-  virtual void startSubMenu(int id, const char *name) = 0;
+  virtual void startSubMenu(int id, const char *name, int tag = 0) = 0;
   virtual void addSubItem(const char *title, const char *name) = 0;
   virtual MENUHANDLE endSubMenu() = 0;
   virtual void addCombo(int sel, String s) = 0;
@@ -126,7 +140,7 @@ protected:   // What platforms must implement:
   virtual void setButtonColor(int sel, Color color) = 0;
 
 protected:
-  virtual void canvasObserverWheelMoved(int degrees);
+  virtual void canvasObserverWheelMoved(double degrees, bool precise);
   virtual void canvasObserverMouseAction(int button);
   virtual void canvasObserverPositionChanged();
   virtual void canvasObserverToolChanged(bool hasTool);
@@ -138,11 +152,9 @@ protected:
 
   void luaSelector(String name, String value);
   void luaAction(String name);
-  void luaShowPathStylePopup(Vector v);
   void luaShowLayerBoxPopup(Vector v, String layer);
   void luaLayerAction(String name, String layer);
   void luaAbsoluteButton(const char *s);
-  void luaBookmarkSelected(int index);
 
   void buildMenus();
   void showInCombo(const Cascade *sheet, Kind kind,
@@ -157,17 +169,22 @@ protected:
   MENUHANDLE iSelectLayerMenu;
   MENUHANDLE iMoveToLayerMenu;
   MENUHANDLE iTextStyleMenu;
+  MENUHANDLE iGridSizeMenu;
+  MENUHANDLE iAngleSizeMenu;
 
   Cascade *iCascade;
-  AllAttributes iAll;  // current settings in UI
+  AllAttributes iAll; // current settings in UI
   std::vector<String> iComboContents[EUiView];
 
   CanvasBase *iCanvas;
 
+  int iWidthNotesBookmarks;
   std::vector<int> iScalings;
   String iCoordinatesFormat;
   int iMouseIn;
   double iMouseFactor;
+  int iUiScale;
+  int iToolbarScale;
 };
 
 // --------------------------------------------------------------------
