@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -60,55 +60,59 @@ using namespace ipelua;
 
 #define SIZE 16
 
+inline QSize adapt_size(const QSize &size, int factor)
+{
+  return QSize(factor * size.width() / 100, factor * size.height() / 100);
+}
+
+static bool add_bitmap(QIcon &icon, String base, const char *ext)
+{
+  if (Platform::fileExists(base + ext)) {
+    icon.addFile(QIpe(base + ext));
+    return true;
+  }
+  return false;
+}
+
 QIcon prefsIcon(String name)
 {
   String dir = ipeIconDirectory();
-  if (Platform::fileExists(dir + name + ".png"))
-    return QIcon(QIpe(dir + name + ".png"));
-  else
+  String base = dir + name;
+  if (Platform::fileExists(base + ".png")) {
+    QIcon icon(QIpe(base + ".png"));
+    if (!add_bitmap(icon, base, "@2x.png"))
+      icon.addPixmap(icon.pixmap(QSize(24, 24)).scaledToHeight(48));
+    add_bitmap(icon, base, "_32x32.png");
+    add_bitmap(icon, base, "_32x32@2x.png");
+    add_bitmap(icon, base, "128.png");  // for application icon
+    add_bitmap(icon, base, "512.png");  // for application icon
+    /*
+    auto l = icon.availableSizes();
+    for (int i = 0; i < l.size(); ++i)
+      ipeDebug("Icon %s has size %d x %d", name.z(), l[i].width(), l[i].height());
+    */
+    return icon;
+  } else
     return QIcon();
 }
 
-QPixmap prefsPixmap(String name)
+QPixmap AppUi::prefsPixmap(String name)
 {
-  return QPixmap(QIpe(ipeIconDirectory() + name + ".png"));
+  QIcon icon = prefsIcon(name);
+  int size = iUiScale * 24 / 100;
+  return icon.pixmap(QSize(size, size));
 }
 
-QIcon prefsColorIcon(Color color)
+QIcon AppUi::prefsColorIcon(Color color)
 {
-  QPixmap pixmap(SIZE, SIZE);
+  int size = iUiScale * SIZE / 100;
+  QPixmap pixmap(size, size);
   pixmap.fill(QIpe(color));
   return QIcon(pixmap);
 }
 
 // --------------------------------------------------------------------
 
-IpeApp::IpeApp(lua_State *L0, int &argc, char **argv)
-  : QApplication(argc, argv)
-{
-  L = L0;
-}
-
-bool IpeApp::event(QEvent *event)
-{
-  switch (event->type()) {
-  case QEvent::FileOpen:
-    loadFile(static_cast<QFileOpenEvent *>(event)->file());
-    return true;
-  default:
-    return QApplication::event(event);
-  }
-}
-
-void IpeApp::loadFile(const QString &fileName)
-{
-  lua_getglobal(L, "file_open_event");
-  lua_pushstring(L, fileName.toUtf8());
-  lua_call(L, 1, 0);
-}
-
-// --------------------------------------------------------------------
-
 QAction *AppUi::findAction(const char *name) const
 {
   std::map<String, QAction*>::const_iterator it = iActions.find(name);
@@ -118,21 +122,8 @@ QAction *AppUi::findAction(const char *name) const
     return 0;
 }
 
-// Not used
-int AppUi::actionId(const char *name) const
-{
-  return 0;
-}
-
-// Not used
-int AppUi::actionAlwaysOn(const char *name) const
-{
-  return 0;
-}
-
 void AppUi::addItem(QMenu *m, const QString &title, const char *name)
 {
-  // bool canUseWhileDrawing = false;
   if (name[0] == '@') {
     // canUseWhileDrawing = true;
     name = name + 1;
@@ -144,6 +135,7 @@ void AppUi::addItem(QMenu *m, const QString &title, const char *name)
     name = name + 1;
   }
   QAction *a = new QAction(title, this);
+  a->setIconVisibleInMenu(false);
   if (checkable)
     a->setCheckable(true);
   lua_getglobal(L, "shortcuts");
@@ -153,23 +145,6 @@ void AppUi::addItem(QMenu *m, const QString &title, const char *name)
     a->setShortcut(QKeySequence(s));
     QString tt = title + " [" + s + "]";
     a->setToolTip(tt);
-  } else if (lua_istable(L, -1)) {
-    int no = lua_rawlen(L, -1);
-    QList<QKeySequence> kl;
-    QString prim;
-    for (int i = 1; i <= no; ++i) {
-      lua_rawgeti(L, -1, i);
-      if (lua_isstring(L, -1)) {
-	QString s = lua_tostring(L, -1);
-	kl.append(QKeySequence(s));
-	if (prim.isEmpty())
-	  prim = s;
-      }
-      lua_pop(L, 1); // pop string
-    }
-    a->setShortcuts(kl);
-    QString tt = title + " [" + prim + "]";
-    a->setToolTip(tt);
   }
   a->setIcon(prefsIcon(name));
   lua_pop(L, 2);
@@ -215,7 +190,7 @@ void AppUi::addItem(int id, const char *title, const char *name)
 static QMenu *submenu = 0;
 static int submenuId = 0;
 
-void AppUi::startSubMenu(int id, const char *name)
+void AppUi::startSubMenu(int id, const char *name, int tag)
 {
   submenuId = id;
   submenu = new QMenu(name);
@@ -237,7 +212,7 @@ Qt::DockWidgetArea getDockSide(lua_State *L, const char *name,
 {
   Qt::DockWidgetArea r = deflt;
   lua_getglobal(L, "prefs");
-  lua_getfield(L, -1, "docks");
+  lua_getfield(L, -1, "tools_placement");
   if (lua_istable(L, -1)) {
     lua_getfield(L, -1, name);
     const char *s = lua_tolstring(L, -1, 0);
@@ -247,13 +222,23 @@ Qt::DockWidgetArea getDockSide(lua_State *L, const char *name,
       r = Qt::RightDockWidgetArea;
     lua_pop(L, 1); // left or right
   }
-  lua_pop(L, 2); // prefs, docks
+  lua_pop(L, 2); // prefs, tools_placement
   return r;
 }
 
-// --------------------------------------------------------------------
+static void set_toolbar_size(QToolBar *bar, int factor)
+{
+  QSize size = bar->iconSize();
+  QSize size1 = adapt_size(size, factor);
+  bar->setIconSize(size1);
+  /*
+  ipeDebug("toolbar %s has size %d x %d, changed to %d x %d",
+	   bar->windowTitle().toUtf8().constData(), size.width(), size.height(),
+	   size1.width(), size1.height());
+  */
+}
 
-extern void qt_set_sequence_auto_mnemonic(bool b);
+// --------------------------------------------------------------------
 
 #if QT_VERSION < 0x050000
 AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
@@ -264,11 +249,9 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
 {
   setWindowIcon(prefsIcon("ipe"));
   QMainWindow::setAttribute(Qt::WA_DeleteOnClose);
+  qApp->setAttribute(Qt::AA_UseHighDpiPixmaps);
   setDockOptions(AnimatedDocks); // do not allow stacking properties and layers
 
-  // enable automatic shortcuts on Mac OS X
-  qt_set_sequence_auto_mnemonic(true);
-
   Canvas *canvas = new Canvas(this);
   iCanvas = canvas;
   setCentralWidget(canvas);
@@ -278,6 +261,11 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   addToolBarBreak();
   iObjectTools = addToolBar("Objects");
 
+  ipeDebug("devicePixelRatio: %g", qApp->devicePixelRatio());
+  set_toolbar_size(iEditTools, iToolbarScale);
+  set_toolbar_size(iSnapTools, iToolbarScale);
+  set_toolbar_size(iObjectTools, iToolbarScale);
+
   iActionMap = new QSignalMapper(this);
   connect(iActionMap, SIGNAL(mapped(const QString &)),
 	  SLOT(qAction(const QString &)));
@@ -308,6 +296,10 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
 	  SLOT(moveToLayerAction(QAction *)));
   connect(iTextStyleMenu, SIGNAL(triggered(QAction *)),
 	  SLOT(textStyleAction(QAction *)));
+  connect(iGridSizeMenu, SIGNAL(triggered(QAction *)),
+	  SLOT(gridSizeAction(QAction *)));
+  connect(iAngleSizeMenu, SIGNAL(triggered(QAction *)),
+	  SLOT(angleSizeAction(QAction *)));
 
   QSignalMapper *comboMap = new QSignalMapper(this);
 
@@ -391,8 +383,9 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   QWidget *wg = new QFrame();
   QGridLayout *lo = new QGridLayout();
   wg->setLayout(lo);
-  lo->setSpacing(0);
-  lo->setContentsMargins(0, 0, 0, 0);
+  int m = iUiScale * 2 / 100;
+  lo->setSpacing(1);
+  lo->setContentsMargins(m, m, m, m); // l t r b
   // wg->setMaximumWidth(150);
   lo->setSizeConstraint(QLayout::SetFixedSize);
   QButtonGroup *bg = new QButtonGroup(wg);
@@ -405,6 +398,7 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
       iButton[i]->setFocusPolicy(Qt::NoFocus);
       bg->addButton(iButton[i], i);
       lo->addWidget(iButton[i], i > 2 ? i+1 : i, 0);
+      iButton[i]->setIconSize(adapt_size(iButton[i]->iconSize(), iUiScale));
     }
     iSelector[i] = new QComboBox();
     iSelector[i]->setFocusPolicy(Qt::NoFocus);
@@ -444,10 +438,14 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   connect(iPathView, SIGNAL(activated(String)), SLOT(action(String)));
   connect(iPathView, SIGNAL(showPathStylePopup(Vector)),
 	  SLOT(showPathStylePopup(Vector)));
-  lo->addWidget(iPathView, 3, 0, 1, 2);
+  lo->addWidget(iPathView, 3, 1, 1, 1);
   // lo->setRowStretch(3, 1);
   iPropertiesTools->setWidget(wg);
 
+  iModeIndicator = new QLabel();
+  iModeIndicator->setPixmap(prefsPixmap("mode_select"));
+  lo->addWidget(iModeIndicator, 3, 0);
+
   QHBoxLayout *hol = new QHBoxLayout();
   iViewNumber = new QToolButton();
   iPageNumber = new QToolButton();
@@ -500,24 +498,26 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
 	  this, SLOT(aboutToShowMoveToLayerMenu()));
   connect(iTextStyleMenu, SIGNAL(aboutToShow()),
 	  this, SLOT(aboutToShowTextStyleMenu()));
+  connect(iGridSizeMenu, SIGNAL(aboutToShow()),
+	  this, SLOT(aboutToShowGridSizeMenu()));
+  connect(iAngleSizeMenu, SIGNAL(aboutToShow()),
+	  this, SLOT(aboutToShowAngleSizeMenu()));
+
+  iSnapIndicator = new QLabel(statusBar());
+  statusBar()->addPermanentWidget(iSnapIndicator, 0);
+  QFont font = iSnapIndicator->font();
+  font.setFamily("Monospace");
+  iSnapIndicator->setFont(font);
 
   iMouse = new QLabel(statusBar());
   findAction("coordinates|points")->setChecked(true);
+  findAction("scaling|1")->setChecked(true);
   statusBar()->addPermanentWidget(iMouse, 0);
-  QFont font = iMouse->font();
-  font.setFamily("Monospace");
   iMouse->setFont(font);
 
   iResolution = new QLabel(statusBar());
   statusBar()->addPermanentWidget(iResolution, 0);
 
-  lua_getglobal(L, "prefs");
-  lua_getfield(L, -1, "retina");
-  int retina = lua_toboolean(L, -1) ? 2 : 1;
-  ipeDebug("setRetina(%d)", retina);
-  iCanvas->setRetina(retina);
-  lua_pop(L, 2); // retina, prefs
-
   iCanvas->setObserver(this);
 }
 
@@ -546,6 +546,7 @@ void AppUi::aboutToShowMoveToLayerMenu()
 {
   iMoveToLayerMenu->clear();
   for (int i = 0; i < iLayerList->count(); ++i) {
+    ipeDebug("action: %s", IpeQ(iLayerList->item(i)->text()).z());
     QAction *a = new QAction(iLayerList->item(i)->text(), iMoveToLayerMenu);
     iMoveToLayerMenu->addAction(a);
   }
@@ -571,11 +572,47 @@ void AppUi::aboutToShowTextStyleMenu()
   }
 }
 
+void AppUi::aboutToShowGridSizeMenu()
+{
+  iGridSizeMenu->clear();
+  for (auto &name : iComboContents[EUiGridSize]) {
+    QString s = QIpe(name);
+    QAction *a = new QAction(s, iGridSizeMenu);
+    a->setCheckable(true);
+    if (s == iSelector[EUiGridSize]->currentText())
+      a->setChecked(true);
+    iGridSizeMenu->addAction(a);
+  }
+}
+
+void AppUi::aboutToShowAngleSizeMenu()
+{
+  iAngleSizeMenu->clear();
+  for (auto &name : iComboContents[EUiAngleSize]) {
+    QString s = QIpe(name);
+    QAction *a = new QAction(s, iAngleSizeMenu);
+    a->setCheckable(true);
+    if (s == iSelector[EUiAngleSize]->currentText())
+      a->setChecked(true);
+    iAngleSizeMenu->addAction(a);
+  }
+}
+
 void AppUi::textStyleAction(QAction *a)
 {
   action(String("textstyle|") + IpeQ(a->text()));
 }
 
+void AppUi::gridSizeAction(QAction *a)
+{
+  action(String("gridsize|") + IpeQ(a->text()));
+}
+
+void AppUi::angleSizeAction(QAction *a)
+{
+  action(String("anglesize|") + IpeQ(a->text()));
+}
+
 void AppUi::toolbarModifiersChanged()
 {
   if (iCanvas) {
@@ -646,7 +683,7 @@ void AppUi::setCheckMark(String name, Attribute a)
 
 void AppUi::setNumbers(String vno, bool vm, String pno, bool pm)
 {
-  if (vno.isEmpty()) {
+  if (vno.empty()) {
     iViewNumber->hide();
     iViewMarked->hide();
   } else {
@@ -655,7 +692,7 @@ void AppUi::setNumbers(String vno, bool vm, String pno, bool pm)
     iViewMarked->setCheckState(vm ? Qt::Checked : Qt::Unchecked);
     iViewMarked->show();
   }
-  if (pno.isEmpty()) {
+  if (pno.empty()) {
     iPageNumber->hide();
     iPageMarked->hide();
   } else {
@@ -807,19 +844,18 @@ void AppUi::comboSelector(int id)
 
 static const char * const aboutText =
 "<qt><h2>Ipe %d.%d.%d</h2>"
-"<p>Copyright (c) 1993-2015 Otfried Cheong</p>"
+"<p>Copyright (c) 1993-2016 Otfried Cheong</p>"
 "<p>The extensible drawing editor Ipe creates figures "
 "in Postscript and PDF format, "
 "using LaTeX to format the text in the figures.</p>"
 "<p>Ipe relies on the following fine pieces of software:"
 "<ul>"
-"<li> Pdflatex"
+"<li> Pdftex, Xetex, or Luatex"
 "<li> %s (%d kB used)" // Lua
 "<li> The font rendering library %s"
 "<li> The rendering library Cairo %s / %s"
 "<li> The GUI toolkit Qt %s / %s"
 "<li> The compression library zlib %s"
-"<li> Some code from Xpdf"
 "</ul>"
 "<p>Ipe is released under the GNU Public License.</p>"
 "<p>See <a href=\"http://ipe.otfried.org\">ipe.otfried.org</a>"
@@ -860,8 +896,11 @@ void AppUi::action(String name)
     setWindowState(windowState() ^ Qt::WindowFullScreen);
   } else if (name == "about") {
     aboutIpe();
-  } else
+  } else {
+    if (name.left(5) == "mode_")
+      iModeIndicator->setPixmap(prefsPixmap(name));
     luaAction(name);
+  }
 }
 
 // --------------------------------------------------------------------
@@ -939,6 +978,11 @@ void AppUi::setMouseIndicator(const char *s)
   iMouse->setText(s);
 }
 
+void AppUi::setSnapIndicator(const char *s)
+{
+  iSnapIndicator->setText(s);
+}
+
 void AppUi::explain(const char *s, int t)
 {
   statusBar()->showMessage(QString::fromUtf8(s), t);
