@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -57,19 +57,6 @@ class QSignalMapper;
 
 // --------------------------------------------------------------------
 
-class IpeApp : public QApplication {
-  Q_OBJECT
-public:
-  IpeApp(lua_State *L0, int &argc, char **argv);
-protected:
-  virtual bool event(QEvent *);
-private:
-  void loadFile(const QString &fileName);
-  lua_State *L;
-};
-
-// --------------------------------------------------------------------
-
 class AppUi : public QMainWindow, public AppUiBase {
   Q_OBJECT
 
@@ -83,28 +70,26 @@ public:
 
   QAction *findAction(const char *name) const;
 
-  virtual void setLayers(const Page *page, int view);
+  virtual void setLayers(const Page *page, int view) override;
 
-  virtual void setZoom(double zoom);
-  virtual void setActionsEnabled(bool mode);
-  virtual void setNumbers(String vno, bool vm, String pno, bool pm);
-  virtual void setNotes(String notes);
+  virtual void setZoom(double zoom) override;
+  virtual void setActionsEnabled(bool mode) override;
+  virtual void setNumbers(String vno, bool vm, String pno, bool pm) override;
+  virtual void setNotes(String notes) override;
 
-  virtual int actionId(const char *name) const;
-  virtual int actionAlwaysOn(const char *name) const;
-  virtual WINID windowId();
-  virtual void closeWindow();
-  virtual bool actionState(const char *name);
-  virtual void setActionState(const char *name, bool value);
-  virtual void setWindowCaption(bool mod, const char *s);
-  virtual void explain(const char *s, int t);
-  virtual void showWindow(int width, int height);
-  virtual void setBookmarks(int no, const String *s);
-  virtual void setToolVisible(int m, bool vis);
+  virtual WINID windowId() override;
+  virtual void closeWindow() override;
+  virtual bool actionState(const char *name) override;
+  virtual void setActionState(const char *name, bool value) override;
+  virtual void setWindowCaption(bool mod, const char *s) override;
+  virtual void explain(const char *s, int t) override;
+  virtual void showWindow(int width, int height) override;
+  virtual void setBookmarks(int no, const String *s) override;
+  virtual void setToolVisible(int m, bool vis) override;
   virtual int pageSorter(lua_State *L, Document *doc,
-			 int width, int height, int thumbWidth);
-  virtual int clipboard(lua_State *L);
-  virtual int setClipboard(lua_State *L);
+			 int width, int height, int thumbWidth) override;
+  virtual int clipboard(lua_State *L) override;
+  virtual int setClipboard(lua_State *L) override;
 
 public slots:
   void action(String name);
@@ -112,6 +97,8 @@ public slots:
   void selectLayerAction(QAction *a);
   void moveToLayerAction(QAction *a);
   void textStyleAction(QAction *a);
+  void gridSizeAction(QAction *a);
+  void angleSizeAction(QAction *a);
 
   void layerAction(String name, String layer);
   void toolbarModifiersChanged();
@@ -127,6 +114,9 @@ public slots:
   void aboutToShowSelectLayerMenu();
   void aboutToShowMoveToLayerMenu();
   void aboutToShowTextStyleMenu();
+  void aboutToShowGridSizeMenu();
+  void aboutToShowAngleSizeMenu();
+
   void showPathStylePopup(Vector v);
   void showLayerBoxPopup(Vector v, String layer);
 
@@ -135,21 +125,24 @@ private:
   void addItem(int m, const QString &title, const char *name);
   void addSnap(const char *name);
   void addEdit(const char *name);
+  QIcon prefsColorIcon(Color color);
+  QPixmap prefsPixmap(String name);
 
 private:
-  virtual void addRootMenu(int id, const char *name);
-  virtual void addItem(int id, const char *title, const char *name);
-  virtual void startSubMenu(int id, const char *name);
-  virtual void addSubItem(const char *title, const char *name);
-  virtual MENUHANDLE endSubMenu();
-  virtual void setMouseIndicator(const char *s);
-  virtual void addCombo(int sel, String s);
-  virtual void resetCombos();
-  virtual void addComboColors(AttributeSeq &sym, AttributeSeq &abs);
-  virtual void setComboCurrent(int sel, int idx);
-  virtual void setCheckMark(String name, Attribute a);
-  virtual void setPathView(const AllAttributes &all, Cascade *sheet);
-  virtual void setButtonColor(int sel, Color color);
+  virtual void addRootMenu(int id, const char *name) override;
+  virtual void addItem(int id, const char *title, const char *name) override;
+  virtual void startSubMenu(int id, const char *name, int tag) override;
+  virtual void addSubItem(const char *title, const char *name) override;
+  virtual MENUHANDLE endSubMenu() override;
+  virtual void setMouseIndicator(const char *s) override;
+  virtual void setSnapIndicator(const char *s) override;
+  virtual void addCombo(int sel, String s) override;
+  virtual void resetCombos() override;
+  virtual void addComboColors(AttributeSeq &sym, AttributeSeq &abs) override;
+  virtual void setComboCurrent(int sel, int idx) override;
+  virtual void setCheckMark(String name, Attribute a) override;
+  virtual void setPathView(const AllAttributes &all, Cascade *sheet) override;
+  virtual void setButtonColor(int sel, Color color) override;
 
 protected:
   void closeEvent(QCloseEvent *ev);
@@ -186,6 +179,8 @@ private:
   LayerBox *iLayerList;
   QTextEdit *iPageNotes;
 
+  QLabel *iModeIndicator;
+  QLabel *iSnapIndicator;
   QLabel *iMouse;
   QLabel *iResolution;
 
