@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -59,6 +59,8 @@
 #define ID_PATHVIEW 8700
 #define ID_MOVETOLAYER_BASE 8800
 #define ID_SELECTINLAYER_BASE 9000
+#define ID_GRIDSIZE_BASE 9100
+#define ID_ANGLESIZE_BASE 9500
 
 using namespace ipe;
 using namespace ipelua;
@@ -77,7 +79,18 @@ const wchar_t AppUi::className[] = L"ipeWindowClass";
 static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
 {
   String dir = ipeIconDirectory();
-  String fname = dir + action + ".png";
+  String base = dir + action;
+  String fname;
+  int ww = w < 0 ? -w : w;
+  if (ww >= 64 && Platform::fileExists(base + "_32x32@2x.png"))
+    fname = base + "_32x32@2x.png";
+  else if (ww >= 48 && Platform::fileExists(base + "@2x.png"))
+    fname = base + "@2x.png";
+  else if (ww >= 32 && Platform::fileExists(base + "_32x32.png"))
+    fname = base + "_32x32.png";
+  else
+    fname = base + ".png";
+  // ipeDebug("loadIcon %s at %d x %d from %s", action.z(), w, h, fname.z());
   if (!Platform::fileExists(fname))
     return NULL;
 
@@ -154,9 +167,10 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
   return bm;
 }
 
-static int loadIcon(String action, HIMAGELIST il)
+static int loadIcon(String action, HIMAGELIST il, int scale)
 {
-  HBITMAP bm = loadIcon(action, TBICONSIZE, TBICONSIZE, -1, -1, -1);
+  int size = scale * TBICONSIZE / 100;
+  HBITMAP bm = loadIcon(action, size, size, -1, -1, -1);
   if (bm == NULL)
     return -1;
   int r = ImageList_Add(il, bm, NULL);
@@ -164,23 +178,24 @@ static int loadIcon(String action, HIMAGELIST il)
   return r;
 }
 
-static HBITMAP loadButtonIcon(String action)
+static HBITMAP loadButtonIcon(String action, int scale)
 {
+  int size = scale * 24 / 100;
   DWORD rgb = GetSysColor(COLOR_BTNFACE);
   int r0 = GetRValue(rgb);
   int g0 = GetGValue(rgb);
   int b0 = GetBValue(rgb);
-  return loadIcon(action, -1, -1, r0, g0, b0);
+  return loadIcon(action, -size, -1, r0, g0, b0);
 }
 
-static HBITMAP colorIcon(Color color)
+static HBITMAP colorIcon(Color color, int scale)
 {
   int r = int(color.iRed.internal() * 255 / 1000);
   int g = int(color.iGreen.internal() * 255 / 1000);
   int b = int(color.iBlue.internal() * 255 / 1000);
   COLORREF rgb = RGB(r, g, b);
-  int cx = 22;
-  int cy = 22;
+  int cx = scale * 22 / 100;
+  int cy = scale * 22 / 100;
   HDC hdc = GetDC(NULL);
   HDC memDC = CreateCompatibleDC(hdc);
   HBITMAP bm = CreateCompatibleBitmap(hdc, cx, cy);
@@ -209,7 +224,7 @@ void AppUi::createAction(String name, String tooltip, bool canWhileDrawing)
 {
   SAction s;
   s.name = String(name);
-  s.icon = loadIcon(name, hIcons);
+  s.icon = loadIcon(name, hIcons, iToolbarScale);
   s.tooltip = tooltip;
   s.alwaysOn = canWhileDrawing;
   iActions.push_back(s);
@@ -232,22 +247,16 @@ void AppUi::addItem(HMENU menu, const char *title, const char *name)
     lua_getglobal(L, "shortcuts");
     lua_getfield(L, -1, name);
     String sc;
-    if (lua_isstring(L, -1)) {
+    if (lua_isstring(L, -1))
       sc = lua_tostring(L, -1);
-    } else if (lua_istable(L, -1) && lua_rawlen(L, -1) > 0) {
-      lua_rawgeti(L, -1, 1);
-      if (lua_isstring(L, -1))
-	sc = lua_tostring(L, -1);
-      lua_pop(L, 1); // pop string
-    }
     lua_pop(L, 2); // shortcuts, shortcuts[name]
 
     String tooltip = title;
-    if (!sc.isEmpty())
+    if (!sc.empty())
       tooltip = tooltip + " [" + sc + "]";
     createAction(name, tooltip, canUseWhileDrawing);
 
-    if (sc.isEmpty()) {
+    if (sc.empty()) {
       WString wtitle(title);
       AppendMenuW(menu, MF_STRING, iActions.size() - 1 + IDBASE, wtitle);
     } else {
@@ -267,7 +276,7 @@ void AppUi::addItem(int id, const char *title, const char *name)
 
 static HMENU submenu = 0;
 
-void AppUi::startSubMenu(int id, const char *name)
+void AppUi::startSubMenu(int id, const char *name, int tag)
 {
   submenu = CreatePopupMenu();
   WString wname(name);
@@ -401,11 +410,20 @@ AppUi::AppUi(lua_State *L0, int model)
   iFullScreen = false;
 }
 
+static void insert_tb(HWND hRebar, REBARBANDINFO &rbBand, HWND tb, int size)
+{
+  rbBand.hwndChild = tb;
+  rbBand.cxMinChild = size;
+  rbBand.cx = size;
+  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
+}
+
 // called during Window creation
 void AppUi::initUi()
 {
-  hIcons = ImageList_Create(TBICONSIZE, TBICONSIZE, ILC_COLOR32, 20, 4);
-  hColorIcons = ImageList_Create(22, 22, ILC_COLOR32, 20, 4);
+  int tbsize = iToolbarScale * TBICONSIZE / 100;
+  hIcons = ImageList_Create(tbsize, tbsize, ILC_COLOR32, 20, 4);
+  hColorIcons = ImageList_Create(uiscale(22), uiscale(22), ILC_COLOR32, 20, 4);
   HINSTANCE hInst = (HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE);
 
   hMenuBar = CreateMenu();
@@ -438,7 +456,7 @@ void AppUi::initUi()
   addTButton(hSnapTools, "snapangle", BTNS_CHECK);
   addTButton(hSnapTools, "snapauto", BTNS_CHECK);
 
-#define SNAP_BUTTONS 6
+#define SNAP_BUTTONS 7
 
   hEditTools = createToolBar(hInst);
 
@@ -475,7 +493,10 @@ void AppUi::initUi()
   addTButton(hObjectTools, "mode_math", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_paragraph", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_marks", BTNS_CHECKGROUP);
-  addTButton(hObjectTools, "mode_rectangles", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_rectangles1", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_rectangles2", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_rectangles3", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_parallelogram", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_lines", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_polygons", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_splines", BTNS_CHECKGROUP);
@@ -495,7 +516,7 @@ void AppUi::initUi()
     hSelector[i] =
       CreateWindowExW(0, WC_COMBOBOXEXW, NULL,
 		      WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST,
-		      0, 0, 100, 300, hwnd,
+		      0, 0, uiscale(100), uiscale(300), hwnd,
 		      (HMENU) (ID_SELECTOR_BASE + i), hInst, NULL);
 
   hRebar = CreateWindowEx(WS_EX_TOOLWINDOW, REBARCLASSNAME, NULL,
@@ -520,31 +541,31 @@ void AppUi::initUi()
   rbBand.fStyle = RBBS_CHILDEDGE | RBBS_GRIPPERALWAYS | RBBS_HIDETITLE;
   rbBand.cyMinChild = HIWORD(dwBtnSize) + 4;
   rbBand.cyChild = HIWORD(dwBtnSize) + 4;
+  int bw = LOWORD(dwBtnSize);
 
-  rbBand.hwndChild = hSnapTools;
-  rbBand.cxMinChild = SNAP_BUTTONS * LOWORD(dwBtnSize);
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hSelector[EUiGridSize];
-  rbBand.cxMinChild = 160;
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hSelector[EUiAngleSize];
-  rbBand.cxMinChild = 100;
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hEditTools;
-  rbBand.cxMinChild = EDIT_BUTTONS * LOWORD(dwBtnSize);
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hObjectTools;
-  rbBand.cxMinChild = MODE_BUTTONS * LOWORD(dwBtnSize);
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
+  lua_getglobal(L, "prefs");
+  lua_getfield(L, -1, "toolbar_order");
+  if (lua_istable(L, -1)) {
+    int n = lua_rawlen(L, -1);
+    for (int i = 1; i <= n; ++i) {
+      lua_rawgeti(L, -1, i);
+      if (lua_isstring(L, -1)) {
+	String s = lua_tostring(L, -1);
+	if (s == "edit")
+	  insert_tb(hRebar, rbBand, hEditTools, EDIT_BUTTONS * bw);
+	else if (s == "grid")
+	  insert_tb(hRebar, rbBand, hSelector[EUiGridSize], 160);
+	else if (s == "angle")
+	  insert_tb(hRebar, rbBand, hSelector[EUiAngleSize], 100);
+	else if (s == "snap")
+	  insert_tb(hRebar, rbBand, hSnapTools, SNAP_BUTTONS * bw);
+	else if (s == "mode")
+	  insert_tb(hRebar, rbBand, hObjectTools, MODE_BUTTONS * bw);
+      }
+      lua_pop(L, 1);
+    }
+  }
+  lua_pop(L, 2); // toolbar_order, prefs
 
   // ------------------------------------------------------------
 
@@ -560,25 +581,25 @@ void AppUi::initUi()
   for (int i = 0; i < EUiGridSize; ++i)
     hSelector[i] = CreateWindowExW(0, WC_COMBOBOXEXW, NULL,
 				   WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST,
-				   0, 0, 100, 300, hwnd,
+				   0, 0, uiscale(100), uiscale(300), hwnd,
 				   (HMENU) (ID_SELECTOR_BASE + i), hInst, NULL);
   SendMessage(hButton[EUiPen], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("pen"));
+	      (LPARAM) loadButtonIcon("pen", iUiScale));
   SendMessage(hButton[EUiTextSize], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("mode_label"));
+	      (LPARAM) loadButtonIcon("mode_label", iUiScale));
   SendMessage(hButton[EUiSymbolSize], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("mode_marks"));
+	      (LPARAM) loadButtonIcon("mode_marks", iUiScale));
   SendMessage(hButton[EUiStroke], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) colorIcon(Color(1000, 0, 0)));
+	      (LPARAM) colorIcon(Color(1000, 0, 0), iUiScale));
   SendMessage(hButton[EUiFill], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) colorIcon(Color(1000, 1000, 0)));
+	      (LPARAM) colorIcon(Color(1000, 1000, 0), iUiScale));
 
   hButton[EUiMarkShape] =
     CreateWindowExW(0, L"static", NULL,
 		    WS_CHILD | WS_VISIBLE | SS_BITMAP | SS_CENTERIMAGE,
 		    0, 0, 0, 0, hwnd, NULL, hInst, NULL);
   SendMessage(hButton[EUiMarkShape], STM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("mode_marks"));
+	      (LPARAM) loadButtonIcon("mode_marks", iUiScale));
 
   iPathView = new PathView(hwnd, ID_PATHVIEW);
 
@@ -653,11 +674,12 @@ void AppUi::initUi()
   setTooltip(hLayers, "Layers of this page");
 
   setCheckMark("coordinates|", "points");
+  setCheckMark("scaling|", "1");
   setCheckMark("mode_", "select");
 
   SetFocus(hwndCanvas);
 
-  hFont = CreateFontW(18, 0, 0, 0, FW_DONTCARE,
+  hFont = CreateFontW(uiscale(18), 0, 0, 0, FW_DONTCARE,
 		      FALSE, FALSE, FALSE, ANSI_CHARSET,
 		      OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
 		      DEFAULT_QUALITY,
@@ -702,54 +724,69 @@ void AppUi::layoutChildren(bool resizeRebar)
   RECT rect;
   rect.left = 0;
   rect.top = tbh;
-  rect.right = 200;
+  rect.right = uiscale(200);
   rect.bottom = rc.bottom - sbh;
   InvalidateRect(hwnd, &rect, TRUE);
-  int pbh = 28 * 7 + 70;
-  MoveWindow(hProperties, 8, tbh + 2, 180, pbh, TRUE);
+  int pbh = uiscale(28 * 7 + 70);
+  MoveWindow(hProperties, uiscale(8), tbh + uiscale(2), uiscale(180), pbh, TRUE);
 
   for (int i = 0; i <= EUiSymbolSize; ++i) {
-    int y = 24 + 28 * i + (i > 2 ? 42 : 0);
-    MoveWindow(hButton[i], 16, tbh + y, 26, 26, TRUE);
-    MoveWindow(hSelector[i], 50, tbh + y, 132, 26, TRUE);
+    int y = uiscale(24 + 28 * i + (i > 2 ? 42 : 0));
+    MoveWindow(hButton[i], uiscale(16), tbh + y,
+	       uiscale(26), uiscale(26), TRUE);
+    MoveWindow(hSelector[i], uiscale(50), tbh + y,
+	       uiscale(132), uiscale(26), TRUE);
   }
 
-  int y = tbh + 24 + 28 * 6 + 42;
-  MoveWindow(hViewNumber, 16, y, 68, 26, TRUE);
-  MoveWindow(hPageNumber, 116, y, 68, 26, TRUE);
-  MoveWindow(hViewMarked, 85, y + 5, 12, 12, TRUE);
-  MoveWindow(hPageMarked, 100, y + 5, 12, 12, TRUE);
+  int y = tbh + uiscale(24 + 28 * 6 + 42);
+  MoveWindow(hViewNumber, uiscale(16), y, uiscale(68), uiscale(26), TRUE);
+  MoveWindow(hPageNumber, uiscale(116), y, uiscale(68), uiscale(26), TRUE);
+  MoveWindow(hViewMarked, uiscale(85), y + uiscale(5),
+	     uiscale(12), uiscale(12), TRUE);
+  MoveWindow(hPageMarked, uiscale(100), y + uiscale(5),
+	     uiscale(12), uiscale(12), TRUE);
 
-  MoveWindow(iPathView->windowId(), 16, tbh + 24 + 28 * 3, 168, 40, TRUE);
+  MoveWindow(iPathView->windowId(), uiscale(16), tbh + uiscale(24 + 28 * 3),
+	     uiscale(168), uiscale(40), TRUE);
 
-  MoveWindow(hLayerGroup, 8, tbh + 6 + pbh, 180, cvh - pbh - 8, TRUE);
-  MoveWindow(hLayers, 16, tbh + 24 + pbh, 164, cvh - pbh - 32, TRUE);
+  MoveWindow(hLayerGroup, uiscale(8), tbh + uiscale(6) + pbh,
+	     uiscale(180), cvh - pbh - uiscale(8), TRUE);
+  MoveWindow(hLayers, uiscale(16), tbh + uiscale(24) + pbh,
+	     uiscale(164), cvh - pbh - uiscale(32), TRUE);
 
   bool visNotes = IsWindowVisible(hNotes);
   bool visBookmarks = IsWindowVisible(hBookmarks);
-  int cvw = (visNotes || visBookmarks) ? rc.right - 400 : rc.right - 200;
-  MoveWindow(hwndCanvas, 200, tbh, cvw, cvh, TRUE);
+  int cvw = (visNotes || visBookmarks) ?
+    rc.right - uiscale(200 + iWidthNotesBookmarks) : rc.right - uiscale(200);
+  MoveWindow(hwndCanvas, uiscale(200), tbh, cvw, cvh, TRUE);
   if (visNotes || visBookmarks) {
-    rect.left = rc.right - 200;
+    rect.left = rc.right - uiscale(iWidthNotesBookmarks);
     rect.top = tbh;
     rect.right = rc.right;
     rect.bottom = rc.bottom - sbh;
     InvalidateRect(hwnd, &rect, TRUE);
   }
 
-  int nth = (visNotes && visBookmarks) ? cvh / 2 - 4: cvh;
+  int nth = (visNotes && visBookmarks) ? cvh / 2 - uiscale(4): cvh;
   int nty = tbh;
   if (visNotes) {
-    MoveWindow(hNotesGroup, rc.right - 200, nty, 200, nth, TRUE);
-    MoveWindow(hNotes, rc.right - 194, nty + 18, 186, nth - 24, TRUE);
-    nty += nth + 2;
+    MoveWindow(hNotesGroup, rc.right - uiscale(iWidthNotesBookmarks),
+	       nty, uiscale(iWidthNotesBookmarks), nth, TRUE);
+    MoveWindow(hNotes, rc.right - uiscale(iWidthNotesBookmarks - 6),
+	       nty + uiscale(18), uiscale(iWidthNotesBookmarks - 14),
+	       nth - uiscale(24), TRUE);
+    nty += nth + uiscale(2);
   }
   if (visBookmarks) {
-    MoveWindow(hBookmarksGroup, rc.right - 200, nty, 200, nth, TRUE);
-    MoveWindow(hBookmarks, rc.right - 194, nty + 18, 186, nth - 24, TRUE);
+    MoveWindow(hBookmarksGroup, rc.right - uiscale(iWidthNotesBookmarks),
+	       nty, uiscale(iWidthNotesBookmarks), nth, TRUE);
+    MoveWindow(hBookmarks, rc.right - uiscale(iWidthNotesBookmarks - 6),
+	       nty + uiscale(18),
+	       uiscale(iWidthNotesBookmarks - 14), nth - uiscale(24), TRUE);
   }
-  int statwidths[] = {rc.right - 200, rc.right - 80, rc.right };
-  SendMessage(hStatusBar, SB_SETPARTS, 3, (LPARAM) statwidths);
+  int statwidths[] = {rc.right - uiscale(320), rc.right - uiscale(220),
+		      rc.right - uiscale(80), rc.right };
+  SendMessage(hStatusBar, SB_SETPARTS, 4, (LPARAM) statwidths);
   iCanvas->update();
 }
 
@@ -781,7 +818,7 @@ void AppUi::addComboColors(AttributeSeq &sym, AttributeSeq &abs)
   item.iItem = (INT_PTR) -1;
   for (uint i = 0; i < sym.size(); ++i) {
     Color color = abs[i].color();
-    HBITMAP bm = colorIcon(color);
+    HBITMAP bm = colorIcon(color, iUiScale);
     ImageList_Add(hColorIcons, bm, NULL);
     DeleteObject(bm);
     String s = sym[i].string();
@@ -812,7 +849,7 @@ void AppUi::setButtonColor(int sel, Color color)
 {
   // ipeDebug("Changing color for %d");
   SendMessage(hButton[sel], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) colorIcon(color));
+	      (LPARAM) colorIcon(color, iUiScale));
 }
 
 void AppUi::setPathView(const AllAttributes &all, Cascade *sheet)
@@ -920,10 +957,10 @@ void AppUi::setNumbers(String vno, bool vm, String pno, bool pm)
   setWindowText(hPageNumber, pno.z());
   Button_SetCheck(hViewMarked, vm ? BST_CHECKED : BST_UNCHECKED);
   Button_SetCheck(hPageMarked, pm ? BST_CHECKED : BST_UNCHECKED);
-  EnableWindow(hViewNumber, !vno.isEmpty());
-  EnableWindow(hViewMarked, !vno.isEmpty());
-  EnableWindow(hPageNumber, !pno.isEmpty());
-  EnableWindow(hPageMarked, !pno.isEmpty());
+  EnableWindow(hViewNumber, !vno.empty());
+  EnableWindow(hViewMarked, !vno.empty());
+  EnableWindow(hPageNumber, !pno.empty());
+  EnableWindow(hPageMarked, !pno.empty());
 }
 
 void AppUi::setNotes(String notes)
@@ -1022,6 +1059,27 @@ void AppUi::populateTextStyleMenu()
 		     TEXT_STYLE_BASE + check, MF_BYCOMMAND);
 }
 
+void AppUi::populateSizeMenu(HMENU h, int sel, int base)
+{
+  clearMenu(h);
+  auto i = 0;
+  for (auto &name : iComboContents[sel]) {
+    WString ws(name);
+    AppendMenuW(h, MF_STRING, base + i, ws);
+    ++i;
+  }
+  auto cur = SendMessage(hSelector[sel], CB_GETCURSEL, 0, 0);
+  if (cur != CB_ERR)
+    CheckMenuRadioItem(h, base, base + iComboContents[sel].size() - 1,
+		       base + cur, MF_BYCOMMAND);
+}
+
+void AppUi::populateSizeMenus()
+{
+  populateSizeMenu(iGridSizeMenu, EUiGridSize, ID_GRIDSIZE_BASE);
+  populateSizeMenu(iAngleSizeMenu, EUiAngleSize, ID_ANGLESIZE_BASE);
+}
+
 void AppUi::populateLayerMenus()
 {
   clearMenu(iSelectLayerMenu);
@@ -1042,6 +1100,11 @@ int AppUi::findAction(const char *name) const
   return -1;
 }
 
+int AppUi::actionId(const char *name) const
+{
+  return findAction(name) + IDBASE;
+}
+
 int AppUi::iconId(const char *name) const
 {
   int i = findAction(name);
@@ -1051,16 +1114,13 @@ int AppUi::iconId(const char *name) const
     return I_IMAGENONE;
 }
 
-int AppUi::actionId(const char *name) const
+int AppUi::actionInfo(lua_State *L) const
 {
-  int i = findAction(name);
-  return (i >= 0) ? i + IDBASE : 0;
-}
-
-int AppUi::actionAlwaysOn(const char *name) const
-{
-  int i = findAction(name);
-  return (i >= 0) ? iActions[i].alwaysOn : 0;
+  const char *action = luaL_checkstring(L, 2);
+  int i = findAction(action);
+  lua_pushinteger(L, (i >= 0) ? i + IDBASE : 0);
+  lua_pushboolean(L, (i >= 0) ? iActions[i].alwaysOn : 0);
+  return 2;
 }
 
 void AppUi::cmd(int id, int notification)
@@ -1080,6 +1140,13 @@ void AppUi::cmd(int id, int notification)
   } else if (ID_MOVETOLAYER_BASE <= id &&
 	     id < ID_MOVETOLAYER_BASE + int(iLayerNames.size())) {
     action(String("movetolayer-") + iLayerNames[id - ID_MOVETOLAYER_BASE]);
+  } else if (ID_GRIDSIZE_BASE <= id &&
+	     id < ID_GRIDSIZE_BASE + int(iComboContents[EUiGridSize].size())) {
+    luaSelector("gridsize", iComboContents[EUiGridSize][id - ID_GRIDSIZE_BASE]);
+  } else if (ID_ANGLESIZE_BASE <= id && id < ID_ANGLESIZE_BASE +
+	     int(iComboContents[EUiAngleSize].size())) {
+    luaSelector("anglesize",
+		iComboContents[EUiAngleSize][id - ID_ANGLESIZE_BASE]);
   } else if (id == ID_PATHVIEW) {
     luaShowPathStylePopup(Vector(iPathView->popupPos().x,
 				 iPathView->popupPos().y));
@@ -1093,17 +1160,16 @@ void AppUi::cmd(int id, int notification)
 
 static const char * const aboutText =
   "Ipe %d.%d.%d\n\n"
-  "Copyright (c) 1993-2015 Otfried Cheong\n\n"
+  "Copyright (c) 1993-2016 Otfried Cheong\n\n"
   "The extensible drawing editor Ipe creates figures "
   "in Postscript and PDF format, "
   "using LaTeX to format the text in the figures.\n"
   "Ipe relies on the following fine pieces of software:\n\n"
-  " * Pdflatex\n"
+  " * Pdftex, Xetex, or Luatex\n"
   " * %s (%d kB used)\n" // Lua
   " * The font rendering library %s\n"
   " * The rendering library Cairo %s\n"
-  " * The compression library zlib %s\n"
-  " * Some code from Xpdf\n\n"
+  " * The compression library zlib %s\n\n"
   "Ipe is released under the GNU Public License.\n"
   "See http://ipe.otfried.org for details.";
 
@@ -1145,10 +1211,10 @@ void AppUi::action(String name)
     int i = name.find("|");
     if (i >= 0)
       setCheckMark(name.left(i+1), name.substr(i+1));
-    if (name.left(5) == "mode_")
+    if (name.hasPrefix("mode_"))
       setCheckMark("mode_", name.substr(5));
-    if (name.left(4) == "snap" || name == "grid_visible" ||
-	name == "pretty_display" || name.left(7) == "toggle_")
+    if (name.hasPrefix("snap") || name == "grid_visible" ||
+	name == "pretty_display" || name.hasPrefix("toggle_"))
       setActionState(name.z(), !GetMenuState(hMenuBar, id, MF_CHECKED));
     luaAction(name);
   }
@@ -1181,17 +1247,22 @@ void AppUi::explain(const char *s, int t)
   sendMessage(hStatusBar, SB_SETTEXT, s);
 }
 
-void AppUi::setMouseIndicator(const char *s)
+void AppUi::setSnapIndicator(const char *s)
 {
   sendMessage(hStatusBar, SB_SETTEXT, s, 1);
 }
 
+void AppUi::setMouseIndicator(const char *s)
+{
+  sendMessage(hStatusBar, SB_SETTEXT, s, 2);
+}
+
 void AppUi::setZoom(double zoom)
 {
   char s[32];
   sprintf(s, "%dppi", int(72.0 * zoom));
   iCanvas->setZoom(zoom);
-  sendMessage(hStatusBar, SB_SETTEXT, s, 2);
+  sendMessage(hStatusBar, SB_SETTEXT, s, 3);
 }
 
 // --------------------------------------------------------------------
@@ -1282,8 +1353,7 @@ int AppUi::clipboard(lua_State *L)
       CloseClipboard();
       return 0;
     }
-    wchar_t *q = (wchar_t *) GlobalLock(hGlobal);
-    String s = Platform::wideToUtf8(q);
+    String s((wchar_t *) GlobalLock(hGlobal));
     GlobalUnlock(hGlobal);
     CloseClipboard();
     lua_pushstring(L, s.z());
@@ -1346,6 +1416,8 @@ LRESULT CALLBACK AppUi::wndProc(HWND hwnd, UINT message, WPARAM wParam,
     if (ui) {
       if (lParam == 2)
 	ui->populateTextStyleMenu();
+      else if (lParam == 3)
+	ui->populateSizeMenus();
       else if (lParam == 6)
 	ui->populateLayerMenus();
     }
