@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -49,31 +49,31 @@ public:
   AppUi(lua_State *L0, int model);
   ~AppUi();
 
-  virtual void action(String name);
-  virtual void setLayers(const Page *page, int view);
+  virtual void action(String name) override;
+  virtual void setLayers(const Page *page, int view) override;
 
-  virtual void setZoom(double zoom);
-  virtual void setActionsEnabled(bool mode);
-  virtual void setNumbers(String vno, bool vm, String pno, bool pm);
-  virtual void setNotes(String notes);
+  virtual void setZoom(double zoom) override;
+  virtual void setActionsEnabled(bool mode) override;
+  virtual void setNumbers(String vno, bool vm, String pno, bool pm) override;
+  virtual void setNotes(String notes) override;
 
-  virtual int actionId(const char *name) const;
-  virtual int actionAlwaysOn(const char *name) const;
-  virtual WINID windowId();
-  virtual void closeWindow();
-  virtual bool actionState(const char *name);
-  virtual void setActionState(const char *name, bool value);
-  virtual void setWindowCaption(bool mod, const char *s);
-  virtual void explain(const char *s, int t);
-  virtual void showWindow(int width, int height);
+  virtual WINID windowId() override;
+  virtual void closeWindow() override;
+  virtual bool actionState(const char *name) override;
+  virtual void setActionState(const char *name, bool value) override;
+  virtual void setWindowCaption(bool mod, const char *s) override;
+  virtual void explain(const char *s, int t) override;
+  virtual void showWindow(int width, int height) override;
 
-  virtual void setBookmarks(int no, const String *s);
-  virtual void setToolVisible(int m, bool vis);
+  virtual void setBookmarks(int no, const String *s) override;
+  virtual void setToolVisible(int m, bool vis) override;
+
+  virtual int actionInfo(lua_State *L) const override;
   virtual int pageSorter(lua_State *L, Document *doc,
-			 int width, int height, int thumbWidth);
+			 int width, int height, int thumbWidth) override;
 
-  virtual int clipboard(lua_State *L);
-  virtual int setClipboard(lua_State *L);
+  virtual int clipboard(lua_State *L) override;
+  virtual int setClipboard(lua_State *L) override;
 
   void toggleFullscreen();
 
@@ -82,23 +82,27 @@ private:
   void layoutChildren(bool resizeRebar);
   void createAction(String name, String tooltip, bool canWhileDrawing = false);
   int findAction(const char *name) const;
+  int actionId(const char *name) const;
   void addItem(HMENU menu, const char *title, const char *name);
-  virtual void addRootMenu(int id, const char *name);
-  virtual void addItem(int id, const char *title, const char *name);
-  virtual void startSubMenu(int id, const char *name);
-  virtual void addSubItem(const char *title, const char *name);
-  virtual MENUHANDLE endSubMenu();
-  virtual void setMouseIndicator(const char *s);
-  virtual void addCombo(int sel, String s);
-  virtual void resetCombos();
-  virtual void addComboColors(AttributeSeq &sym, AttributeSeq &abs);
-  virtual void setComboCurrent(int sel, int idx);
-  virtual void setCheckMark(String name, Attribute a);
-  virtual void setPathView(const AllAttributes &all, Cascade *sheet);
-  virtual void setButtonColor(int sel, Color color);
+  virtual void addRootMenu(int id, const char *name) override;
+  virtual void addItem(int id, const char *title, const char *name) override;
+  virtual void startSubMenu(int id, const char *name, int tag) override;
+  virtual void addSubItem(const char *title, const char *name) override;
+  virtual MENUHANDLE endSubMenu() override;
+  virtual void setSnapIndicator(const char *s) override;
+  virtual void setMouseIndicator(const char *s) override;
+  virtual void addCombo(int sel, String s) override;
+  virtual void resetCombos() override;
+  virtual void addComboColors(AttributeSeq &sym, AttributeSeq &abs) override;
+  virtual void setComboCurrent(int sel, int idx) override;
+  virtual void setCheckMark(String name, Attribute a) override;
+  virtual void setPathView(const AllAttributes &all, Cascade *sheet) override;
+  virtual void setButtonColor(int sel, Color color) override;
 
   void populateTextStyleMenu();
   void populateLayerMenus();
+  void populateSizeMenus();
+  void populateSizeMenu(HMENU h, int sel, int base);
   void cmd(int id, int notification);
   void setCheckMark(String name, String value);
   void aboutIpe();
@@ -111,6 +115,8 @@ private:
   HWND createButton(HINSTANCE hInst, int id,
 		    int flags = BS_BITMAP|BS_PUSHBUTTON);
 
+  inline int uiscale(int x) { return iUiScale * x / 100; }
+
 private:
   static LRESULT CALLBACK wndProc(HWND hwnd, UINT Message,
 				  WPARAM wParam, LPARAM lParam);
