@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -45,7 +45,9 @@ function MODEL:action(a)
   end
 end
 
-function MODEL:paction(a)
+function MODEL:paction(a1)
+  -- work-around for bug in Qt 5.5, to be replaced with something better:
+  local a = a1:gsub("&", "")
   -- print("MODEL:paction(" .. a .. ")")
   if a:sub(1,5) == "mode_" then
     self.mode = a:sub(6)
@@ -67,17 +69,15 @@ function MODEL:paction(a)
       for i,e in ipairs(MODEL.snapmodes) do
 	self.ui:setActionState(e, self.snap[e])
       end
-      self.ui:setSnap(self.snap)
+      self:setSnap()
     end
     if prefs.no_ink_highlight then
       self.ui:setFifiVisible(self.mode ~= "ink")
       self.ui:setSelectionVisible(self.mode ~= "ink")
     end
-    -- TODO: This seems redundant?
-    self.ui:setCursor()
   elseif a:sub(1,4) == "snap" then
     self.snap[a] = self.ui:actionState(a)
-    self.ui:setSnap(self.snap)
+    self:setSnap()
     self.ui:setFifiVisible(true)
   elseif a:sub(1,7) == "ipelet_" then
     self:action_ipelet(a)
@@ -107,8 +107,9 @@ function MODEL:selector(prop, value)
   if prop == "gridsize" or prop == "anglesize" then
     local abs = self.doc:sheets():find(prop, value)
     self.snap[prop] = abs
-    self.ui:setSnap(self.snap)
+    self:setSnap()
     self:setPage()
+    self.ui:setGridAngleSize(self.snap.gridsize, self.snap.anglesize)
     return
   end
   if prop == "markshape" then
@@ -608,8 +609,12 @@ end
 
 ----------------------------------------------------------------------
 
+function action_new_window()
+  MODEL.new()
+end
+
 function MODEL:action_new_window()
-  MODEL:new()
+  MODEL.new()
 end
 
 function MODEL:action_run_latex()
@@ -632,6 +637,13 @@ function MODEL:action_new()
   self:newDocument()
 end
 
+function action_open()
+  local s, f = ipeui.fileDialog(nil, "open", "Open file", filter_ipe)
+  if s then
+    MODEL.new(nil, s)
+  end
+end
+
 function MODEL:action_open()
   if not self:checkModified() then return end
   local s, f = ipeui.fileDialog(self.ui:win(), "open", "Open file", filter_ipe)
@@ -672,10 +684,11 @@ function MODEL:action_save_as()
     if not formatFromFileName(s) then
       s = s .. fmap[f]
     end
-    if ipe.fileExists(s) then
+    -- Cocoa handles the overwrite dialog itself
+    if config.toolkit ~= "cocoa" and ipe.fileExists(s) then
       local b = messageBox(self.ui:win(), "question",
 			   "File already exists!",
-			   "Do you wish to overwrite?<br>" .. s,
+			   "Do you wish to overwrite '" .. s .. "' ?",
 			   "okcancel")
       if b ~= 1 then
 	self.ui:explain("File not saved")
@@ -721,10 +734,10 @@ function MODEL:action_export_png()
   local s, f =
     ipeui.fileDialog(self.ui:win(), "save", "Export to PNG", filter_png, dir)
   if s then
-    if ipe.fileExists(s) then
+    if config.toolkit ~= "cocoa" and ipe.fileExists(s) then
       local b = messageBox(self.ui:win(), "question",
 			   "File already exists!",
-			   "Do you wish to overwrite?<br>" .. s,
+			   "Do you wish to overwrite?\n\n" .. s,
 			   "okcancel")
       if b ~= 1 then
 	self.ui:explain("Document not exported")
@@ -744,10 +757,10 @@ function MODEL:action_export_eps()
   local s, f =
     ipeui.fileDialog(self.ui:win(), "save", "Export to EPS", filter_eps, dir)
   if s then
-    if ipe.fileExists(s) then
+    if config.toolkit ~= "cocoa" and ipe.fileExists(s) then
       local b = messageBox(self.ui:win(), "question",
 			   "File already exists!",
-			   "Do you wish to overwrite?<br>" .. s,
+			   "Do you wish to overwrite?\n\n" .. s,
 			   "okcancel")
       if b ~= 1 then
 	self.ui:explain("Document not exported")
@@ -783,7 +796,7 @@ end
 
 ----------------------------------------------------------------------
 
-function MODEL:action_manual()
+function action_manual()
   local url = "file:///" .. config.docdir .. "/manual.html"
   if ipeui.startBrowser then
     if not ipeui.startBrowser(url) then
@@ -794,7 +807,11 @@ function MODEL:action_manual()
   end
 end
 
-function MODEL:action_show_configuration()
+function MODEL:action_manual()
+  action_manual()
+end
+
+function action_show_configuration(win)
   local s = ""
   s = s .. " * Lua code: " .. package.path
   s = s .. "\n * Style directories:\n  - " ..
@@ -805,31 +822,39 @@ function MODEL:action_show_configuration()
   s = s .. "\n * Save-as directory: " .. prefs.save_as_directory
   s = s .. "\n * Documentation: " .. config.docdir
   s = s .. "\n * Ipelets:\n   - " .. table.concat(config.ipeletDirs, "\n   - ")
+  s = s .. "\n * Latex program path: " .. config.latexpath
   s = s .. "\n * Latex directory: " .. config.latexdir
-  s = s .. "\n * Fontmap: " .. config.fontmap
   s = s .. "\n * Icons: " .. config.icons
   s = s .. "\n * External editor: " .. (prefs.external_editor or "none")
-  messageBox(self.ui:win(), "information", "Ipe configuration", s)
+  messageBox(win, "information", "Ipe configuration", s)
 end
 
-function MODEL:action_about_ipelets()
+function MODEL:action_show_configuration()
+  action_show_configuration(self.ui:win())
+end
+
+function action_about_ipelets(win)
   local s = ""
   for i,v in ipairs(ipelets) do
-    if v.label then
+    if v.label and v.about then
       s = s .. "### " .. v.label .. " ###\n\n" .. v.about .. "\n"
     end
   end
-  local d = ipeui.Dialog(self.ui:win(), "Ipe: About the ipelets")
+  local d = ipeui.Dialog(win, "Ipe: About the ipelets")
   d:add("text", "text", { read_only=true }, 1, 1)
   d:set("text", s)
   d:addButton("ok", "Ok", "accept")
   d:execute(prefs.latexlog_size)
 end
 
+function MODEL:action_about_ipelets()
+  action_about_ipelets(self.ui:win())
+end
+
 function MODEL:action_keyboard()
   local s = prefs.keyboard
   if s then
-    if ipeui.startBrowser then
+    if config.platform == "win" then
       ipeui.startBrowser(s)
     else
       os.execute(s)
@@ -877,7 +902,7 @@ function MODEL:action_set_origin_snap()
   self.snap.snapangle = true
   self.snap.with_axes = true
   self.ui:setActionState("snapangle", true)
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:setFifiVisible(true)
   self.ui:update()
 end
@@ -886,7 +911,7 @@ function MODEL:action_hide_axes()
   self.snap.with_axes = false
   self.snap.snapangle = false
   self.ui:setActionState("snapangle", false)
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:update()
 end
 
@@ -899,7 +924,7 @@ end
 
 function MODEL:action_reset_direction()
   self.snap.orientation = 0
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:update()
 end
 
@@ -919,7 +944,7 @@ end
 
 function MODEL:action_set_line()
   if self:setLine() then
-    self.ui:setSnap(self.snap)
+    self:setSnap()
   end
 end
 
@@ -928,7 +953,7 @@ function MODEL:action_set_line_snap()
     self.snap.snapangle = true
     self.ui:setFifiVisible(true)
     self.ui:setActionState("snapangle", true)
-    self.ui:setSnap(self.snap)
+    self:setSnap()
   end
 end
 
@@ -1009,7 +1034,7 @@ end
 
 function MODEL:action_grid_visible()
   self.snap.grid_visible = self.ui:actionState("grid_visible");
-  self.ui:setSnap(self.snap)
+  self:setSnap()
   self.ui:update()
 end
 
@@ -1032,16 +1057,14 @@ function MODEL:action_zoom_out()
   self.ui:update()
 end
 
-function MODEL:wheel_zoom(delta)
+function MODEL:wheel_zoom(delta, precise)
+  print("wheel_zoom", delta, precise)
   local origin = self.ui:unsnappedPos()
   local zoom = self.ui:zoom()
   local offset = zoom * (self.ui:pan() - origin)
-  local nzoom
-  if delta > 0 then
-      nzoom = prefs.wheel_zoom_factor * zoom
-  else
-    nzoom = zoom / prefs.wheel_zoom_factor
-  end
+  local base = prefs.wheel_zoom_factor
+  if not precise then base = prefs.wheel_other_factor end
+  local nzoom = zoom * base ^ delta
   if nzoom > prefs.max_zoom then nzoom = prefs.max_zoom end
   if nzoom < prefs.min_zoom then nzoom = prefs.min_zoom end
   self.ui:setZoom(nzoom)
@@ -1049,14 +1072,6 @@ function MODEL:wheel_zoom(delta)
   self.ui:update()
 end
 
-function MODEL:action_wheel_zoom_out()
-  self:wheel_zoom(-1)
-end
-
-function MODEL:action_wheel_zoom_in()
-  self:wheel_zoom(1)
-end
-
 -- Change resolution to 72 dpi and maximize interesting visible area.
 -- As suggested by Rene:
 
@@ -1286,7 +1301,7 @@ function MODEL:action_edit_effects()
   local d = ipeui.Dialog(self.ui:win(), "Edit view effect")
   local label = "Choose effect for view " .. self.vno
   d:add("label", "label", { label=label }, 1, 1)
-  d:add("combo", "combo", list, 2, 1)
+  d:add("combo", "combo", list, 0, 1)
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:set("combo", indexOf(effect, list))
@@ -1447,20 +1462,20 @@ end
 function MODEL:action_edit_title()
   local d = ipeui.Dialog(self.ui:win(), "Ipe: Edit page title and sections")
   d:add("label1", "label", { label="Page title"}, 1, 1, 1, 4)
-  d:add("title", "text", {}, 2, 1, 1, 4)
-  d:add("label2", "label", { label="Sections"}, 3, 1, 1, 4)
-  d:add("label3", "label", { label="Section:" }, 4, 1)
+  d:add("title", "text", {}, 0, 1, 1, 4)
+  d:add("label2", "label", { label="Sections"}, 0, 1, 1, 4)
+  d:add("label3", "label", { label="Section:" }, 0, 1)
   d:add("tsection", "checkbox",
 	{ label="Use &title",
 	  action=function (d) update_dialog(d, "section") end
-	} , 4, 2)
-  d:add("section", "input", {}, 5, 2, 1, 3)
-  d:add("label4", "label", { label="Subsection:" }, 6, 1)
+	} , -1, 2)
+  d:add("section", "input", {}, 0, 2, 1, 3)
+  d:add("label4", "label", { label="Subsection:" }, 0, 1)
   d:add("tsubsection", "checkbox",
 	{ label="Use titl&e",
 	  action=function (d) update_dialog(d, "subsection") end
-	} , 6, 2)
-  d:add("subsection", "input", {}, 7, 2, 1, 3)
+	} , -1, 2)
+  d:add("subsection", "input", {}, 0, 2, 1, 3)
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("row", 2, 1)
@@ -1511,10 +1526,11 @@ end
 function MODEL:action_edit_notes()
   local d = ipeui.Dialog(self.ui:win(), "Ipe: Edit page notes")
   d:add("label1", "label", { label="Page notes"}, 1, 1)
-  d:add("notes", "text", {}, 2, 1)
+  d:add("notes", "text", {}, 0, 1)
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("row", 2, 1)
+  d:setStretch("column", 1, 1)
   -- setup original values
   local n = self:page():notes()
   d:set("notes", n)
@@ -1573,6 +1589,7 @@ function MODEL:action_page_sorter()
     self:warning("You cannot delete all pages of the document")
     return
   end
+  printTable(arr)
   -- compute reverse 'permutation'
   local rev = {}
   for i = 1,#self.doc do rev[i] = 0 end
@@ -1594,10 +1611,12 @@ end
 
 function MODEL:action_toggle_notes()
   self.ui:showTool("notes", self.ui:actionState("toggle_notes"))
+  self.ui:setNotes(self:page():notes())
 end
 
 function MODEL:action_toggle_bookmarks()
   self.ui:showTool("bookmarks", self.ui:actionState("toggle_bookmarks"))
+  self:setBookmarks()
 end
 
 ----------------------------------------------------------------------
@@ -2101,6 +2120,9 @@ local function sheets_edit(d, dd)
   if not i or dd.list[i]:isStandard() then return end
   local data = dd.list[i]:xml(true)
   local fname = os.tmpname()
+  if prefs.editable_textfile then
+    fname = prefs.editable_textfile
+  end
   local f = io.open(fname, "w")
   f:write(data)
   f:close()
@@ -2229,31 +2251,34 @@ end
 ----------------------------------------------------------------------
 
 function MODEL:action_document_properties()
+  local engines = { "default", "pdftex", "xetex" } -- "luatex" }
   local p = self.doc:properties()
   local d = ipeui.Dialog(self.ui:win(), "Ipe document properties")
   d:add("label1", "label", { label="Title"}, 1, 1)
-  d:add("title", "input", {}, 1, 2, 1, 6)
-  d:add("label2", "label", { label="Author"}, 2, 1)
-  d:add("author", "input", {}, 2, 2, 1, 6)
-  d:add("label3", "label", { label="Subject"}, 3, 1)
-  d:add("subject", "input", {}, 3, 2, 1, 6)
-  d:add("label4", "label", { label="Keywords"}, 4, 1)
-  d:add("keywords", "input", {}, 4, 2, 1, 6)
-  d:add("label5", "label", { label="Latex preamble"}, 5, 1)
-  d:add("preamble", "text", {}, 5, 2, 2, 6)
-  d:add("label6", "label", { label="Page mode"}, 7, 1)
-  d:add("fullscreen", "checkbox", { label="&Full screen"}, 7, 2)
-  d:add("numberpages", "checkbox", { label="&Number pages"}, 7, 3)
-  d:add("label7", "label", { label="Created"}, 8, 1)
-  d:add("created", "label",  { label="" }, 8, 2)
-  d:add("label8", "label", { label="Modified"}, 9, 1)
-  d:add("modified", "label",  { label="" }, 9, 2)
-  d:add("label9", "label", { label="Creator" }, 10, 1)
-  d:add("creator", "label",  { label="" }, 10, 2)
+  d:add("title", "input", {}, -1, 2, 1, 6)
+  d:add("label2", "label", { label="Author"}, 0, 1)
+  d:add("author", "input", {}, -1, 2, 1, 6)
+  d:add("label3", "label", { label="Subject"}, 0, 1)
+  d:add("subject", "input", {}, -1, 2, 1, 6)
+  d:add("label4", "label", { label="Keywords"}, 0, 1)
+  d:add("keywords", "input", {}, -1, 2, 1, 6)
+  d:add("label5", "label", { label="Latex engine"}, 0, 1)
+  d:add("tex", "combo", engines, -1, 2, 1, 6)
+  d:add("label6", "label", { label="Latex preamble"}, 0, 1)
+  d:add("preamble", "text", {}, -1, 2, 2, 6)
+  d:add("label7", "label", { label="Page mode"}, 8, 1)
+  d:add("fullscreen", "checkbox", { label="&Full screen"}, -1, 2)
+  d:add("numberpages", "checkbox", { label="&Number pages"}, -1, 3)
+  d:add("label8", "label", { label="Created"}, 0, 1)
+  d:add("created", "label",  { label="" }, -1, 2)
+  d:add("label9", "label", { label="Modified"}, 0, 1)
+  d:add("modified", "label",  { label="" }, -1, 2)
+  d:add("label10", "label", { label="Creator" }, 0, 1)
+  d:add("creator", "label",  { label="" }, -1, 2)
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("column", 5, 1)
-  d:setStretch("row", 6, 1)
+  d:setStretch("row", 7, 1)
   for n in pairs(p) do d:set(n, p[n]) end
   if not d:execute() then return end
 
@@ -2264,6 +2289,8 @@ function MODEL:action_document_properties()
   for n in pairs(p) do
     if n == "creator" or n == "created" or n == "modified" then
       t.final[n] = p[n]
+    elseif n == "tex" then
+      t.final[n] = engines[d:get(n)]
     else
       t.final[n] = d:get(n)
     end
@@ -2275,6 +2302,30 @@ end
 
 ----------------------------------------------------------------------
 
+function MODEL:saction_set_link_url()
+  local p = self:page()
+  local prim = p:primarySelection()
+  local obj = p[prim]
+  assert(obj:type() == "group")
+  local url = self:getString("Enter link destination URL", nil, obj:text())
+  if not url then return end
+  if url:match("^%s*$)") then url = "" end
+  local t = { label="set link URL",
+	      pno=self.pno,
+	      vno=self.vno,
+	      primary=prim,
+	      original=obj:clone(),
+	      url=url,
+	    }
+  t.undo = function (t, doc)
+	     doc[t.pno]:replace(t.primary, t.original)
+	   end
+  t.redo = function (t, doc)
+	     doc[t.pno][t.primary]:setText(t.url)
+	   end
+  self:register(t)
+end
+
 function MODEL:saction_remove_clipping()
   local p = self:page()
   local prim = p:primarySelection()
