@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 MODEL = {}
 MODEL.__index = MODEL
 
-MODEL.snapmodes = { "snapvtx", "snapbd", "snapint", "snapgrid",
+MODEL.snapmodes = { "snapvtx", "snapctl", "snapbd", "snapint", "snapgrid",
 		    "snapangle", "snapauto"}
 
 function MODEL:new(fname)
@@ -74,6 +74,7 @@ function MODEL:init(fname)
 
   self.snap = {
     snapvtx = prefs.snap.vertex,
+    snapctl = prefs.snap.ctrlpoints,
     snapbd = prefs.snap.boundary,
     snapint = prefs.snap.intersection,
     snapgrid = prefs.snap.grid,
@@ -94,6 +95,23 @@ function MODEL:init(fname)
   self.pristine = false
   self.first_show = true
 
+  self.auto_latex = prefs.auto_run_latex
+  self.ui:setActionState("auto_latex", self.auto_latex)
+
+  self.mode = "select"
+  self.ui:setActionState("mode_select", true)
+  self.ui:setActionState("grid_visible", self.snap.grid_visible)
+
+  for i,e in ipairs(MODEL.snapmodes) do
+    self.ui:setActionState(e, self.snap[e])
+  end
+  self:setSnap()
+  self.ui:setFifiVisible(true)
+
+  for dock, vis in pairs(prefs.tools_visible) do
+    self.ui:showTool(dock, vis)
+  end
+
   local err = nil
   if fname then
     if ipe.fileExists(fname) then
@@ -109,27 +127,6 @@ function MODEL:init(fname)
     self.pristine = true
   end
 
-  self.mode = "select"
-  self.ui:setActionState("mode_select", true)
-  self.ui:setActionState("grid_visible", self.snap.grid_visible)
-
-  for i,e in ipairs(MODEL.snapmodes) do
-    self.ui:setActionState(e, self.snap[e])
-  end
-  self.ui:setSnap(self.snap)
-  self.ui:setFifiVisible(true)
-
-  self.auto_latex = prefs.auto_run_latex
-  self.ui:setActionState("auto_latex", self.auto_latex)
-
-  if self.auto_latex then
-    self:runLatex()
-  end
-
-  for i, dock in ipairs(prefs.hide_docks) do
-    self.ui:showTool(dock, false)
-  end
-
   if prefs.autosave_interval then
     self.timer = ipeui.Timer(self, "autosave")
     self.timer:setInterval(1000 * prefs.autosave_interval) -- millisecs
@@ -249,6 +246,25 @@ function MODEL:setCaption()
   self.ui:setWindowTitle(self:isModified(), s)
 end
 
+function MODEL:setSnap()
+  local function ind(is_on, s, letter)
+    if is_on then return s .. letter
+    else return s .. "-" end
+  end
+
+  self.ui:setSnap(self.snap)
+  local s = ind(self.snap.snapvtx, "", "v")
+  s = ind(self.snap.snapctl, s, "c")
+  s = ind(self.snap.snapbd, s, "b")
+  s = ind(self.snap.snapint, s, "x")
+  s = ind(self.snap.snapgrid, s, "+")
+  s = ind(self.snap.snapangle, s, "r")
+  s = ind(self.snap.snapauto, s, "a")
+  s = s .. " " .. self.snap.gridsize
+  s = s .. " " .. self.snap.anglesize
+  self.ui:setSnapIndicator(s)
+end
+
 -- show a warning messageBox
 function MODEL:warning(text, details)
   messageBox(self.ui:win(), "warning", text, details)
@@ -444,7 +460,7 @@ end
 function MODEL:runLatex()
   local success, errmsg, result, log = self.doc:runLatex()
   if success then
-    self.ui:setFontPool(self.doc)
+    self.ui:setResources(self.doc)
     self.ui:update()
     return true
   elseif result == "latex" then
@@ -490,6 +506,9 @@ function MODEL:newDocument()
     local sheet = assert(ipe.Sheet(w))
     self.doc:sheets():insert(1, sheet)
   end
+  local p = self.doc:properties()
+  p.tex = prefs.tex_engine
+  self.doc:setProperties(p)
 
   self:setPage()
 
@@ -498,7 +517,7 @@ function MODEL:newDocument()
   self:resetGridSize()
 
   self:action_normal_size()
-  self.ui:setSnap(self.snap)
+  self:setSnap()
 
   self.ui:explain("New document")
 end
@@ -530,10 +549,14 @@ function MODEL:tryLoadDocument(fname)
     self:resetGridSize()
 
     self:action_normal_size()
-    self.ui:setSnap(self.snap)
+    self:setSnap()
 
     self.ui:explain("Document '" .. fname .. "' loaded")
 
+    if self.auto_latex then
+      self:runLatex()
+    end
+
     local syms = self.doc:checkStyle()
     if #syms > 0 then
       self:warning("The document contains symbolic attributes " ..
