@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,6 +37,10 @@ prefs = {}
 -- directory.
 prefs.styles = { "basic" }
 
+-- The TeX engine for new documents
+-- Possible values are "default", "pdftex", "xetex", and "luatex"
+prefs.tex_engine = "default"
+
 -- Interval for autosaving in seconds
 -- set to nil to disable autosaving
 prefs.autosave_interval = 600 -- 10 minutes
@@ -51,6 +55,18 @@ else
   prefs.autosave_filename = home .. "/%s.autosave"
 end
 
+-- Should Ipe show the Developer menu
+-- (only useful if you develop ipelets or want to customize Ipe)
+prefs.developer = false
+
+-- Should Ipe terminate when the last window was closed?
+-- (Currently only on OS X)
+prefs.terminate_on_close = true
+
+-- Enable continuous spell-checking in text fields by default?
+-- (Currently only on OS X)
+prefs.spell_check = true
+
 -- External editor for editing text objects
 -- must contain '%s' for the temporary filename
 -- set this to nil to hide the "Editor" button on text dialogs
@@ -77,19 +93,24 @@ else
   prefs.delete_key = "\127"
 end
 
--- Where the tools should be put: Left or right?
--- (currently not used on Windows)
-prefs.docks = {
+-- Where the tools should be placed (left or right)
+-- and which ones should be displayed on start-up
+-- tools_placement is only used with Qt toolkit
+prefs.tools_placement = {
   properties = "left",
   bookmarks = "right",
   notes = "right",
-  layers = "left",
+  layers = "left"
+}
+prefs.tools_visible = {
+  properties = true,
+  bookmarks = false,
+  notes = false,
+  layers = true
 }
 
--- Which docks should be hidden when Ipe starts?
-prefs.hide_docks = {
-  "bookmarks", "notes"
-}
+-- initial ordering of the toolbars (windows only)
+prefs.toolbar_order = { "snap", "grid", "angle", "edit", "mode" }
 
 -- Should LaTeX be run automatically every time text has changed?
 prefs.auto_run_latex = true
@@ -100,9 +121,18 @@ prefs.auto_external_editor = nil
 -- Should closing the external editor immediately close the editing dialog?
 prefs.editor_closes_dialog = nil
 
+-- If the user interface is too small, scale it here (Windows and Qt only)
+-- 100 is the original size, 150 is 50% larger
+prefs.ui_scale = 100
+-- The following influences the size of the toolbar buttons
+prefs.toolbar_scale = 100
+
 -- Minimum size of dialog window for creating/editing text objects
 prefs.editor_size = { 0, 0 }
 
+-- Width of Notes and Bookmarks tools (Windows only)
+prefs.width_notes_bookmarks = 200
+
 -- Size of main window at startup
 prefs.window_size = { 1080, 600 }
 
@@ -119,6 +149,10 @@ prefs.thumbnail_width = 200
 prefs.canvas_style = {
   paper_color = { r = 1.0, g = 1.0, b = 1.0 },  -- white
   -- paper_color = { r = 1.0, g = 1.0, b = 0.5 }  -- classic Ipe 6 yellow
+  -- color of the primary selection:
+  primary_color = { r = 1.0, g = 0.0, b = 0.0 },
+  -- color of the secondary selection:
+  secondary_color = { r = 1.0, g = 0.0, b = 1.0 },
   -- classic grid uses dots instead of lines
   classic_grid = false,
   -- line width of grid lines
@@ -135,7 +169,7 @@ prefs.grid_visible = true
 
 -- Snap modes active when Ipe starts:
 prefs.snap = { vertex = false,
-	       ctrlpoints = true,
+	       ctrlpoints = false,
 	       boundary = false,
 	       intersection = false,
 	       grid = true,
@@ -148,12 +182,6 @@ prefs.angle_size = 45    -- degrees
 
 -- Resolution settings
 
--- Do you use a Retina display on Mac OS X?
-prefs.retina = false
-
--- If you set prefs.retina, you should also double the following four
--- parameters that are expressed in pixels:
-
 -- When you select "normal size" in Ipe, this is the pixels
 -- per inch you want to see.
 prefs.normal_resolution = 72
@@ -167,7 +195,14 @@ prefs.close_distance = 48
 
 -- Zoom factors, minimal and maximal possible zoom
 prefs.zoom_factor = 1.3
-prefs.wheel_zoom_factor = 1.3  -- used when zooming by scroll wheel
+-- when zooming by scroll wheel or
+-- by scroll gesture on OS X
+if config.toolkit == "cocoa" then
+  prefs.wheel_zoom_factor = 1.1
+  prefs.wheel_other_factor = 1.3 -- used on some external mice
+else
+  prefs.wheel_zoom_factor = 1.3
+end
 prefs.min_zoom = 0.1
 prefs.max_zoom = 100
 
@@ -178,30 +213,31 @@ prefs.text_transformable = false
 -- the check box "section: use title" is checked automatically.
 prefs.automatic_use_title = false
 
--- How to start browser to show Ipe manual
--- (not used on Windows)
-if config.platform == "apple" then
-  prefs.browser = "open %s"
-else
-  -- On Linux, there are various options depending on your Desktop:
-  -- Gnome 2: gnome-open
-  -- prefs.browser = "gnome-open %s"
-  -- Gnome 3: gvfs-open
-  prefs.browser = "gvfs-open %s"
-  -- KDE: kde-open
-  -- prefs.browser = "kde-open %s"
-  -- XFCE: exo-open
-  -- prefs.browser = "exo-open %s"
-  -- "sensible-browser" starts a new browser
-  -- (which is not nice, ideally we'd like a tab in an open browser window)
-  -- prefs.browser = "sensible-browser %s &"
-end
+-- How to start browser to show Ipe manual:
+-- (not used with Windows or Cocoa toolkit)
+
+-- On Linux, there are various options depending on your Desktop:
+-- Gnome 2: gnome-open
+-- prefs.browser = "gnome-open %s"
+-- Gnome 3: gvfs-open
+prefs.browser = "gvfs-open %s"
+-- KDE: kde-open
+-- prefs.browser = "kde-open %s"
+-- XFCE: exo-open
+-- prefs.browser = "exo-open %s"
+-- "sensible-browser" starts a new browser
+  local engines = { "default", "pdftex", "xetex" } -- "luatex" }
+  local engines = { "default", "pdftex", "xetex" } -- "luatex" }
+-- (which is not nice, ideally we'd like a tab in an open browser window)
+-- prefs.browser = "sensible-browser %s &"
 
 -- How to start onscreen keyboard
--- prefs.keyboard = "onboard &"
 if config.platform == "win" then
   prefs.keyboard = "tabtip.exe"
+elseif config.platform == "apple" then
+  prefs.keyboard = "open -a KeyboardViewer -n"
 else
+  -- On Linux, you could use: prefs.keyboard = "onboard &"
   prefs.keyboard = nil
 end
 
@@ -248,8 +284,4 @@ else
   prefs.basename_pattern = "/([^/]+)$"
 end
 
--- Should Ipe show the Developer menu
--- (only useful if you develop ipelets or want to customize Ipe)
-prefs.developer = false
-
 ----------------------------------------------------------------------
