@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -172,10 +172,16 @@ function MODEL:properties_group(obj, m)
   m:add("comment", "Group object")
   local s = string.format("%d elements", obj:count())
   local clip = obj:clip()
+  local url = obj:text()
   if clip then
     s = s .. ", with clipping"
   end
   m:add("comment", s)
+  if url == "" then
+    m:add("action_set_link_url", "Set link URL")
+  else
+    m:add("action_set_link_url", "URL: " .. url)
+  end
   self:insertBasic(m, obj)
   if clip then
     m:add("action_remove_clipping", "Remove clipping")
@@ -266,6 +272,7 @@ function MODEL:properties_path(obj, m)
   if #shape > 1 then
     m:add("action_decompose", "Decompose path")
   end
+  m:add("action_pick_properties", "Pick properties")
   m:add("action_edit_as_xml", "Edit as XML")
   m:add("action_edit", "Edit path")
 end
@@ -303,6 +310,7 @@ function MODEL:properties_text(obj, m)
   if minipage then
     m:add("action_change_width", "Change width")
   end
+  m:add("action_pick_properties", "Pick properties")
   m:add("action_edit_as_xml", "Edit as XML")
   m:add("action_edit", "Edit text")
 end
@@ -335,6 +343,7 @@ function MODEL:properties_reference(obj, m)
   m:add("symbolsize", "Size: " .. ss, sizes, nil, ss)
   local pen = obj:get("pen")
   m:add("pen", "Pen: " .. pen, pens, nil, pen)
+  m:add("action_pick_properties", "Pick properties")
   m:add("action_edit_as_xml", "Edit as XML")
 end
 
@@ -468,3 +477,148 @@ end
 
 --------------------------------------------------------------------
 
+function MODEL:saction_pick_properties()
+  local p = self:page()
+  local prim = p:primarySelection()
+  local obj = p[prim]
+  self["pick_properties_" .. obj:type()](self, obj)
+  self.ui:setAttributes(self.doc:sheets(), self.attributes)
+end
+
+function MODEL:pick_properties_group(obj)
+  -- nothing
+end
+
+function MODEL:pick_properties_text(obj)
+  local a = self.attributes
+  a.stroke = obj:get("stroke")
+  a.textsize = obj:get("textsize")
+  if obj:get("minipage") then
+    a.textstyle = obj:get("textstyle")
+  else
+    a.horizontalalignment = obj:get("horizontalalignment")
+  end
+  a.verticalalignment = obj:get("verticalalignment")
+  a.opacity = obj:get("opacity")
+end
+
+function MODEL:pick_properties_path(obj)
+  local a = self.attributes
+  a.pathmode = obj:get("pathmode")
+  if a.pathmode ~= "filled" then
+    a.stroke = obj:get("stroke")
+  end
+  if a.pathmode ~= "stroked" then
+    a.fill = obj:get("fill")
+  end
+  a.pen = obj:get("pen")
+  a.dashstyle = obj:get("dashstyle")
+  a.farrow = obj:get("farrow")
+  a.rarrow = obj:get("rarrow")
+  if a.farrow then
+    a.farrowsize = obj:get("farrowsize")
+    a.farrowshape = obj:get("farrowshape")
+  end
+  if a.rarrow then
+    a.rarrowsize = obj:get("rarrowsize")
+    a.rarrowshape = obj:get("rarrowshape")
+  end
+  a.linecap = obj:get("linecap")
+  a.linejoin = obj:get("linejoin")
+  a.fillrule = obj:get("fillrule")
+  a.tiling = obj:get("tiling")
+  a.gradient = obj:get("gradient")
+  a.opacity = obj:get("opacity")
+end
+
+function MODEL:pick_properties_image(obj)
+  -- nothing
+end
+
+function MODEL:pick_properties_reference(obj)
+  local a = self.attributes
+  a.stroke = obj:get("stroke")
+  a.fill = obj:get("fill")
+  a.pen = obj:get("pen")
+  a.symbolsize = obj:get("symbolsize")
+end
+
+----------------------------------------------------------------------
+
+function MODEL:saction_apply_properties()
+  local a = { }
+  for k, v in pairs(self.attributes) do
+    a[k] = v
+  end
+  local t = { label="apply properties",
+	      pno = self.pno,
+	      vno = self.vno,
+	      attributes=a,
+	      selection=self:selection(),
+	      original=self:page():clone(),
+	      undo=revertOriginal,
+	    }
+  t.redo = function (t, doc)
+	     local p = doc[t.pno]
+	     for i,obj,sel,layer in p:objects() do
+	       local ty = obj:type()
+	       if ty == "text" then
+		 apply_properties_text(obj, t.attributes)
+	       elseif ty == "path" then
+		 apply_properties_path(obj, t.attributes)
+	       elseif ty == "reference" then
+		 apply_properties_reference(obj, t.attributes)
+	       end
+	     end
+	   end
+  self:register(t)
+end
+
+function apply_properties_text(obj, a)
+  obj:set("stroke", a.stroke)
+  obj:set("textsize", a.textsize)
+  if obj:get("minipage") then
+    obj:set("textstyle", a.textstyle)
+  else
+    obj:set("horizontalalignment", a.horizontalalignment)
+  end
+  obj:set("verticalalignment", a.verticalalignment)
+  obj:set("opacity", a.opacity)
+end
+
+function apply_properties_path(obj, a)
+  obj:set("pathmode", a.pathmode)
+  if a.pathmode ~= "filled" then
+    obj:set("stroke", a.stroke)
+  end
+  if a.pathmode ~= "stroked" then
+    obj:set("fill", a.fill)
+  end
+  obj:set("pen", a.pen)
+  obj:set("dashstyle", a.dashstyle)
+  obj:set("farrow", a.farrow)
+  obj:set("rarrow", a.rarrow)
+  if a.farrow then
+    obj:set("farrowsize", a.farrowsize)
+    obj:set("farrowshape", a.farrowshape)
+  end
+  if a.rarrow then
+    obj:set("rarrowsize", a.rarrowsize)
+    obj:set("rarrowshape", a.rarrowshape)
+  end
+  obj:set("linecap", a.linecap)
+  obj:set("linejoin", a.linejoin)
+  obj:set("fillrule", a.fillrule)
+  obj:set("tiling", a.tiling)
+  obj:set("gradient", a.gradient)
+  obj:set("opacity", a.opacity)
+end
+
+function apply_properties_reference(obj, a)
+  obj:set("stroke", a.stroke)
+  obj:set("fill", a.fill)
+  obj:set("pen", a.pen)
+  obj:set("symbolsize", a.symbolsize)
+end
+
+----------------------------------------------------------------------
