@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -28,7 +28,9 @@
 
 --]]
 
--- Multiple shortcuts for an action are possible, see 'delete'.
+-- on OSX, Ctrl means the Command key (for platform compatibility)
+-- if you really want the Control key, write "Control"
+-- you can also write "Command" for the Command key for clarity.
 
 shortcuts = {
   new_window = nil,
@@ -45,7 +47,7 @@ shortcuts = {
   paste = nil,
   paste_at_cursor = "Ctrl+V",
   cut = "Ctrl+X",
-  delete = { "delete", "backspace" },
+  delete = "delete",
   group = "Ctrl+G",
   ungroup = "Ctrl+U",
   front = "Ctrl+F",
@@ -57,6 +59,8 @@ shortcuts = {
   duplicate = "D",
   select_all = "Ctrl+A",
   join_paths = "Ctrl+J",
+  pick_properties = nil,
+  apply_properties = nil,
   insert_text_box = "F10",
   edit = "Ctrl+E",
   edit_as_xml = nil,
@@ -153,6 +157,11 @@ shortcuts = {
   ipelet_2_symbols = "Alt+Y", -- Use current symbol
 }
 
+if config.platform == "apple" then
+  shortcuts.delete = "backspace"
+  shortcuts.fullscreen = "Control+Command+F"
+end
+
 -- These are the shortcuts for the lines/polylines, polygons and splines
 -- functions.
 -- Contrary to the above, global shortcuts, the following rules apply:
