@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -78,6 +78,8 @@ static void push_modifiers(lua_State *L, int button)
   lua_setfield(L, -2, "shift");
   lua_pushboolean(L, (button & CanvasBase::EControl));
   lua_setfield(L, -2, "control");
+  lua_pushboolean(L, (button & CanvasBase::ECommand));
+  lua_setfield(L, -2, "command");
   lua_pushboolean(L, (button & CanvasBase::EAlt));
   lua_setfield(L, -2, "alt");
   lua_pushboolean(L, (button & CanvasBase::EMeta));
@@ -135,7 +137,8 @@ bool LuaTool::key(String text, int modifiers)
   push_modifiers(L, modifiers);
   lua_State *L0 = L; // need to save L since
   lua_call(L, 3, 1); // this may delete tool
-  return lua_toboolean(L0, 1);
+  bool used = lua_toboolean(L0, -1);
+  return used;
 }
 
 // --------------------------------------------------------------------
