@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ CairoPainter::CairoPainter(const Cascade *sheet, Fonts *fonts,
     iCairo(cc), iZoom(zoom), iPretty(pretty)
 {
   iDimmed = false;
-  iFont = 0;
+  iFont = nullptr;
 }
 
 void CairoPainter::doPush()
@@ -336,7 +336,7 @@ void CairoPainter::doDrawBitmap(Bitmap bitmap)
   matrix.y0 = tf.a[5];
   cairo_transform(iCairo, &matrix);
   cairo_set_source_surface(iCairo, image, 0, 0);
-  cairo_pattern_set_filter(cairo_get_source(iCairo), CAIRO_FILTER_BEST);
+  cairo_pattern_set_filter(cairo_get_source(iCairo), CAIRO_FILTER_FAST);
   cairo_paint(iCairo);
   cairo_restore(iCairo);
 }
@@ -403,7 +403,9 @@ void CairoPainter::doDrawText(const Text *text)
       }
     }
   } else {
-    transform(Matrix(xf->iStretch, 0, 0, xf->iStretch, 0, 0));
+    transform(Matrix(xf->iStretch, 0, 0, xf->iStretch,
+		     -xf->iBBox.bottomLeft().x,
+		     -xf->iBBox.bottomLeft().y));
     execute(xf->iStream);
   }
 }
@@ -441,7 +443,7 @@ void CairoPainter::execute(const Buffer &buffer)
 {
   BufferSource source(buffer);
   PdfParser parser(source);
-  iFont = 0;
+  iFont = nullptr;
   iFillRgb[0] = iFillRgb[1] = iFillRgb[2] = 0.0;
   iStrokeRgb[0] = iStrokeRgb[1] = iStrokeRgb[2] = 0.0;
   while (!parser.eos()) {
@@ -642,7 +644,7 @@ void CairoPainter::opre()
 
 void CairoPainter::opBT()
 {
-  iFont = 0;
+  iFont = nullptr;
   iTextPos = iTextLinePos = Vector::ZERO;
 }
 
@@ -677,12 +679,17 @@ void CairoPainter::opTJ()
     if (obj->number()) {
       iTextPos.x -= 0.001 * iFontSize * obj->number()->value();
     } else if (obj->string()) {
-      String s = obj->string()->value();
-      for (int j = 0; j < s.size(); ++j) {
-	uchar ch = s[j];
+      String s = obj->string()->decode();
+      bool ucs = (iFont->type() == FontType::CIDType0 ||
+		  iFont->type() == FontType::CIDType2);
+      int j = 0;
+      while (j < s.size()) {
+	int ch = uchar(s[j++]);
+	if (ucs && j < s.size())
+	  ch = (ch << 8) | uchar(s[j++]);
 	Vector pt = matrix() * iTextPos;
 	cairo_glyph_t g;
-	g.index = iFont->getGlyph(ch);
+	g.index = iFont->glyphIndex(ch);
 	g.x = pt.x;
 	g.y = pt.y;
 	glyphs.push_back(g);
