@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 
 #include "ipebase.h"
 #include "ipegeo.h"
-#include "ipefontpool.h"
+#include "iperesources.h"
 
 #include <cairo.h>
 
@@ -44,37 +44,53 @@ struct FT_FaceRec_;
 
 namespace ipe {
 
+  class PdfResources;
+
+  enum class FontType { Type1, Truetype, CIDType0, CIDType2, Unsupported };
+
   class Face {
   public:
-    Face(int id, const Font &font);
-    ~Face();
-    inline bool matches(int id) const { return id == iId; }
-    inline Font::TType type() const { return iType; }
-    inline int width(int ch) const { return iWidth[ch]; }
-    inline cairo_font_face_t *cairoFont() { return iCairoFont; }
-    int getGlyph(int ch);
+    Face(int id, const PdfResources *resources) noexcept;
+    ~Face() noexcept;
+    inline bool matches(int id) const noexcept { return id == iId; }
+    inline FontType type() const noexcept { return iType; }
+    int width(int ch) const noexcept;
+    int glyphIndex(int ch) noexcept;
+    inline cairo_font_face_t *cairoFont() noexcept { return iCairoFont; }
+
+  private:
+    const PdfObj *getPdf(const PdfDict *d, String key) const noexcept;
+    bool getFontFile(const PdfDict *d, Buffer &data) noexcept;
+    void getSimpleWidth(const PdfDict *d) noexcept;
+    void getType1Encoding(const PdfDict *d) noexcept;
+    void setupTruetypeEncoding() noexcept;
+    void getCIDWidth(const PdfDict *d) noexcept;
 
   private:
     int iId;
-    Font::TType iType;
-    int iGlyphIndex[0x100];
-    int iWidth[0x100];
-    cairo_font_face_t *iCairoFont;
+    const PdfResources *iResources;
+    FontType iType;
+    String iName;
+    cairo_font_face_t *iCairoFont { nullptr };
+    FT_FaceRec_ *iFace { nullptr };
+    std::vector<int> iEncoding;
+    std::vector<int> iWidth;
+    int iDefaultWidth { 1000 };
   };
 
   class Fonts {
   public:
-    static Fonts *New(const FontPool *fontPool);
+    static Fonts *New(const PdfResources *resources);
     ~Fonts();
     Face *getFace(int id);
     static cairo_font_face_t *screenFont();
     static String freetypeVersion();
 
   private:
-    Fonts(const FontPool *fontPool);
+    Fonts(const PdfResources *resources);
 
   private:
-    const FontPool *iFontPool;
+    const PdfResources *iResources;
 
     typedef std::list<Face *> FaceSeq;
     FaceSeq iFaces;
