@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -45,14 +45,13 @@ using namespace ipe;
 
   These classes are not in Ipelib, but in a separate library
   libipecanvas.
-
-  On a Mac OS Retina display, the pixel coordinates used by Ipe refer
-  to the actual device pixels.
 */
 
 // --------------------------------------------------------------------
 
-void CanvasObserver::canvasObserverWheelMoved(int degrees) { /* nothing */ }
+CanvasObserver::~CanvasObserver() { /* nothing */ }
+void CanvasObserver::canvasObserverWheelMoved(double degrees,
+					      bool precise) { /* nothing */ }
 void CanvasObserver::canvasObserverMouseAction(int button) { /* nothing */ }
 void CanvasObserver::canvasObserverPositionChanged() { /* nothing */ }
 void CanvasObserver::canvasObserverToolChanged(bool hasTool) { /* nothing */ }
@@ -66,8 +65,8 @@ void CanvasObserver::canvasObserverSizeChanged() { /* nothing */ }
 //! Construct a new canvas.
 CanvasBase::CanvasBase()
 {
-  iObserver = 0;
-  iTool = 0;
+  iObserver = nullptr;
+  iTool = nullptr;
   iPage = 0;
   iCascade = 0;
   iSurface = 0;
@@ -76,9 +75,11 @@ CanvasBase::CanvasBase()
   iDimmed = false;
   iWidth = 0;   // not yet known (canvas is not yet mapped)
   iHeight = 0;
-  iRetina = 1;
+  iBWidth = 0;
+  iBHeight = 0;
   iRepaintObjects = false;
-  iFonts = 0;
+  iFonts = nullptr;
+  iResources = nullptr;
   iAutoSnap = false;
   iFifiVisible = false;
   iFifiMode = Snap::ESnapNone;
@@ -87,6 +88,8 @@ CanvasBase::CanvasBase()
   iAdditionalModifiers = 0;
 
   iStyle.paperColor = Color(1000,1000,1000);
+  iStyle.primarySelectionColor = Color(1000, 0, 0);
+  iStyle.secondarySelectionColor= Color(1000, 0, 1000);
   iStyle.pretty = false;
   iStyle.classicGrid = false;
   iStyle.thinLine = 0.2;
@@ -99,7 +102,7 @@ CanvasBase::CanvasBase()
   iSnap.iSnap = 0;
   iSnap.iGridVisible = false;
   iSnap.iGridSize = 8;
-  iSnap.iAngleSize = M_PI / 6.0;
+  iSnap.iAngleSize = IPE_PI / 6.0;
   iSnap.iSnapDistance = 10;
   iSnap.iWithAxes = false;
   iSnap.iOrigin = Vector::ZERO;
@@ -119,10 +122,11 @@ CanvasBase::~CanvasBase()
 // --------------------------------------------------------------------
 
 //! set information about Latex fonts (from ipe::Document)
-void CanvasBase::setFontPool(const FontPool *fontPool)
+void CanvasBase::setResources(const PdfResources *resources)
 {
   delete iFonts;
-  iFonts = Fonts::New(fontPool);
+  iResources = resources;
+  iFonts = Fonts::New(resources);
 }
 
 // --------------------------------------------------------------------
@@ -162,13 +166,6 @@ void CanvasBase::setZoom(double zoom)
   iZoom = zoom;
 }
 
-//! Set MacOS Retina display.
-void CanvasBase::setRetina(int r)
-{
-  iRetina = r;
-  update();
-}
-
 //! Set the snapping information.
 void CanvasBase::setSnap(const Snap &s)
 {
@@ -184,8 +181,8 @@ void CanvasBase::setDimmed(bool dimmed)
 
 //! Set additional modifiers.
 /*! These modifier bits are passed to the Tool when a key is pressed
-    or a drawing action is performed in addition to the actual
-    keyboard modifiers. */
+  or a drawing action is performed in addition to the actual
+  keyboard modifiers. */
 void CanvasBase::setAdditionalModifiers(int mod)
 {
   iAdditionalModifiers = mod;
@@ -221,9 +218,7 @@ Vector CanvasBase::userToDev(const Vector &arg) const
 //! Matrix mapping user coordinates to canvas coordinates
 Matrix CanvasBase::canvasTfm() const
 {
-  return Matrix(center()) *
-    Linear(iZoom, 0, 0, -iZoom) *
-    Matrix(-iPan);
+  return Matrix(center()) * Linear(iZoom, 0, 0, -iZoom) * Matrix(-iPan);
 }
 
 // --------------------------------------------------------------------
@@ -371,26 +366,10 @@ void CanvasBase::drawObjects(cairo_t *cc)
   if (background && iPage->findLayer("BACKGROUND") < 0)
     background->iObject->draw(painter);
 
-  if (iStyle.numberPages) {
-    const StyleSheet::PageNumberStyle *pns =
-      iCascade->findPageNumberStyle();
-    cairo_save(cc);
-    cairo_set_source_rgb(cc, pns->iColor.iRed.toDouble(),
-			 pns->iColor.iGreen.toDouble(),
-			 pns->iColor.iBlue.toDouble());
-    cairo_select_font_face(cc, "sans-serif",
-			   CAIRO_FONT_SLANT_NORMAL,
-			   CAIRO_FONT_WEIGHT_NORMAL);
-    cairo_set_font_size(cc, pns->iFontSize);
-    cairo_move_to(cc, pns->iPos.x, pns->iPos.y);
-    cairo_scale(cc, 1.0, -1.0);
-    char buf[20];
-    if (iPage->countViews() > 0)
-      sprintf(buf, "%d-%d", iPageNumber + 1, iView + 1);
-    else
-      sprintf(buf, "%d", iPageNumber + 1);
-    cairo_show_text(cc, buf);
-    cairo_restore(cc);
+  if (iResources && iStyle.numberPages) {
+    const Text *pn = iResources->pageNumber(iPageNumber, iView);
+    if (pn)
+      pn->draw(painter);
   }
 
   const Text *title = iPage->titleText();
@@ -504,11 +483,11 @@ void CanvasBase::drawTool(Painter &painter)
     for (int i = 0; i < iPage->count(); ++i) {
       if (iPage->objectVisible(iView, i)) {
 	if (iPage->select(i) == EPrimarySelected) {
-	  painter.setStroke(Attribute(Color(1000, 0, 0)));
+	  painter.setStroke(Attribute(iStyle.primarySelectionColor));
 	  painter.setPen(Attribute(Fixed(2)));
 	  iPage->object(i)->drawSimple(painter);
 	} else if (iPage->select(i) == ESecondarySelected) {
-	  painter.setStroke(Attribute(Color(1000, 0, 1000)));
+	  painter.setStroke(Attribute(iStyle.secondarySelectionColor));
 	  painter.setPen(Attribute(Fixed(1)));
 	  iPage->object(i)->drawSimple(painter);
 	}
@@ -646,7 +625,7 @@ void CanvasBase::updateTool()
   snapping is enabled, and stores snapped position in iMousePos. */
 void CanvasBase::computeFifi(double x, double y)
 {
-  iUnsnappedMousePos = devToUser(Vector(iRetina * x, iRetina * y));
+  iUnsnappedMousePos = devToUser(Vector(x, y));
   iMousePos = iUnsnappedMousePos;
 
   if (!iPage)
@@ -662,15 +641,15 @@ void CanvasBase::computeFifi(double x, double y)
     // convert fifi coordinates back into device space
     Vector fifi = userToDev(iMousePos);
     if (iFifiVisible && fifi != iOldFifi) {
-      invalidate(int(iOldFifi.x / iRetina - 10),
-		 int(iOldFifi.y / iRetina - 10), 21, 21);
-      invalidate(int(fifi.x / iRetina - 10),
-		 int(fifi.y / iRetina - 10), 21, 21);
+      invalidate(int(iOldFifi.x - 10),
+		 int(iOldFifi.y - 10), 21, 21);
+      invalidate(int(fifi.x - 10),
+		 int(fifi.y - 10), 21, 21);
     }
   } else if (iFifiVisible) {
     // remove old fifi
-    invalidate(int(iOldFifi.x / iRetina - 10),
-	       int(iOldFifi.y / iRetina - 10), 21, 21);
+    invalidate(int(iOldFifi.x - 10),
+	       int(iOldFifi.y - 10), 21, 21);
     iFifiVisible = false;
   }
 }
@@ -680,10 +659,11 @@ void CanvasBase::computeFifi(double x, double y)
 void CanvasBase::refreshSurface()
 {
   if (!iSurface
-      || iWidth != cairo_image_surface_get_width(iSurface)
-      || iHeight != cairo_image_surface_get_height(iSurface)) {
+      || iBWidth != cairo_image_surface_get_width(iSurface)
+      || iBHeight != cairo_image_surface_get_height(iSurface)) {
     // size has changed
-    ipeDebug("size has changed to %d x %d", iWidth, iHeight);
+    ipeDebug("size has changed to %g x %g (%g x %g)",
+	     iWidth, iHeight, iBWidth, iBHeight);
     if (iSurface)
       cairo_surface_destroy(iSurface);
     iSurface = 0;
@@ -695,15 +675,16 @@ void CanvasBase::refreshSurface()
   if (iRepaintObjects) {
     iRepaintObjects = false;
     if (!iSurface)
-      iSurface =
-	cairo_image_surface_create(CAIRO_FORMAT_RGB24, iWidth, iHeight);
+      iSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,
+					    iBWidth, iBHeight);
     cairo_t *cc = cairo_create(iSurface);
     // background
     cairo_set_source_rgb(cc, 0.4, 0.4, 0.4);
-    cairo_rectangle(cc, 0, 0, iWidth, iHeight);
+    cairo_rectangle(cc, 0, 0, iBWidth, iBHeight);
     cairo_fill(cc);
 
-    cairo_translate(cc, 0.5 * iWidth, 0.5 * iHeight);
+    cairo_translate(cc, 0.5 * iBWidth, 0.5 * iBHeight);
+    cairo_scale(cc, iBWidth / iWidth, iBHeight / iHeight);
     cairo_scale(cc, iZoom, -iZoom);
     cairo_translate(cc, -iPan.x, -iPan.y);
 
