@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -176,7 +176,8 @@ void SelectTool::mouseButton(int button, bool press)
       int new_primary = -1;
       for (int i = 0; i < iPage->count(); ++i) {
 	Matrix id;
-	if (iPage->objectVisible(iView, i)) {
+	if (iPage->objectVisible(iView, i) &&
+	    !iPage->isLocked(iPage->layerOf(i))) {
 	  Rect s;
 	  iPage->object(i)->addToBBox(s, id, false);
 	  if (alternate ? r.intersects(s) : r.contains(s)) {
@@ -204,7 +205,8 @@ void SelectTool::mouseButton(int button, bool press)
       for (int i = 0; i < iPage->count(); ++i) {
 	iPage->setSelect(i, ENotSelected);
 	Matrix id;
-	if (iPage->objectVisible(iView, i)) {
+	if (iPage->objectVisible(iView, i) &&
+	    !iPage->isLocked(iPage->layerOf(i))) {
 	  Rect s;
 	  iPage->object(i)->addToBBox(s, id, false);
 	  if (alternate ? r.intersects(s) : r.contains(s))
