@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -225,7 +225,7 @@ String Repository::toString(int index) const
 /*! The string is added to the repository if it doesn't exist yet. */
 int Repository::toIndex(String str)
 {
-  assert(!str.isEmpty());
+  assert(!str.empty());
   std::vector<String>::const_iterator it =
     std::find(iStrings.begin(), iStrings.end(), str);
   if (it != iStrings.end())
@@ -371,7 +371,7 @@ Color::Color(String str)
 */
 Attribute Attribute::makeColor(String str, Attribute deflt)
 {
-  if (str.isEmpty())
+  if (str.empty())
     return deflt;
   else if (('a' <= str[0] && str[0] <= 'z') ||
 	   ('A' <= str[0] && str[0] <= 'Z'))
@@ -386,7 +386,7 @@ Attribute Attribute::makeColor(String str, Attribute deflt)
   Otherwise, must be a number. */
 Attribute Attribute::makeScalar(String str, Attribute deflt)
 {
-  if (str.isEmpty()) {
+  if (str.empty()) {
     return deflt;
   } else if (('a' <= str[0] && str[0] <= 'z') ||
 	     ('A' <= str[0] && str[0] <= 'Z')) {
@@ -403,7 +403,7 @@ Attribute Attribute::makeScalar(String str, Attribute deflt)
 */
 Attribute Attribute::makeDashStyle(String str)
 {
-  if (str.isEmpty())
+  if (str.empty())
     return Attribute::NORMAL();
   else if (str[0] == '[')
     return Attribute(false, str);
