@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -492,6 +492,7 @@ const char *Bitmap::readJpegInfo(FILE *file, int &width, int &height,
 				 int &bitsPerComponent)
 {
   static char jpg_id[] = "JFIF";
+  bool app0_seen = false;
 
   dotsPerInch = Vector(0, 0);
 
@@ -503,9 +504,13 @@ const char *Bitmap::readJpegInfo(FILE *file, int &width, int &height,
     int ch = fgetc(file);
     if (ch != 0xff)
       return "Reading JPEG image failed";
-    ch = fgetc(file);
+    do {
+      ch = fgetc(file);
+    } while (ch == 0xff);
+    ipeDebug("JPEG tag %x", ch & 0xff);
     int fpos = ftell(file);
     switch (ch & 0xff) {
+
     case M_SOF5:
     case M_SOF6:
     case M_SOF7:
@@ -516,10 +521,10 @@ const char *Bitmap::readJpegInfo(FILE *file, int &width, int &height,
     case M_SOF14:
     case M_SOF15:
       return "Unsupported type of JPEG compression";
-    case M_SOF2:
-      // cannot use progressive DCT with PDF-1.2
+
     case M_SOF0:
     case M_SOF1:
+    case M_SOF2: // progressive DCT allowed since PDF-1.3
     case M_SOF3:
       read2bytes(file);    /* read segment length  */
       ch = fgetc(file);
@@ -541,9 +546,14 @@ const char *Bitmap::readJpegInfo(FILE *file, int &width, int &height,
 	return "Unsupported color space in JPEG image";
       }
       fseek(file, 0, SEEK_SET);
-      return 0;      //  success!
+      return nullptr;      //  success!
+
     case M_APP0: {
       int len = read2bytes(file);
+      if (app0_seen) {
+	fseek(file, fpos + len, SEEK_SET);
+	break;
+      }
       for (int i = 0; i < 5; i++) {
 	ch = fgetc(file);
 	if (ch != jpg_id[i]) {
@@ -566,8 +576,10 @@ const char *Bitmap::readJpegInfo(FILE *file, int &width, int &height,
 	  break;
 	}
       }
+      app0_seen = true;
       fseek(file, fpos + len, SEEK_SET);
       break; }
+
     case M_SOI:      // ignore markers without parameters
     case M_EOI:
     case M_TEM:
@@ -579,8 +591,8 @@ const char *Bitmap::readJpegInfo(FILE *file, int &width, int &height,
     case M_RST5:
     case M_RST6:
     case M_RST7:
-    case 0xff:       // What is this?
       break;
+
     default:         // skip variable length markers
 	fseek(file, fpos + read2bytes(file), SEEK_SET);
       break;
