@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -34,17 +34,12 @@
 #include "ipereference.h"
 #include "ipepainter.h"
 #include "ipeutils.h"
-#include "ipefontpool.h"
 #include "ipepdfparser.h"
 #include "ipepdfwriter.h"
 #include "ipelatex.h"
 
 #include <errno.h>
 
-#ifdef IPE_USE_ICONV
-#include <iconv.h>
-#endif
-
 using namespace ipe;
 
 // --------------------------------------------------------------------
@@ -68,19 +63,12 @@ using namespace ipe;
   all the methods necessary to load, save, and modify it.
 */
 
-//! Constructor clears the boolean flags.
-Document::SProperties::SProperties()
-{
-  iFullScreen = false;
-  iNumberPages = false;
-}
-
 //! Construct an empty document for filling by a client.
 /*! As constructed, it has no pages, A4 media, and
   only the standard style sheet. */
 Document::Document()
 {
-  iFontPool = 0;
+  iResources = nullptr;
   iCascade = new Cascade();
   iCascade->insert(0, StyleSheet::standard());
 }
@@ -91,7 +79,7 @@ Document::~Document()
   for (int i = 0; i < countPages(); ++i)
     delete page(i);
   delete iCascade;
-  delete iFontPool;
+  delete iResources;
 }
 
 //! Copy constructor.
@@ -101,7 +89,7 @@ Document::Document(const Document &rhs)
   for (int i = 0; i < rhs.countPages(); ++i)
     iPages.push_back(new Page(*rhs.page(i)));
   iProperties = rhs.iProperties;
-  iFontPool = 0;
+  iResources = nullptr;
 }
 
 // ---------------------------------------------------------------------
@@ -446,7 +434,7 @@ bool Document::save(TellStream &stream, TFormat format, uint flags) const
     compresslevel = 0;
 
   if (format == EPdf) {
-    PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
+    PdfWriter writer(stream, this, iResources, (flags & EMarkedView),
 		     0, -1, compresslevel);
     writer.createPages();
     writer.createBookmarks();
@@ -498,7 +486,7 @@ bool Document::exportView(const char *fname, TFormat format, uint flags,
     return false;
   FileStream stream(fd);
 
-  PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
+  PdfWriter writer(stream, this, iResources, (flags & EMarkedView),
 		   pno, pno, compresslevel);
   writer.createPageView(pno, vno);
   writer.createTrailer();
@@ -517,7 +505,7 @@ bool Document::exportPages(const char *fname, uint flags,
   if (!fd)
     return false;
   FileStream stream(fd);
-  PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
+  PdfWriter writer(stream, this, iResources, (flags & EMarkedView),
 		   fromPage, toPage, compresslevel);
   writer.createPages();
   writer.createTrailer();
@@ -545,7 +533,22 @@ void Document::findBitmaps(BitmapFinder &bm) const
 //! Save in XML format into an Stream.
 void Document::saveAsXml(Stream &stream, bool usePdfBitmaps) const
 {
-  stream << "<ipe version=\"" << ipe::FILE_FORMAT << "\"";
+  // check file format update to 7.2.4
+  int ff = ipe::FILE_FORMAT;
+  for (int i = 0; i < countPages(); ++i) {
+    const Page *p = page(i);
+    for (int j = 0; j < p->count(); ++j) {
+      const Group *g = p->object(j)->asGroup();
+      if (g && !g->url().empty()) {
+	ff = ipe::FILE_FORMAT_NEW;
+      }
+    }
+  }
+  for (int i = 0; i < iCascade->count() - 1; ++i) { // skip standard stylesheet
+    if (iCascade->sheet(i)->pageNumberStyle())
+      ff = ipe::FILE_FORMAT_NEW;
+  }
+  stream << "<ipe version=\"" << ff << "\"";
   if (!iProperties.iCreator.empty())
     stream << " creator=\"" << iProperties.iCreator << "\"";
   stream << ">\n";
@@ -582,6 +585,20 @@ void Document::saveAsXml(Stream &stream, bool usePdfBitmaps) const
   if (iProperties.iNumberPages) {
     infoStr << " numberpages=\"yes\"";
   }
+  switch (iProperties.iTexEngine) {
+  case LatexType::Pdftex:
+    infoStr << " tex=\"pdftex\"";
+    break;
+  case LatexType::Xetex:
+    infoStr << " tex=\"xetex\"";
+    break;
+  case LatexType::Luatex:
+    infoStr << " tex=\"luatex\"";
+    break;
+  case LatexType::Default:
+  default:
+    break;
+  }
   infoStr << "/>\n";
   if (info.size() > 10)
     stream << info;
@@ -659,56 +676,12 @@ bool Document::checkStyle(AttributeSeq &seq) const
   return (seq.size() == 0);
 }
 
-//! Return whether this document uses any Truetype fonts.
-bool Document::hasTrueTypeFonts() const
+//! Update the PDF resources (after running latex).
+/*! Takes ownership. */
+void Document::setResources(PdfResources *resources)
 {
-  if (iFontPool) {
-    for (FontPool::const_iterator it = iFontPool->begin();
-	 it != iFontPool->end(); ++it) {
-      if (it->iType == Font::ETrueType)
-	return true;
-    }
-  }
-  return false;
-}
-
-//! Does this document make any use of transparency?
-/*! The document is considered to make use of transparency if its
-    style sheets define any opacities whose value is not 1.0. */
-bool Document::hasTransparency() const
-{
-  AttributeSeq os;
-  iCascade->allNames(EOpacity, os);
-  for (uint i = 0; i < os.size(); ++i) {
-    Attribute val = iCascade->find(EOpacity, os[i]);
-    if (val != Attribute::ONE())
-      return true;
-  }
-  return false;
-}
-
-//! Does this document contain any tiling patterns?
-bool Document::hasTilings() const
-{
-  AttributeSeq ts;
-  iCascade->allNames(ETiling, ts);
-  return (ts.size() > 0);
-}
-
-//! Does this document contain any gradients?
-bool Document::hasGradients() const
-{
-  AttributeSeq gs;
-  iCascade->allNames(EGradient, gs);
-  return (gs.size() > 0);
-}
-
-//! Update the font pool (after running Pdflatex).
-/*! Takes ownership of the font pool. */
-void Document::setFontPool(FontPool *fontPool)
-{
-  delete iFontPool;
-  iFontPool = fontPool;
+  delete iResources;
+  iResources = resources;
 }
 
 #if 0
@@ -770,11 +743,11 @@ Page *Document::remove(int no)
 
 // --------------------------------------------------------------------
 
-//! Run PdfLatex
+//! Run PdfLatex or Xelatex
 int Document::runLatex(String &texLog)
 {
   texLog = "";
-  Latex converter(cascade());
+  Latex converter(cascade(), iProperties.iTexEngine);
 
   AttributeSeq seq;
   cascade()->allNames(ESymbol, seq);
@@ -791,6 +764,14 @@ int Document::runLatex(String &texLog)
   if (count == 0)
     return ErrNoText;
 
+  if (iProperties.iNumberPages) {
+    for (int i = 0; i < countPages(); ++i) {
+      int nviews = page(i)->countViews();
+      for (int j = 0; j < nviews; ++j)
+	converter.addPageNumber(i, j, countPages(), nviews);
+    }
+  }
+
   // First we need a directory
   String latexDir = Platform::latexDirectory();
   if (latexDir.empty())
@@ -802,65 +783,6 @@ int Document::runLatex(String &texLog)
 
   std::remove(logFile.z());
 
-  String encoding = cascade()->findEncoding();
-
-#if defined(IPE_USE_ICONV) || defined(WIN32)
-  String utf8;
-  StringStream stream(utf8);
-  int err = converter.createLatexSource(stream, properties().iPreamble);
-  if (err < 0)
-    return ErrWritingSource;
-
-  if (encoding.empty()) {
-    std::FILE *file = Platform::fopen(texFile.z(), "wb");
-    if (!file)
-      return ErrWritingSource;
-    FileStream fstream(file);
-    fstream.putRaw(utf8.data(), utf8.size());
-    std::fclose(file);
-  } else {
-#ifdef WIN32
-    std::FILE *file = Platform::fopen(texFile.z(), "wb");
-    if (!file)
-      return ErrWritingSource;
-    String raw = Platform::winConv(utf8, encoding);
-    std::fputs(raw.z(), file);
-    std::fclose(file);
-#else
-    iconv_t conv = iconv_open(encoding.z(), "UTF-8");
-    if (conv == iconv_t(-1))
-      return ErrWritingSource;
-
-    std::FILE *file = Platform::fopen(texFile.z(), "wb");
-    if (!file)
-      return ErrWritingSource;
-
-    char *inbuf = (char *) utf8.data();
-    size_t inbytesleft = utf8.size();
-
-    FileStream fstream(file);
-    while (inbytesleft > 0) {
-      char outbuf[0x100];
-      char *outp = outbuf;
-      size_t outbytesleft = 0x100;
-      size_t result = iconv(conv, &inbuf, &inbytesleft, &outp, &outbytesleft);
-      // E2BIG means output buffer exhausted, we continue in the next round
-      if (result == size_t(-1) && errno != E2BIG) {
-	std::fclose(file);
-	iconv_close(conv);
-	return ErrWritingSource;
-      }
-      if (outp > outbuf)
-	fstream.putRaw(outbuf, outp - outbuf);
-    }
-    iconv_close(conv);
-    std::fclose(file);
-#endif
-  }
-#else
-  if (!encoding.empty())
-    return ErrNoIconv;
-
   std::FILE *file = Platform::fopen(texFile.z(), "wb");
   if (!file)
     return ErrWritingSource;
@@ -870,10 +792,8 @@ int Document::runLatex(String &texLog)
 
   if (err < 0)
     return ErrWritingSource;
-#endif
 
-
-  int result = Platform::runPdfLatex(latexDir);
+  int result = Platform::runLatex(latexDir, iProperties.iTexEngine);
 
   if (result != 0 && result != 1)
     return ErrRunLatex;
@@ -881,13 +801,16 @@ int Document::runLatex(String &texLog)
   // Check log file for Pdflatex version and errors
   texLog = Platform::readFile(logFile);
   if (texLog.left(14) != "This is pdfTeX" &&
-      texLog.left(15) != "This is pdfeTeX")
+      texLog.left(15) != "This is pdfeTeX" &&
+      texLog.left(13) != "This is XeTeX" &&
+      texLog.left(14) != "This is LuaTeX" &&
+      texLog.left(22) != "entering extended mode")
     return ErrRunLatex;
-  int i = texLog.find('-');
+  int i = texLog.find('\n');
   if (i < 0)
     return ErrRunLatex;
-  String version = texLog.substr(i+1, 30);
-  ipeDebug("pdfTeX version %s", version.z());
+  String version = texLog.substr(8, i);
+  ipeDebug("%s", version.z());
   // Check for error
   if (texLog.find("\n!") >= 0)
     return ErrLatex;
@@ -900,7 +823,7 @@ int Document::runLatex(String &texLog)
   std::fclose(pdfF);
 
   if (okay) {
-    setFontPool(converter.takeFontPool());
+    setResources(converter.takeResources());
     return ErrNone;
   } else
     return ErrLatexOutput;
@@ -934,11 +857,6 @@ int Document::runLatex()
   case ErrLatexOutput:
     fprintf(stderr, "There was an error reading the Pdflatex output.\n");
     return 1;
-  case ErrNoIconv:
-    fprintf(stderr,
-	    "This document needs charset conversion to run Pdflatex,\n"
-	    "but Ipe is compiled without this feature.\n");
-    return 1;
   case ErrNone:
   default:
     fprintf(stderr, "Pdflatex was run sucessfully.\n");
