@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -114,6 +114,13 @@ int ImlParser::parseDocument(Document &doc)
     properties.iNumberPages = (att["numberpages"] == "yes");
     properties.iCreated = att["created"];
     properties.iModified = att["modified"];
+    String tex = att["tex"];
+    if (tex == "pdftex")
+      properties.iTexEngine = LatexType::Pdftex;
+    else if (tex == "xetex")
+      properties.iTexEngine = LatexType::Xetex;
+    else if (tex == "luatex")
+      properties.iTexEngine = LatexType::Luatex;
 
     tag = parseToTag();
   }
@@ -313,14 +320,14 @@ Page *ImlParser::parsePageSelection()
   if (tag != "page")
     return 0;
 
-  IpeAutoPtr<Page> page(new Page);
+  std::unique_ptr<Page> page(new Page);
   if (!parsePage(*page))
     return 0;
 
   tag = parseToTag();
   if (tag != "/ipepage")
     return 0;
-  return page.take();
+  return page.release();
 }
 
 #if 0
@@ -496,27 +503,26 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
       ts.iVerticalAlignment = Text::makeVAlign(att["valign"], EAlignBaseline);
       sheet.setTitleStyle(ts);
     } else if (tag == "pagenumberstyle") {
-      if (!parseAttributes(att) || !att.slash())
+      if (!parseAttributes(att))
 	return false;
       StyleSheet::PageNumberStyle pns;
       pns.iDefined = true;
       Lex lex1(att["pos"]);
       lex1 >> pns.iPos.x >> pns.iPos.y;
-      pns.iFontSize = Lex(att["size"]).getDouble();
-      pns.iColor = Color(att["color"]);
+      pns.iSize = Attribute::makeTextSize(att["size"]);
+      pns.iColor = Attribute::makeColor(att["color"], Attribute::BLACK());
+      pns.iVerticalAlignment = Text::makeVAlign(att["valign"], EAlignBaseline);
+      pns.iHorizontalAlignment = Text::makeHAlign(att["halign"], EAlignLeft);
+      if (!att.slash() && !parsePCDATA(tag, pns.iText))
+	return false;
       sheet.setPageNumberStyle(pns);
     } else if (tag == "preamble") {
       if (!parseAttributes(att))
 	return false;
-      sheet.setEncoding(att["encoding"]);
       String pcdata;
       if (!att.slash() && !parsePCDATA(tag, pcdata))
 	return false;
       sheet.setPreamble(pcdata);
-    } else if (tag == "cmap") {
-      if (!parseAttributes(att) || !att.slash())
-	return false;
-      sheet.addCMap(att["font"]);
     } else if (tag == "pathstyle") {
       if (!parseAttributes(att) || !att.slash())
 	return false;
