@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -171,7 +171,7 @@ Object::Object(const XmlAttributes &attr)
       iPinned = EVerticalPin;
   }
   iTransformations = ETransformationsAffine;
-  if (attr.has("transformations", str) && !str.isEmpty()) {
+  if (attr.has("transformations", str) && !str.empty()) {
     if (str == "rigid")
       iTransformations = ETransformationsRigidMotions;
     else if (str == "translations")
@@ -211,7 +211,7 @@ Object::~Object()
 //! Write layer, pin, transformations, matrix to XML stream.
 void Object::saveAttributesAsXml(Stream &stream, String layer) const
 {
-  if (!layer.isEmpty())
+  if (!layer.empty())
     stream << " layer=\"" << layer << "\"";
   if (!iMatrix.isIdentity())
     stream << " matrix=\"" << iMatrix << "\"";
@@ -235,34 +235,40 @@ void Object::saveAttributesAsXml(Stream &stream, String layer) const
     stream << " transformations=\"rigid\"";
 }
 
-//! Return pointer to this object if it is an Group, 0 otherwise.
+//! Return pointer to this object if it is an Group, nullptr otherwise.
 Group *Object::asGroup()
 {
-  return 0;
+  return nullptr;
 }
 
-//! Return pointer to this object if it is an Text, 0 otherwise.
+//! Return pointer to this object if it is an Group, nullptr otherwise.
+const Group *Object::asGroup() const
+{
+  return nullptr;
+}
+
+//! Return pointer to this object if it is an Text, nullptr otherwise.
 Text *Object::asText()
 {
-  return 0;
+  return nullptr;
 }
 
-//! Return pointer to this object if it is an Path, 0 otherwise.
+//! Return pointer to this object if it is an Path, nullptr otherwise.
 Path *Object::asPath()
 {
-  return 0;
+  return nullptr;
 }
 
-//! Return pointer to this object if it is an Image , 0 otherwise.
+//! Return pointer to this object if it is an Image , nullptr otherwise.
 Image *Object::asImage()
 {
-  return 0;
+  return nullptr;
 }
 
-//! Return pointer to this object if it is an Ref, 0 otherwise.
+//! Return pointer to this object if it is an Ref, nullptr otherwise.
 Reference *Object::asReference()
 {
-  return 0;
+  return nullptr;
 }
 
 // --------------------------------------------------------------------
