@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -60,11 +60,11 @@ using namespace ipe;
 //! Construct from XML data.
 Path *Path::create(const XmlAttributes &attr, String data)
 {
-  IpeAutoPtr<Path> self(new Path(attr));
+  std::unique_ptr<Path> self(new Path(attr));
   if (!self->iShape.load(data))
     return 0;
   self->makeArrowData();
-  return self.take();
+  return self.release();
 }
 
 //! Create empty path with attributes taken from XML
@@ -312,6 +312,7 @@ void Path::drawArrow(Painter &painter, Vector pos, Angle angle,
   if (symbol) {
     double s = painter.cascade()->find(EArrowSize, size).number().toDouble();
     Color color = painter.stroke();
+    // Fixed opaq = painter.opacity();
 
     painter.push();
     painter.pushMatrix();
@@ -352,6 +353,7 @@ void Path::drawArrow(Painter &painter, Vector pos, Angle angle,
 	painter.setFill(Attribute(Color(1000, 1000, 1000)));
       else
 	painter.setFill(Attribute(color));
+      // painter.setOpacity(Attribute(opaq));
       painter.newPath();
       painter.moveTo(arc1.beginp());
       painter.drawArc(arc1);
