@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -79,31 +79,31 @@ PdfObj::~PdfObj()
 }
 
 //! Return this object as PDF null object.
-const PdfNull *PdfObj::null() const { return 0; }
+const PdfNull *PdfObj::null() const noexcept { return nullptr; }
 
 //! Return this object as PDF bool object.
-const PdfBool *PdfObj::boolean() const { return 0; }
+const PdfBool *PdfObj::boolean() const noexcept { return nullptr; }
 
 //! Return this object as PDF number object.
-const PdfNumber *PdfObj::number() const { return 0; }
+const PdfNumber *PdfObj::number() const noexcept { return nullptr; }
 
 //! Return this object as PDF string object.
-const PdfString *PdfObj::string() const { return 0; }
+const PdfString *PdfObj::string() const noexcept { return nullptr; }
 
 //! Return this object as PDF name object.
-const PdfName *PdfObj::name() const { return 0; }
+const PdfName *PdfObj::name() const noexcept { return nullptr; }
 
 //! Return this object as PDF reference object.
-const PdfRef *PdfObj::ref() const { return 0; }
+const PdfRef *PdfObj::ref() const noexcept { return nullptr; }
 
 //! Return this object as PDF array object.
-const PdfArray *PdfObj::array() const { return 0; }
+const PdfArray *PdfObj::array() const noexcept { return nullptr; }
 
 //! Return this object as PDF dictionary object.
-const PdfDict *PdfObj::dict() const { return 0; }
+const PdfDict *PdfObj::dict() const noexcept { return nullptr; }
 
 //! Return PDF representation of the object.
-String PdfObj::repr() const
+String PdfObj::repr() const noexcept
 {
   String d;
   StringStream ss(d);
@@ -115,9 +115,9 @@ String PdfObj::repr() const
  * \ingroup base
  * \brief The PDF null object.
  */
-const PdfNull *PdfNull::null() const { return this; }
+const PdfNull *PdfNull::null() const noexcept { return this; }
 
-void PdfNull::write(Stream &stream) const
+void PdfNull::write(Stream &stream, const PdfRenumber *) const noexcept
 {
   stream << "null";
 }
@@ -126,9 +126,9 @@ void PdfNull::write(Stream &stream) const
  * \ingroup base
  * \brief The PDF bool object.
  */
-const PdfBool *PdfBool::boolean() const { return this; }
+const PdfBool *PdfBool::boolean() const noexcept { return this; }
 
-void PdfBool::write(Stream &stream) const
+void PdfBool::write(Stream &stream, const PdfRenumber *) const noexcept
 {
   stream << (iValue ? "true" : "false");
 }
@@ -137,9 +137,9 @@ void PdfBool::write(Stream &stream) const
  * \ingroup base
  * \brief The PDF number object.
  */
-const PdfNumber *PdfNumber::number() const { return this; }
+const PdfNumber *PdfNumber::number() const noexcept { return this; }
 
-void PdfNumber::write(Stream &stream) const
+void PdfNumber::write(Stream &stream, const PdfRenumber *) const noexcept
 {
   stream << iValue;
 }
@@ -148,30 +148,46 @@ void PdfNumber::write(Stream &stream) const
  * \ingroup base
  * \brief The PDF string object.
  */
-const PdfString *PdfString::string() const { return this; }
+const PdfString *PdfString::string() const noexcept { return this; }
 
-void PdfString::write(Stream &stream) const
+void PdfString::write(Stream &stream, const PdfRenumber *) const noexcept
 {
-  char octbuf[5];
-  stream << "(";
-  for (int i = 0; i < iValue.size(); ++i) {
-    int ch = iValue[i];
-    if ((0 <= ch && ch < 0x20) || ch == '\\' || ch == '(' || ch == ')') {
-      sprintf(octbuf, "\\%.3o", (ch & 0xff));
-      stream << octbuf;
-    } else
-      stream.putChar(ch);
+  if (iBinary) {
+    stream << "<" << iValue << ">";
+  } else {
+    char octbuf[5];
+    stream << "(";
+    for (int i = 0; i < iValue.size(); ++i) {
+      int ch = iValue[i];
+      if ((0 <= ch && ch < 0x20) || ch == '\\' || ch == '(' || ch == ')') {
+	sprintf(octbuf, "\\%.3o", (ch & 0xff));
+	stream << octbuf;
+      } else
+	stream.putChar(ch);
+    }
+    stream << ")";
   }
-  stream << ")";
+}
+
+//! Return value of string after decoding binary strings.
+String PdfString::decode() const noexcept
+{
+  if (!iBinary)
+    return iValue;
+  String result;
+  Lex lex(iValue);
+  while (!lex.eos())
+    result += char(lex.getHexByte());
+  return result;
 }
 
 /*! \class ipe::PdfName
  * \ingroup base
  * \brief The PDF name object.
  */
-const PdfName *PdfName::name() const { return this; }
+const PdfName *PdfName::name() const noexcept { return this; }
 
-void PdfName::write(Stream &stream) const
+void PdfName::write(Stream &stream, const PdfRenumber *) const noexcept
 {
   stream << "/" << iValue;
 }
@@ -180,10 +196,17 @@ void PdfName::write(Stream &stream) const
  * \ingroup base
  * \brief The PDF reference object (indirect object).
  */
-const PdfRef *PdfRef::ref() const { return this; }
+const PdfRef *PdfRef::ref() const noexcept { return this; }
 
-void PdfRef::write(Stream &stream) const
+void PdfRef::write(Stream &stream, const PdfRenumber *renumber) const noexcept
 {
+  if (renumber) {
+    auto it = renumber->find(iValue);
+    if (it != renumber->end()) {
+      stream << it->second << " 0 R";
+      return;
+    }
+  }
   stream << iValue << " 0 R";
 }
 
@@ -191,16 +214,16 @@ void PdfRef::write(Stream &stream) const
  * \ingroup base
  * \brief The PDF array object.
  */
-const PdfArray *PdfArray::array() const { return this; }
+const PdfArray *PdfArray::array() const noexcept { return this; }
 
-void PdfArray::write(Stream &stream) const
+void PdfArray::write(Stream &stream, const PdfRenumber *renumber) const noexcept
 {
   stream << "[";
   String sep = "";
   for (int i = 0; i < count(); ++i) {
     stream << sep;
     sep = " ";
-    obj(i, 0)->write(stream);
+    obj(i, 0)->write(stream, renumber);
   }
   stream << "]";
 }
@@ -223,10 +246,10 @@ void PdfArray::append(const PdfObj *obj)
 
 //! Return object with \a index in array.
 /*! Indirect objects (references) are looked up if \a file is not
-  zero, and the object referred to is returned (0 if it does not
+  nullptr, and the object referred to is returned (nullptr if it does not
   exist).  Object remains owned by array.
 */
-const PdfObj *PdfArray::obj(int index, const PdfFile *file) const
+const PdfObj *PdfArray::obj(int index, const PdfFile *file) const noexcept
 {
   const PdfObj *obj = iObjects[index];
   if (file && obj->ref()) {
@@ -243,9 +266,9 @@ const PdfObj *PdfArray::obj(int index, const PdfFile *file) const
  A dictionary may or may not have attached stream data.
  */
 
-const PdfDict *PdfDict::dict() const { return this; }
+const PdfDict *PdfDict::dict() const noexcept { return this; }
 
-void PdfDict::write(Stream &stream) const
+void PdfDict::write(Stream &stream, const PdfRenumber *renumber) const noexcept
 {
   String sep = "<<";
   for (std::vector<Item>::const_iterator it = iItems.begin();
@@ -253,7 +276,7 @@ void PdfDict::write(Stream &stream) const
     stream << sep;
     sep = " ";
     stream << "/" << it->iKey << " ";
-    it->iVal->write(stream);
+    it->iVal->write(stream, renumber);
   }
   stream << ">>";
   if (iStream.size() > 0) {
@@ -290,11 +313,11 @@ void PdfDict::add(String key, const PdfObj *obj)
 }
 
 //! Look up key in dictionary.
-/*! Indirect objects (references) are looked up if \a file is not zero,
+/*! Indirect objects (references) are looked up if \a file is not nullptr,
   and the object referred to is returned.
-  Returns 0 if key is not in dictionary.
+  Returns nullptr if key is not in dictionary.
 */
-const PdfObj *PdfDict::get(String key, const PdfFile *file) const
+const PdfObj *PdfDict::get(String key, const PdfFile *file) const noexcept
 {
   for (std::vector<Item>::const_iterator it = iItems.begin();
        it != iItems.end(); ++it) {
@@ -305,29 +328,52 @@ const PdfObj *PdfDict::get(String key, const PdfFile *file) const
 	return it->iVal;
     }
   }
-  return 0; // not in dictionary
+  return nullptr; // not in dictionary
+}
+
+//! Retrieve a single number and stor in \a val.
+bool PdfDict::getNumber(String key, double &val, const PdfFile *file)
+  const noexcept
+{
+  const PdfObj *obj = get(key, file);
+  if (!obj || !obj->number())
+    return false;
+  val = obj->number()->value();
+  return true;
+}
+
+//! Retrieve an array of numbers and store in \a vals.
+bool PdfDict::getNumberArray(String key, const PdfFile *file,
+			     std::vector<double> &vals) const noexcept
+{
+  const PdfObj *obj = get(key, file);
+  if (!obj || !obj->array())
+    return false;
+  vals.clear();
+  for (int i = 0; i < obj->array()->count(); i++) {
+    const PdfObj *a = obj->array()->obj(i, file);
+    if (!a || !a->number())
+      return false;
+    vals.push_back(a->number()->value());
+  }
+  return true;
 }
 
 //! Is this stream compressed with flate compression?
-bool PdfDict::deflated() const
+bool PdfDict::deflated() const noexcept
 {
-  const PdfObj *f = get("Filter", 0);
+  const PdfObj *f = get("Filter", nullptr);
   return !(!f || !f->name() || f->name()->value() != "FlateDecode");
 }
 
-#if 0
 //! Return the (uncompressed) stream data.
 /*! This only handles the /Flate compression. */
-Buffer PdfDict::Inflate() const
+Buffer PdfDict::inflate() const noexcept
 {
-  if (iStream.size() == 0)
-    return iStream;
-  const PdfObj *f = get("Filter", 0);
-  if (!f || !f->Name() || f->Name()->Value() != "FlateDecode")
+  if (iStream.size() == 0 || !deflated())
     return iStream;
 
   String dest;
-
   BufferSource bsource(iStream);
   InflateSource source(bsource);
 
@@ -338,7 +384,6 @@ Buffer PdfDict::Inflate() const
   }
   return Buffer(dest.data(), dest.size());
 }
-#endif
 
 // --------------------------------------------------------------------
 
@@ -349,7 +394,7 @@ Buffer PdfDict::Inflate() const
  The parser understands the syntax of PDF files, but very little of
  its semantics.  It is meant to be able to parse PDF documents created
  by Ipe for loading, and to extract information from PDF files created
- by Pdflatex.
+ by Pdflatex or Xelatex.
 
  The parser reads a PDF file sequentially from front to back, ignores
  the contents of 'xref' sections, stores only generation 0 objects,
@@ -451,8 +496,7 @@ void PdfParser::getToken()
     }
     // We don't bother to decode it
     getChar(); // skip '>'
-    iTok.iType = PdfToken::EString;
-    ipeDebug("Found binary string <%s>", iTok.iString.z());
+    iTok.iType = PdfToken::EStringBinary;
     return;
   }
 
@@ -506,12 +550,12 @@ void PdfParser::getToken()
 //! Parse elements of an array.
 PdfArray *PdfParser::makeArray()
 {
-  IpeAutoPtr<PdfArray> arr(new PdfArray);
+  std::unique_ptr<PdfArray> arr(new PdfArray);
   for (;;) {
     if (iTok.iType == PdfToken::EArrayEnd) {
       // finish array
       getToken();
-      return arr.take();
+      return arr.release();
     }
     // check for reference object
     if (iTok.iType == PdfToken::ENumber) {
@@ -533,7 +577,7 @@ PdfArray *PdfParser::makeArray()
     } else {
       PdfObj *obj = getObject();
       if (!obj)
-	return 0;
+	return nullptr;
       arr->append(obj);
     }
   }
@@ -541,7 +585,7 @@ PdfArray *PdfParser::makeArray()
 
 PdfDict *PdfParser::makeDict()
 {
-  IpeAutoPtr<PdfDict> dict(new PdfDict);
+  std::unique_ptr<PdfDict> dict(new PdfDict);
   for (;;) {
     if (iTok.iType == PdfToken::EDictEnd) {
       // finish
@@ -549,7 +593,7 @@ PdfDict *PdfParser::makeDict()
 
       // check whether stream follows
       if (iTok.iType != PdfToken::EOp || iTok.iString != "stream")
-	return dict.take();
+	return dict.release();
 
       // time to read the stream
       while (!eos() && iCh != '\n')
@@ -558,7 +602,7 @@ PdfDict *PdfParser::makeDict()
       // now at beginning of stream
       const PdfObj *len = dict->get("Length", 0);
       if (!len || !len->number())
-	return 0;
+	return nullptr;
       int bytes = int(len->number()->value());
       Buffer buf(bytes);
       char *p = buf.data();
@@ -569,14 +613,14 @@ PdfDict *PdfParser::makeDict()
       dict->setStream(buf);
       getToken();
       if (iTok.iType != PdfToken::EOp || iTok.iString != "endstream")
-	return 0;
+	return nullptr;
       getToken();
-      return dict.take();
+      return dict.release();
     }
 
     // must read name
     if (iTok.iType != PdfToken::EName)
-      return 0;
+      return nullptr;
     String name = iTok.iString.substr(1);
     getToken();
 
@@ -591,13 +635,13 @@ PdfDict *PdfParser::makeDict()
 	  dict->add(name, new PdfRef(toInt(t1.iString)));
 	  getToken();
 	} else
-	  return 0; // should be name or '>>'
+	  return nullptr; // should be name or '>>'
       } else
 	dict->add(name, new PdfNumber(Platform::toDouble(t1.iString)));
     } else {
       PdfObj *obj = getObject();
       if (!obj)
-	return 0;
+	return nullptr;
       dict->add(name, obj);
     }
   }
@@ -614,6 +658,8 @@ PdfObj *PdfParser::getObject()
     return new PdfNumber(Platform::toDouble(tok.iString));
   case PdfToken::EString:
     return new PdfString(tok.iString);
+  case PdfToken::EStringBinary:
+    return new PdfString(tok.iString, true);
   case PdfToken::EName:
     return new PdfName(tok.iString.substr(1));
   case PdfToken::ENull:
@@ -629,7 +675,7 @@ PdfObj *PdfParser::getObject()
     // anything else is an error
   case PdfToken::EErr:
   default:
-    return 0;
+    return nullptr;
   }
 }
 
@@ -638,16 +684,16 @@ PdfObj *PdfParser::getObjectDef()
 {
   getToken();
   if (iTok.iType != PdfToken::ENumber || iTok.iString != "0")
-    return 0;
+    return nullptr;
   getToken();
   if (iTok.iType != PdfToken::EOp || iTok.iString != "obj")
-    return 0;
+    return nullptr;
   getToken();
   PdfObj *obj = getObject();
   if (!obj)
-    return 0;
+    return nullptr;
   if (iTok.iType != PdfToken::EOp || iTok.iString != "endobj")
-    return 0;
+    return nullptr;
   getToken();
   return obj;
 }
@@ -671,7 +717,7 @@ PdfDict *PdfParser::getTrailer()
 {
   getToken();
   if (iTok.iType != PdfToken::EDictBg)
-    return 0;
+    return nullptr;
   getToken();
   return makeDict();
 }
@@ -683,20 +729,36 @@ PdfDict *PdfParser::getTrailer()
  * \brief All information obtained by parsing a PDF file.
  */
 
-//! Create empty container.
-PdfFile::PdfFile()
+bool PdfFile::parseObjectStream(const PdfDict *d)
 {
-  iTrailer = 0;
-}
-
-// Destroy all the objects from the file.
-PdfFile::~PdfFile()
-{
-  delete iTrailer;
-  std::map<int, const PdfObj *>::const_iterator it;
-  for (it = iObjects.begin(); it != iObjects.end(); ++it) {
-    delete it->second;
+  const PdfObj *objn = d->get("N", this);
+  const PdfObj *objfirst = d->get("First", this);
+  int n = objn->number() ? objn->number()->value() : -1;
+  int first = objfirst->number() ? objfirst->number()->value() : -1;
+  if (n < 0 || first < 0)
+    return false;
+  Buffer stream = d->inflate();
+  BufferSource source(stream);
+  PdfParser parser(source);
+  std::vector<int> dir;
+  for (int i = 0; i < 2 * n; ++i) {
+    PdfToken t = parser.token();
+    if (t.iType != PdfToken::ENumber)
+      return false;
+    dir.push_back(toInt(t.iString));
+    parser.getToken();
   }
+  for (int i = 0; i < n; ++i) {
+    int num = dir[2*i];
+    source.setPosition(first + dir[2*i+1]);
+    parser.getChar();
+    parser.getToken();
+    PdfObj *obj = parser.getObject();
+    if (!obj)
+      return false;
+    iObjects[num] = std::unique_ptr<const PdfObj>(obj);
+  }
+  return true;
 }
 
 //! Parse entire PDF stream, and store objects.
@@ -710,15 +772,23 @@ bool PdfFile::parse(DataSource &source)
     if (t.iType == PdfToken::ENumber) {
       // <num> 0 obj starts an object
       int num = toInt(t.iString);
-      PdfObj *obj = parser.getObjectDef();
+      std::unique_ptr<const PdfObj> obj(parser.getObjectDef());
       if (!obj) {
 	ipeDebug("Failed to get object %d", num);
 	return false;
       }
-      iObjects[num] = obj;
+      const PdfDict *d = obj->dict();
+      const PdfObj *type = d ? d->get("Type", this) : nullptr;
+      if (type && type->name() && type->name()->value() == "ObjStm") {
+	if (!parseObjectStream(d))
+	  return false;
+      } else if (type && type->name() && type->name()->value() == "XRef") {
+	iTrailer = std::unique_ptr<const PdfDict>(obj.release()->dict());
+      } else
+	iObjects[num] = std::move(obj);
     } else if (t.iType == PdfToken::EOp) {
       if (t.iString == "trailer") {
-	iTrailer = parser.getTrailer();
+	iTrailer = std::unique_ptr<const PdfDict>(parser.getTrailer());
 	if (!iTrailer) {
 	  ipeDebug("Failed to get trailer");
 	  return false;
@@ -726,6 +796,8 @@ bool PdfFile::parse(DataSource &source)
 	return true;
       } else if (t.iString == "xref") {
 	parser.skipXRef();
+      } else if (t.iString == "startxref") {
+	return true; // this is the end
       } else {
 	ipeDebug("Weird token: %s", t.iString.z());
 	// don't know what's happening
@@ -740,18 +812,29 @@ bool PdfFile::parse(DataSource &source)
 }
 
 //! Return object with number \a num.
-const PdfObj *PdfFile::object(int num) const
+const PdfObj *PdfFile::object(int num) const noexcept
 {
-  std::map<int, const PdfObj *>::const_iterator it =
-    iObjects.find(num);
-  if (it != iObjects.end())
-    return it->second;
+  auto got = iObjects.find(num);
+  if (got != iObjects.end())
+    return got->second.get();
   else
-    return 0;
+    return nullptr;
+}
+
+//! Take ownership of object with number \a num, remove from PdfFile.
+std::unique_ptr<const PdfObj> PdfFile::take(int num)
+{
+  auto got = iObjects.find(num);
+  if (got != iObjects.end()) {
+    std::unique_ptr<const PdfObj> obj = std::move(got->second);
+    iObjects.erase(got);
+    return obj;
+  } else
+    return std::unique_ptr<const PdfObj>();
 }
 
 //! Return root catalog of PDF file.
-const PdfDict *PdfFile::catalog() const
+const PdfDict *PdfFile::catalog() const noexcept
 {
   const PdfObj *root = iTrailer->get("Root", this);
   assert(root && root->dict());
@@ -759,18 +842,18 @@ const PdfDict *PdfFile::catalog() const
 }
 
 //! Return first page of the document.
-const PdfDict *PdfFile::page() const
+const PdfDict *PdfFile::page() const noexcept
 {
   const PdfObj *pages = catalog()->get("Pages", this);
   assert(pages && pages->dict());
   const PdfObj *kids = pages->dict()->get("Kids", this);
   assert(kids);
   if (!kids->array())
-    return 0;
+    return nullptr;
   const PdfObj *page = kids->array()->obj(0, this);
   // this should be page 1
   if (!page || !page->dict())
-    return 0;
+    return nullptr;
   return page->dict();
 }
 
