@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,8 +36,8 @@
 #include "ipeutils.h"
 
 #include "ipepdfwriter.h"
-#include "ipefontpool.h"
 #include "ipepdfparser.h"
+#include "iperesources.h"
 
 using namespace ipe;
 
@@ -214,28 +214,11 @@ void PdfPainter::doDrawText(const Text *text)
   drawOpacity();
 
   iStream << matrix() << " cm " ;
-  iStream << Matrix(xf->iStretch, 0, 0, xf->iStretch, 0, 0) << " cm ";
-#if 0
-  const char *data = xf->iStream.data();
-  const char *fin = data + xf->iStream.size() - 1;
-  // skip whitespace at beginning
-  while (data <= fin && (*data == ' ' || *data == '\n' || *data == '\r'))
-    ++data;
-  // remove whitespace at end
-  while (data <= fin && (*fin == ' ' || *fin == '\n' || *fin == '\r'))
-    --fin;
-  // now output string
-  // replace "0 0 0 0 k 0 0 0 0 K" by color command
-  while (data <= fin) {
-    if (data + 19 <= fin && !::strncmp(data, "0 0 0 0 k 0 0 0 0 K", 19)) {
-      drawColor(iStream, s, "g", "rg");
-      drawColor(iStream, s, "G", "RG");
-      data += 19;
-    } else
-      iStream.putChar(*data++);
-  }
-  iStream << "\n";
-#else
+  if (xf->iStretch != 1.0 || xf->iBBox.bottomLeft() != Vector::ZERO)
+    iStream << Matrix(xf->iStretch, 0, 0, xf->iStretch,
+		      -xf->iBBox.bottomLeft().x,
+		      -xf->iBBox.bottomLeft().y) << " cm ";
+
   BufferSource source(xf->iStream);
   PdfParser parser(source);
   std::vector<const PdfObj *> args;
@@ -296,7 +279,6 @@ void PdfPainter::doDrawText(const Text *text)
     delete args.back();
     args.pop_back();
   }
-#endif
 }
 
 void PdfPainter::doDrawSymbol(Attribute symbol)
@@ -333,17 +315,16 @@ void PdfPainter::doDrawSymbol(Attribute symbol)
 
 //! Create a PDF writer operating on this (open and empty) file.
 PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
-		     const FontPool *pool,
+		     const PdfResources *resources,
 		     bool markedView, int fromPage, int toPage,
 		     int compression)
-  : iStream(stream), iDoc(doc), iMarkedView(markedView),
+  : iStream(stream), iDoc(doc), iResources(resources), iMarkedView(markedView),
     iFromPage(fromPage), iToPage(toPage)
 {
   iCompressLevel = compression;
   iObjNum = 3;  // 0 - 2 are reserved
   iXmlStreamNum = -1; // no XML stream yet
   iResourceNum = -1;
-  iPageNumberFont = -1;
   iExtGState = -1;
   iPatternNum = -1;
   iBookmarks = -1;
@@ -363,19 +344,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
     --id;
   }
 
-  if (iDoc->hasTransparency())
-    iStream << "%PDF-1.4\n";
-  else
-    iStream << "%PDF-1.3\n";
-
-  if (iDoc->properties().iNumberPages) {
-    iPageNumberFont = startObject();
-    iStream << "<<\n"
-	    << "/Type /Font\n"
-	    << "/Subtype /Type1\n"
-	    << "/BaseFont /Helvetica\n"
-	    << ">> endobj\n";
-  }
+  iStream << "%PDF-1.4\n";
 
   // embed all opacities
   AttributeSeq os;
@@ -483,8 +452,8 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
     iStream << ">> endobj\n";
   }
 
-  // embed all fonts
-  embedFonts(pool);
+  // embed all fonts and other resources from Pdflatex
+  embedResources(resources);
 
   // embed all symbols with xform attribute
   AttributeSeq sys;
@@ -536,88 +505,26 @@ int PdfWriter::startObject(int objnum)
   return objnum;
 }
 
-/*! Write all fonts to the PDF file, and fill in their object numbers.
-  Embeds no fonts if \c pool is 0, but must be called nevertheless. */
-void PdfWriter::embedFonts(const FontPool *pool)
+/*! Write all PDF resources to the PDF file, and record their object
+  numbers.  Embeds nothing if \c resources is nullptr, but must be
+  called nevertheless. */
+void PdfWriter::embedResources(const PdfResources *resources)
 {
-  std::map<int, int> fontNumber;
+  std::unordered_map<int, int> resourceNumber;
 
-  // fonts embedded?
-  if (pool) {
-
-    // get fonts that require a cmap
-    std::vector<String> cmapFonts;
-    iDoc->cascade()->allCMaps(cmapFonts);
-
-    for (FontPool::const_iterator font = pool->begin();
-	 font != pool->end(); ++font) {
-      int fontDescriptor = -1;
-      if (!font->iFontDescriptor.empty()) {
-	int streamId = startObject();
-	iStream << "<<\n" << font->iStreamDict;
-	createStream(font->iStreamData.data(),
-		     font->iStreamData.size(), false);
-	fontDescriptor = startObject();
-	iStream << "<<\n" << font->iFontDescriptor
-		<< streamId << " 0 R\n" << ">> endobj\n";
-      }
-      int cmap = -1;
-      int j = font->iName.size() - 2;
-      if (j > 0 && std::find(cmapFonts.begin(), cmapFonts.end(),
-			     font->iName.substr(0, j)) != cmapFonts.end()) {
-	String fpage = font->iName.substr(j);
-	// int page = std::strtol(fpage.z(), 0, 16);
-	String s;
-	StringStream ss(s);
-	ss << "/CIDInit /ProcSet findresource begin\n"
-	   << "12 dict begin\n"
-	   << "begincmap\n"
-	   << "/CIDSystemInfo\n"
-	   << "<< /Registry (Adobe)\n"
-	   << "/Ordering (UCS)\n"
-	   << "/Supplement 0\n"
-	   << ">> def\n"
-	   << "/CMapName /Adobe.Identity.UCS def\n"
-	   << "/CMapType 2 def\n"
-	   << "1 begincodespacerange\n"
-	   << "<00> <FF>\n"
-	   << "endcodespacerange\n"
-	   << "1 beginbfrange\n"
-	   << "<00> <FF> <" << fpage << "00>\n"
-	   << "endbfrange\n"
-	   << "endcmap\n"
-	   << "CMapName currentdict /CMap defineresource pop\n"
-	   << "end\n"
-	   << "end\n";
-	cmap = startObject();
-	iStream << "<<\n";
-	createStream(s.data(), s.size(), false);
-      }
-      int objectNumber = startObject();
-      fontNumber[font->iLatexNumber] = objectNumber;
-      iStream << "<<\n" << font->iFontDict;
-      if (fontDescriptor >= 0)
-	iStream << "/FontDescriptor " << fontDescriptor << " 0 R\n";
-      if (cmap >= 0)
-	iStream << "/ToUnicode " << cmap << " 0 R\n";
-      iStream << ">> endobj\n";
+  if (resources) {
+    const std::vector<int> &seq = resources->embedSequence();
+    for (auto &num : seq) {
+      const PdfObj *obj = resources->object(num);
+      int embedNum = startObject();
+      obj->write(iStream, &resourceNumber);
+      iStream << " endobj\n";
+      resourceNumber[num] = embedNum;
     }
-  }
-
-  if (pool || iPageNumberFont >= 0) {
     iResourceNum = startObject();
-    iStream << "<< ";
-    if (pool) {
-      for (FontPool::const_iterator font = pool->begin();
-	   font != pool->end(); ++font) {
-	iStream << "/F" << font->iLatexNumber << " "
-		<< fontNumber[font->iLatexNumber] << " 0 R ";
-      }
-    }
-    if (iPageNumberFont >= 0)
-      iStream << "/F" << iPageNumberFont << " "
-	      << iPageNumberFont << " 0 R ";
-    iStream << ">> endobj\n";
+    const PdfDict *font = resources->get("Font");
+    font->write(iStream,  &resourceNumber);
+    iStream << " endobj\n";
   }
 }
 
@@ -773,18 +680,10 @@ void PdfWriter::paintView(Stream &stream, int pno, int view)
   if (background && page->findLayer("BACKGROUND") < 0)
     painter.drawSymbol(Attribute::BACKGROUND());
 
-  // page numbers
-  if (iPageNumberFont >= 0) {
-    const StyleSheet::PageNumberStyle *pns =
-      iDoc->cascade()->findPageNumberStyle();
-    stream << "q ";
-    PdfPainter::drawColor(stream, pns->iColor, "g", "rg");
-    stream << "BT /F" << iPageNumberFont << " " << pns->iFontSize
-	   << " Tf " << pns->iPos << " Td\n"
-	   << "[(" << (pno + 1);
-    if (page->countMarkedViews() > 1)
-      stream << "-" << (view + 1);
-    stream << ")] TJ ET Q\n";
+  if (iDoc->properties().iNumberPages) {
+    const Text *pn = iResources->pageNumber(pno, view);
+    if (pn)
+      pn->draw(painter);
   }
 
   const Text *title = page->titleText();
@@ -820,12 +719,34 @@ void PdfWriter::createPageView(int pno, int view)
   } else
     paintView(sstream, pno, view);
 
+  int firstLink = -1;
+  int lastLink = -1;
+  for (int i = 0; i < page->count(); ++i) {
+    const Group *g = page->object(i)->asGroup();
+    if (g && page->objectVisible(view, i) && !g->url().empty()) {
+      lastLink = startObject();
+      if (firstLink < 0) firstLink = lastLink;
+      iStream << "<<\n"
+	      << "/Type /Annot\n"
+	      << "/Subtype /Link\n"
+	      << "/Rect [" << page->bbox(i) << "]\n"
+	      << "/A <</Type/Action/S/URI/URI";
+      writeString(g->url());
+      iStream << ">>\n>> endobj\n";
+    }
+  }
   int contentsobj = startObject();
   iStream << "<<\n";
   createStream(pagedata.data(), pagedata.size(), (iCompressLevel > 0));
   int pageobj = startObject();
   iStream << "<<\n";
   iStream << "/Type /Page\n";
+  if (firstLink >= 0) {
+    iStream << "/Annots [ ";
+    while (firstLink <= lastLink)
+      iStream << firstLink++ << " 0 R ";
+    iStream << "]\n";
+  }
   iStream << "/Contents " << contentsobj << " 0 R\n";
   // iStream << "/Rotate 0\n";
   createResources(bm);
