@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -75,7 +75,7 @@ int Platform::libVersion()
 
 static bool initialized = false;
 static bool showDebug = false;
-static Platform::DebugHandler debugHandler = 0;
+static Platform::DebugHandler debugHandler = nullptr;
 
 #ifdef WIN32
 static ULONG_PTR gdiplusToken = 0;
@@ -86,12 +86,21 @@ locale_t ipeLocale;
 #endif
 
 #ifdef WIN32
-// Counterpart to Platform::wideToUtf8
-void Platform::utf8ToWide(const char *utf8, std::vector<wchar_t> &wbuf)
+static void readIpeConf()
 {
-  int rw = MultiByteToWideChar(CP_UTF8, 0, utf8, -1, NULL, 0);
-  wbuf.resize(rw);
-  MultiByteToWideChar(CP_UTF8, 0, utf8, -1, &wbuf[0], rw);
+  String fname = Platform::ipeDir("", nullptr);
+  fname += "\\ipe.conf";
+  String conf = Platform::readFile(fname);
+  if (conf.empty())
+    return;
+  ipeDebug("ipe.conf = %s", conf.z());
+  int i = 0;
+  while (i < conf.size()) {
+    String line = conf.getLine(i);
+    std::vector<wchar_t> wline;
+    Platform::utf8ToWide(line.z(), wline);
+    _wputenv(&wline[0]);
+  }
 }
 #endif
 
@@ -139,6 +148,7 @@ void Platform::initLib(int version)
   }
   debugHandler = debugHandlerImpl;
 #ifdef WIN32
+  readIpeConf();
   // ipeLocale = ::_create_locale(LC_NUMERIC, "C");
   Gdiplus::GdiplusStartupInput gdiplusStartupInput;
   Gdiplus::GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, 0);
@@ -167,7 +177,7 @@ void Platform::setDebug(bool debug)
 
 // --------------------------------------------------------------------
 
-void ipeDebug(const char *msg, ...)
+void ipeDebug(const char *msg, ...) noexcept
 {
   if (debugHandler) {
     char buf[8196];
@@ -186,10 +196,8 @@ void ipeDebug(const char *msg, ...)
 String Platform::currentDirectory()
 {
 #ifdef WIN32
-  wchar_t *buffer;
-  if ((buffer = _wgetcwd(0, 0)) == 0)
-    return String();
-  return Platform::wideToUtf8(buffer);
+  wchar_t *buffer = _wgetcwd(0, 0);
+  return String(buffer);
 #else
   char buffer[1024];
   if (getcwd(buffer, 1024) != buffer)
@@ -204,7 +212,7 @@ String Platform::ipeDir(const char *suffix, const char *fname)
 #ifdef WIN32
   wchar_t exename[OFS_MAXPATHNAME];
   GetModuleFileNameW(0, exename, OFS_MAXPATHNAME);
-  String exe = Platform::wideToUtf8(exename);
+  String exe(exename);
 #else
   char path[PATH_MAX];
   uint32_t size = sizeof(path);
@@ -254,6 +262,23 @@ static String dotIpe()
 }
 #endif
 
+//! Return path for the directory containing pdflatex and xelatex.
+/*! If empty means look on PATH. */
+String Platform::latexPath()
+{
+  String result;
+#ifdef WIN32
+  const wchar_t *p = _wgetenv(L"IPELATEXPATH");
+  if (p)
+    result = String(p);
+#else
+  char *p = getenv("IPELATEXPATH");
+  if (p)
+    result = p;
+#endif
+  return result;
+}
+
 //! Returns directory for running Latex.
 /*! The directory is created if it does not exist.  Returns an empty
   string if the directory cannot be found or cannot be created.
@@ -264,19 +289,17 @@ String Platform::latexDirectory()
 #ifdef WIN32
   String latexDir;
   const wchar_t *p = _wgetenv(L"IPELATEXDIR");
-  if (getenv("IPEWINE"))
-    latexDir = "z:\\tmp\\ipelatex";
-  else if (p) {
-    latexDir = Platform::wideToUtf8(p);
-  } else {
+  if (p)
+    latexDir = String(p);
+  else {
     wchar_t szPath[MAX_PATH];
     if (SUCCEEDED(SHGetFolderPathW(NULL, CSIDL_LOCAL_APPDATA,
 				   NULL, 0, szPath))) {
-      latexDir = Platform::wideToUtf8(szPath) + "\\ipe";
+      latexDir = String(szPath) + "\\ipe";
     } else {
       p = _wgetenv(L"LOCALAPPDATA");
       if (p)
-	latexDir = Platform::wideToUtf8(p) + "\\ipe";
+	latexDir = String(p) + "\\ipe";
       else
 	latexDir = ipeDir("latexrun");
     }
@@ -306,19 +329,6 @@ String Platform::latexDirectory()
 #endif
 }
 
-//! Returns filename of fontmap.
-String Platform::fontmapFile()
-{
-  const char *p = getenv("IPEFONTMAP");
-  if (p)
-    return String(p);
-#ifdef IPEBUNDLE
-  return ipeDir("data", "fontmap.xml");
-#else
-  return IPEFONTMAP;
-#endif
-}
-
 //! Determine whether file exists.
 bool Platform::fileExists(String fname)
 {
@@ -343,9 +353,9 @@ bool Platform::listDirectory(String path, std::vector<String> &files)
   intptr_t h = _wfindfirst(&wname[0], &info);
   if (h == -1L)
     return false;
-  files.push_back(wideToUtf8(info.name));
+  files.push_back(String(info.name));
   while (_wfindnext(h, &info) == 0)
-    files.push_back(wideToUtf8(info.name));
+    files.push_back(String(info.name));
   _findclose(h);
   return true;
 #else
@@ -380,68 +390,96 @@ String Platform::readFile(String fname)
   return s;
 }
 
-//! Runs pdflatex on file text.tex in given directory.
-int Platform::runPdfLatex(String dir)
+//! Runs latex on file text.tex in given directory.
+int Platform::runLatex(String dir, LatexType engine) noexcept
 {
+  const char *latex = (engine == LatexType::Xetex) ?
+    "xelatex" : (engine == LatexType::Luatex) ?
+    "lualatex" : "pdflatex";
 #ifdef WIN32
-  const char *wine = getenv("IPEWINE");
-  if (wine) {
-    int secs = Lex(wine).getInt();
-    String s = "start /unix /home/otfried/Lib/Bin/ipelatexhelper";
-    std::system(s.z());
-    fprintf(stderr, "Waiting %d seconds for pdflatex\n", secs);
-    Sleep(secs * 1000);
-    return 0;
-  } else {
-    String s = dir + "runlatex.bat";
-    std::FILE *f = Platform::fopen(s.z(), "wb");
-    if (!f)
-      return -1;
-
-    if (dir.size() > 2 && dir[1] == ':')
-      fprintf(f, "%s\r\n", dir.substr(0, 2).z());
-
-    // CMD.EXE input needs to be encoded in "OEM codepage",
-    // which can be different from "Windows codepage"
-    std::vector<wchar_t> wbuf;
-    utf8ToWide(dir.z(), wbuf);
-    Buffer oemDir(2 * wbuf.size() + 1);
-    CharToOemW(&wbuf[0], oemDir.data());
-
-    fprintf(f, "cd \"%s\"\r\n", oemDir.data());
-    fprintf(f, "pdflatex ipetemp.tex\r\n");
-    std::fclose(f);
-
-    // Declare and initialize process blocks
-    PROCESS_INFORMATION processInformation;
-    STARTUPINFOW startupInfo;
-
-    memset(&processInformation, 0, sizeof(processInformation));
-    memset(&startupInfo, 0, sizeof(startupInfo));
-    startupInfo.cb = sizeof(startupInfo);
-
-    // Call the executable program
-    s = String("cmd /c call \"") + dir + String("runlatex.bat\"");
-    utf8ToWide(s.z(), wbuf);
-
-    int result = CreateProcessW(NULL, &wbuf[0], NULL, NULL, FALSE,
-				NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
-				NULL, NULL, &startupInfo, &processInformation);
-    if (result == 0)
-      return -1;  // failure to create process
-
-    // Wait until child process exits.
-    WaitForSingleObject(processInformation.hProcess, INFINITE);
-
-    // Close process and thread handles.
-    CloseHandle(processInformation.hProcess);
-    CloseHandle(processInformation.hThread);
-    return 0;
+  if (getenv("IPETEXFORMAT")) {
+    latex = (engine == LatexType::Xetex) ?
+      "xetex ^&latex" : (engine == LatexType::Luatex) ?
+      "luatex ^&latex" : "pdftex ^&pdflatex";
   }
+
+  String bat;
+  if (dir.size() > 2 && dir[1] == ':') {
+    bat += dir.substr(0, 2);
+    bat += "\r\n";
+  }
+  bat += "cd \"";
+  bat += dir;
+  bat += "\"\r\n";
+  String path = latexPath();
+  if (!path.empty()) {
+    bat += "PATH ";
+    bat += path;
+    bat += ";%PATH%\r\n";
+  }
+  bat += latex;
+  bat += " ipetemp.tex\r\n";
+  // bat += "pause\r\n";  // alternative for Wine
+
+  // CMD.EXE input needs to be encoded in "OEM codepage",
+  // which can be different from "Windows codepage"
+  std::vector<wchar_t> wbat;
+  utf8ToWide(bat.z(), wbat);
+  Buffer oemBat(2 * wbat.size() + 1);
+  CharToOemW(&wbat[0], oemBat.data());
+
+  String s = dir + "runlatex.bat";
+  std::FILE *f = Platform::fopen(s.z(), "wb");
+  if (!f)
+    return -1;
+  std::fputs(oemBat.data(), f);
+  std::fclose(f);
+
+  // Declare and initialize process blocks
+  PROCESS_INFORMATION processInformation;
+  STARTUPINFOW startupInfo;
+
+  memset(&processInformation, 0, sizeof(processInformation));
+  memset(&startupInfo, 0, sizeof(startupInfo));
+  startupInfo.cb = sizeof(startupInfo);
+
+  // Call the executable program
+  s = String("cmd /c call \"") + dir + String("runlatex.bat\"");
+  utf8ToWide(s.z(), wbat);
+
+  int result = CreateProcessW(NULL, &wbat[0], NULL, NULL, FALSE,
+			      NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
+			      NULL, NULL, &startupInfo, &processInformation);
+  if (result == 0)
+    return -1;  // failure to create process
+
+  // Wait until child process exits.
+  WaitForSingleObject(processInformation.hProcess, INFINITE);
+
+  // Close process and thread handles.
+  CloseHandle(processInformation.hProcess);
+  CloseHandle(processInformation.hThread);
+
+  // Apparently WaitForSingleObject doesn't work in Wine
+  const char *wine = getenv("IPEWINE");
+  if (wine)
+    Sleep(Lex(wine).getInt());
+  return 0;
 #else
-  String s("cd ");
+  if (getenv("IPETEXFORMAT")) {
+    latex = (engine == LatexType::Xetex) ?
+      "xetex \\&latex" : (engine == LatexType::Luatex) ?
+      "luatex \\&latex" : "pdftex \\&pdflatex";
+  }
+  String s("cd \"");
   s += dir;
-  s += "; rm -f ipetemp.log; pdflatex ipetemp.tex > /dev/null";
+  s += "\"; rm -f ipetemp.log; ";
+  String path = latexPath();
+  if (path.empty())
+    s += latex;
+  else
+    s += String("\"") + path + "/" + latex + "\"";
+  s += " ipetemp.tex > /dev/null";
   int result = std::system(s.z());
   if (result != -1)
     result = WEXITSTATUS(result);
@@ -466,61 +504,27 @@ int Platform::mkdir(const char *dname)
   return _wmkdir(&wname[0]);
 }
 
-struct Codepage {
-  UINT cp;
-  const char *name;
-};
-
-static const Codepage codepages[] = {
-  { 28591, "iso-8859-1" },
-  { 51949, "euc-kr" },
-  { 51932, "euc-jp" },
-  { 28592, "iso-8859-2" },
-  { 21866, "koi8-u" },
-  { 20866, "koi8-r" },
-  { 932, "shift_jis" },
-  { 0, 0 }
-};
-
-// See http://msdn.microsoft.com/en-us/library/
-//  windows/desktop/dd317756%28v=vs.85%29.aspx
-static UINT codepage(const String &encoding)
+String::String(const wchar_t *wbuf)
 {
-  const Codepage *cp = codepages;
-  while (cp->name) {
-    if (encoding == cp->name)
-      return cp->cp;
-    ++cp;
+  if (!wbuf) {
+    iImp = emptyString();
+  } else {
+    int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, NULL, 0, NULL, NULL);
+    iImp = new Imp;
+    iImp->iRefCount = 1;
+    iImp->iSize = rm - 1; // rm includes the trailing zero
+    iImp->iCapacity = (rm + 32) & ~15;
+    iImp->iData = new char[iImp->iCapacity];
+    WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, iImp->iData, rm, NULL, NULL);
   }
-  String s("You requested the character encoding '");
-  s += encoding;
-  s += "' for Latex conversion.\n"
-    "There is no mapping for this encoding in Ipe.\n"
-    "This may be a bug, please report it.";
-  MessageBoxA(NULL, s.z(), "Error", MB_ICONEXCLAMATION | MB_OK);
-  return CP_UTF8;
 }
 
-String Platform::winConv(const String &utf8, const String &encoding)
+void Platform::utf8ToWide(const char *utf8, std::vector<wchar_t> &wbuf)
 {
-  UINT cp = codepage(encoding);
-  int rw = MultiByteToWideChar(CP_UTF8, 0, utf8.z(), -1, NULL, 0);
-  wchar_t wbuf[rw];
-  MultiByteToWideChar(CP_UTF8, 0, utf8.z(), -1, wbuf, rw);
-  int rm = WideCharToMultiByte(cp, 0, wbuf, -1, NULL, 0, NULL, NULL);
-  char multi[rm];
-  WideCharToMultiByte(cp, 0, wbuf, -1, multi, rm, NULL, NULL);
-  return String(multi);
+  int rw = MultiByteToWideChar(CP_UTF8, 0, utf8, -1, NULL, 0);
+  wbuf.resize(rw);
+  MultiByteToWideChar(CP_UTF8, 0, utf8, -1, &wbuf[0], rw);
 }
-
-String Platform::wideToUtf8(const wchar_t *wbuf)
-{
-  int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, NULL, 0, NULL, NULL);
-  char multi[rm];
-  WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, multi, rm, NULL, NULL);
-  return String(multi);
-}
-
 #endif
 
 // --------------------------------------------------------------------
