@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ static const char *styleStandard[] = {
   "<layout paper=\"595 842\" origin=\"0 0\" frame=\"595 842\"/>",
   "<titlestyle pos=\"0 800\" size=\"large\" color=\"black\" ",
   "halign=\"left\" valign=\"baseline\"/>\n",
-  "<pagenumberstyle pos=\"10 10\" size=\"10\" color=\"black\"/>",
+  "<pagenumberstyle pos=\"10 10\" size=\"normal\" color=\"black\"/>",
   "<pathstyle cap=\"0\" join=\"1\" fillrule=\"eofill\"/>",
   "<textstyle name=\"normal\" begin=\"\" end=\"\"/>",
   "<opacity name=\"opaque\" value=\"1\"/>",
