@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -181,7 +181,7 @@ const StyleSheet::PageNumberStyle *StyleSheet::pageNumberStyle()
   if (iPageNumberStyle.iDefined)
     return &iPageNumberStyle;
   else
-    return 0;
+    return nullptr;
 }
 
 //! Add gradient to this style sheet.
@@ -261,19 +261,6 @@ void StyleSheet::setFillRule(TFillRule s)
 
 // --------------------------------------------------------------------
 
-//! Add font \a s to list of fonts with cmap generation.
-void StyleSheet::addCMap(String s)
-{
-  iCMaps.push_back(s);
-}
-
-void StyleSheet::allCMaps(std::vector<String> &seq) const
-{
-  std::copy(iCMaps.begin(), iCMaps.end(), std::back_inserter(seq));
-}
-
-// --------------------------------------------------------------------
-
 //! Add a symbol object.
 void StyleSheet::addSymbol(Attribute name, const Symbol &symbol)
 {
@@ -481,13 +468,7 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
       stream << " value=\"" << it->second.string() << "\"/>\n";
   }
   if (!iPreamble.empty()) {
-    stream << "<preamble";
-    if (!iEncoding.empty()) {
-      stream << " encoding=\"";
-      stream.putXmlString(iEncoding);
-      stream << "\"";
-    }
-    stream << ">";
+    stream << "<preamble>";
     stream.putXmlString(iPreamble);
     stream << "</preamble>\n";
   }
@@ -510,10 +491,13 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
 
   }
   if (iPageNumberStyle.iDefined) {
-    stream << "<pagenumberstyle pos=\"" << iPageNumberStyle.iPos
-	   << "\" size=\"" << iPageNumberStyle.iFontSize
-	   << "\" color=\"" << iPageNumberStyle.iColor
-	   << "\"/>\n";
+    stream << "<pagenumberstyle pos=\"" << iPageNumberStyle.iPos << "\""
+	   << " color=\"" << iPageNumberStyle.iColor.string() << "\""
+	   << " size=\"" << iPageNumberStyle.iSize.string() << "\"";
+    Text::saveAlignment(stream,
+			iPageNumberStyle.iHorizontalAlignment,
+			iPageNumberStyle.iVerticalAlignment);
+    stream << ">" << iPageNumberStyle.iText << "</pagenumberstyle>\n";
   }
   if (iTitleStyle.iDefined) {
     stream << "<titlestyle pos=\"" << iTitleStyle.iPos
@@ -775,16 +759,6 @@ String Cascade::findPreamble() const
   return s;
 }
 
-//! Return LaTeX encoding (of the whole cascade).
-String Cascade::findEncoding() const
-{
-  for (int i = 0; i < count(); ++i) {
-    if (!iSheets[i]->encoding().isEmpty())
-      return iSheets[i]->encoding();
-  }
-  return String();
-}
-
 TLineCap Cascade::lineCap() const
 {
   for (int i = 0; i < count(); ++i) {
@@ -839,11 +813,4 @@ int Cascade::findDefinition(Kind kind, Attribute sym) const
   return -1;
 }
 
-void Cascade::allCMaps(std::vector<String> &seq) const
-{
-  for (int i = 0; i < count(); ++i) {
-    iSheets[i]->allCMaps(seq);
-  }
-}
-
 // --------------------------------------------------------------------
