@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -236,16 +236,16 @@ void PDialog::setMapped(lua_State *L, int idx)
   QWidget *w = iWidgets[idx];
   switch (m.type) {
   case ELabel:
-    (qobject_cast<QLabel *>(w))->setText(QString::fromUtf8(m.text.z()));
+    (qobject_cast<QLabel *>(w))->setText(QString::fromUtf8(m.text.c_str()));
     break;
   case ECheckBox:
     (qobject_cast<QCheckBox *>(w))->setChecked(m.value);
     break;
   case ETextEdit:
-    (qobject_cast<QTextEdit *>(w))->setText(QString::fromUtf8(m.text.z()));
+    (qobject_cast<QTextEdit *>(w))->setText(QString::fromUtf8(m.text.c_str()));
     break;
   case EInput:
-    (qobject_cast<QLineEdit *>(w))->setText(QString::fromUtf8(m.text.z()));
+    (qobject_cast<QLineEdit *>(w))->setText(QString::fromUtf8(m.text.c_str()));
     break;
   case EList:
     {
@@ -253,7 +253,7 @@ void PDialog::setMapped(lua_State *L, int idx)
       if (!lua_isnumber(L, 3)) {
 	l->clear();
 	for (int k = 0; k < int(m.items.size()); ++k)
-	  l->addItem(QString::fromUtf8(m.items[k].z()));
+	  l->addItem(QString::fromUtf8(m.items[k].c_str()));
       }
       l->setCurrentRow(m.value);
     }
@@ -264,7 +264,7 @@ void PDialog::setMapped(lua_State *L, int idx)
       if (!lua_isnumber(L, 3)) {
 	b->clear();
 	for (int k = 0; k < int(m.items.size()); ++k)
-	  b->addItem(QString::fromUtf8(m.items[k].z()));
+	  b->addItem(QString::fromUtf8(m.items[k].c_str()));
       }
       b->setCurrentIndex(m.value);
     }
@@ -279,7 +279,7 @@ bool PDialog::buildAndRun(int w, int h)
   for (int i = 0; i < int(iElements.size()); ++i) {
     SElement &m = iElements[i];
     if (m.row < 0) {
-      QPushButton *b = new QPushButton(QString::fromUtf8(m.text.z()), this);
+      QPushButton *b = new QPushButton(QString::fromUtf8(m.text.c_str()), this);
       iWidgets.push_back(b);
       if (m.flags & EAccept) {
 	b->setDefault(true);
@@ -293,11 +293,11 @@ bool PDialog::buildAndRun(int w, int h)
       QWidget *w = 0;
       switch (m.type) {
       case ELabel:
-	w = new QLabel(QString::fromUtf8(m.text.z()), this);
+	w = new QLabel(QString::fromUtf8(m.text.c_str()), this);
 	break;
       case EButton:
 	{
-	  QPushButton *b = new QPushButton(QString::fromUtf8(m.text.z()), this);
+	  QPushButton *b = new QPushButton(QString::fromUtf8(m.text.c_str()), this);
 	  if (m.flags & EAccept)
 	    connect(b, SIGNAL(clicked()), SLOT(accept()));
 	  else if (m.flags & EReject)
@@ -309,7 +309,7 @@ bool PDialog::buildAndRun(int w, int h)
 	break;
       case ECheckBox:
 	{
-	  QCheckBox *ch = new QCheckBox(QString::fromUtf8(m.text.z()), this);
+	  QCheckBox *ch = new QCheckBox(QString::fromUtf8(m.text.c_str()), this);
 	  if (m.lua_method != LUA_NOREF)
 	    connect(ch, SIGNAL(stateChanged(int)), SLOT(notifyLua()));
 	  ch->setChecked(m.value);
@@ -319,7 +319,7 @@ bool PDialog::buildAndRun(int w, int h)
       case EInput:
 	{
 	  QLineEdit *e = new QLineEdit(this);
-	  e->setText(QString::fromUtf8(m.text.z()));
+	  e->setText(QString::fromUtf8(m.text.c_str()));
 	  if (m.flags & ESelectAll)
 	    e->selectAll();
 	  w = e;
@@ -335,7 +335,7 @@ bool PDialog::buildAndRun(int w, int h)
 	    (void) new XmlHighlighter(t);
 	  else if (m.flags & ELatex)
 	    (void) new LatexHighlighter(t);
-	  QString text = QString::fromUtf8(m.text.z());
+	  QString text = QString::fromUtf8(m.text.c_str());
 	  if (m.flags & ELogFile) {
 	    markupLog(t, text);
 	  } else
@@ -349,7 +349,7 @@ bool PDialog::buildAndRun(int w, int h)
 	{
 	  QComboBox *b = new QComboBox(this);
 	  for (int k = 0; k < int(m.items.size()); ++k)
-	    b->addItem(QString::fromUtf8(m.items[k].z()));
+	    b->addItem(QString::fromUtf8(m.items[k].c_str()));
 	  b->setCurrentIndex(m.value);
 	  w = b;
 	}
@@ -358,7 +358,7 @@ bool PDialog::buildAndRun(int w, int h)
 	{
 	  QListWidget *l = new QListWidget(this);
 	  for (int k = 0; k < int(m.items.size()); ++k)
-	    l->addItem(QString::fromUtf8(m.items[k].z()));
+	    l->addItem(QString::fromUtf8(m.items[k].c_str()));
 	  w = l;
 	}
 	break;
@@ -384,11 +384,12 @@ void PDialog::retrieveValues()
     QWidget *w = iWidgets[i];
     switch (m.type) {
     case EInput:
-      m.text =
-	String((qobject_cast<QLineEdit *>(w))->text().toUtf8());
+      m.text = std::move(std::string((qobject_cast<QLineEdit *>(w))->
+				     text().toUtf8()));
       break;
     case ETextEdit:
-      m.text = String((qobject_cast<QTextEdit *>(w))->toPlainText().toUtf8());
+      m.text = std::move(std::string((qobject_cast<QTextEdit *>(w))->
+				     toPlainText().toUtf8()));
       break;
     case EList:
       {
